@@ -258,8 +258,14 @@ void Audio_SetOutputAudioFreq(int nNewFrequency)
 		}
 	}
 
-	/* Apply YM2149 C10 filter? */
-	UseLowPassFilter = Config_IsMachineST() && nAudioFrequency >= 40000;
+	/* Apply YM2149 C10 low pass filter ? (except if forced to NONE) */
+	if ( YM2149_LPF_Filter != YM2149_LPF_FILTER_NONE )
+	{
+		if ( Config_IsMachineST() && nAudioFrequency >= 40000 )
+			YM2149_LPF_Filter = YM2149_LPF_FILTER_LPF_STF;
+		else
+			YM2149_LPF_Filter = YM2149_LPF_FILTER_PWM;
+	}
 }
 
 
