@@ -28,6 +28,7 @@ const char Cart_fileid[] = "Hatari cart.c : " __DATE__ " " __TIME__;
 #include "log.h"
 #include "m68000.h"
 #include "stMemory.h"
+#include "inffile.h"
 #include "tos.h"
 #include "vdi.h"
 #include "hatari-glue.h"
@@ -116,11 +117,13 @@ void Cart_ResetImage(void)
 	}
 
 	/* Use internal cartridge trampoline code when user wants extended VDI
-	 * resolution, GEMDOS HD emulation or to trace GEMDOS, VDI or AES.
-	 * (OS_BASE does subset of GEMDOS tracing)
-	 * But don't use it on TOS 0.00, it does not work there. */
+	 * resolution, use GEMDOS HD emulation / Autostarting, or to trace GEMDOS,
+	 * VDI or AES (OS_BASE does subset of GEMDOS tracing).
+	 * But don't use it on TOS 0.00, it does not work there.
+	 */
 	PatchIllegal = false;				/* By default, don't patch opcodes */
-	if ((bUseVDIRes || ConfigureParams.HardDisk.bUseHardDiskDirectories ||
+	if ((bUseVDIRes || INF_Overriding(AUTOSTART_INTERCEPT) ||
+	     ConfigureParams.HardDisk.bUseHardDiskDirectories ||
 	    LogTraceFlags & (TRACE_OS_GEMDOS | TRACE_OS_BASE | TRACE_OS_VDI | TRACE_OS_AES))
 	    && TosVersion >= 0x100)
 	{
