@@ -148,12 +148,12 @@ bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 		return true;
 
 	/* Did change size of TT-RAM? */
-	if (current->Memory.nTTRamSize != changed->Memory.nTTRamSize)
+	if (current->Memory.TTRamSize_KB != changed->Memory.TTRamSize_KB)
 		return true;
 #endif
 
 	/* Did change size of memory? */
-	if (current->Memory.nMemorySize != changed->Memory.nMemorySize)
+	if (current->Memory.STRamSize_KB != changed->Memory.STRamSize_KB)
 		return true;
 
 	/* MIDI related IRQs start/stop needs reset */
@@ -200,6 +200,7 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	     || changed->Screen.bShowStatusbar != current->Screen.bShowStatusbar
 #if WITH_SDL2
 	     || changed->Screen.nRenderScaleQuality != current->Screen.nRenderScaleQuality
+	     || changed->Screen.bResizable != current->Screen.bResizable
 	     || changed->Screen.bUseVsync != current->Screen.bUseVsync
 #endif
 	    ))
@@ -331,9 +332,14 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 
 	/* Did change MIDI settings? */
 	if (current->Midi.bEnableMidi != changed->Midi.bEnableMidi
-	    || ((strcmp(changed->Midi.sMidiOutFileName, current->Midi.sMidiOutFileName)
-	         || strcmp(changed->Midi.sMidiInFileName, current->Midi.sMidiInFileName))
-	        && changed->Midi.bEnableMidi))
+#ifdef HAVE_PORTMIDI
+	    || strcmp(changed->Midi.sMidiOutPortName, current->Midi.sMidiOutPortName)
+	    || strcmp(changed->Midi.sMidiInPortName, current->Midi.sMidiInPortName)
+#else
+	    || strcmp(changed->Midi.sMidiOutFileName, current->Midi.sMidiOutFileName)
+	    || strcmp(changed->Midi.sMidiInFileName, current->Midi.sMidiInFileName)
+#endif
+	    )
 	{
 		Dprintf("- midi>\n");
 		Midi_UnInit();
