@@ -34,6 +34,7 @@ const char Configuration_fileid[] = "Hatari configuration.c : " __DATE__ " " __T
 #include "dsp.h"
 #include "joy.h"
 #include "falcon/crossbar.h"
+#include "stMemory.h"
 
 
 CNF_PARAMS ConfigureParams;                 /* List of configuration for the emulator */
@@ -45,7 +46,6 @@ static const struct Config_Tag configs_Log[] =
 {
 	{ "sLogFileName", String_Tag, ConfigureParams.Log.sLogFileName },
 	{ "sTraceFileName", String_Tag, ConfigureParams.Log.sTraceFileName },
-	{ "nExceptionDebugMask", Int_Tag, &ConfigureParams.Log.nExceptionDebugMask },
 	{ "nTextLogLevel", Int_Tag, &ConfigureParams.Log.nTextLogLevel },
 	{ "nAlertDlgLogLevel", Int_Tag, &ConfigureParams.Log.nAlertDlgLogLevel },
 	{ "bConfirmQuit", Bool_Tag, &ConfigureParams.Log.bConfirmQuit },
@@ -58,10 +58,14 @@ static const struct Config_Tag configs_Log[] =
 static const struct Config_Tag configs_Debugger[] =
 {
 	{ "nNumberBase", Int_Tag, &ConfigureParams.Debugger.nNumberBase },
-	{ "nDisasmLines", Int_Tag, &ConfigureParams.Debugger.nDisasmLines },
+	{ "nSymbolLines", Int_Tag, &ConfigureParams.Debugger.nSymbolLines },
 	{ "nMemdumpLines", Int_Tag, &ConfigureParams.Debugger.nMemdumpLines },
+	{ "nDisasmLines", Int_Tag, &ConfigureParams.Debugger.nDisasmLines },
+	{ "nExceptionDebugMask", Int_Tag, &ConfigureParams.Debugger.nExceptionDebugMask },
 	{ "nDisasmOptions", Int_Tag, &ConfigureParams.Debugger.nDisasmOptions },
 	{ "bDisasmUAE", Bool_Tag, &ConfigureParams.Debugger.bDisasmUAE },
+	{ "bSymbolsResident", Bool_Tag, &ConfigureParams.Debugger.bSymbolsResident },
+	{ "bMatchAllSymbols", Bool_Tag, &ConfigureParams.Debugger.bMatchAllSymbols },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -72,7 +76,9 @@ static const struct Config_Tag configs_Screen[] =
 	{ "nFrameSkips", Int_Tag, &ConfigureParams.Screen.nFrameSkips },
 	{ "bFullScreen", Bool_Tag, &ConfigureParams.Screen.bFullScreen },
 	{ "bKeepResolution", Bool_Tag, &ConfigureParams.Screen.bKeepResolution },
-#if !WITH_SDL2
+#if WITH_SDL2
+	{ "bResizable", Bool_Tag, &ConfigureParams.Screen.bResizable },
+#else
 	{ "bKeepResolutionST", Bool_Tag, &ConfigureParams.Screen.bKeepResolutionST },
 #endif
 	{ "bAllowOverscan", Bool_Tag, &ConfigureParams.Screen.bAllowOverscan },
@@ -91,8 +97,9 @@ static const struct Config_Tag configs_Screen[] =
 	{ "nMaxWidth", Int_Tag, &ConfigureParams.Screen.nMaxWidth },
 	{ "nMaxHeight", Int_Tag, &ConfigureParams.Screen.nMaxHeight },
 #if WITH_SDL2
+	{ "bUseSdlRenderer", Bool_Tag, &ConfigureParams.Screen.bUseSdlRenderer },
 	{ "nRenderScaleQuality", Int_Tag, &ConfigureParams.Screen.nRenderScaleQuality },
-	{ "bUseVsync", Int_Tag, &ConfigureParams.Screen.bUseVsync },
+	{ "bUseVsync", Bool_Tag, &ConfigureParams.Screen.bUseVsync },
 #endif
 	{ NULL , Error_Tag, NULL }
 };
@@ -325,6 +332,7 @@ static const struct Config_Tag configs_ShortCutWithMod[] =
 {
 	{ "kOptions",    Key_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_OPTIONS] },
 	{ "kFullScreen", Key_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_FULLSCREEN] },
+	{ "kBorders",    Key_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_BORDERS] },
 	{ "kMouseMode",  Key_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_MOUSEGRAB] },
 	{ "kColdReset",  Key_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_COLDRESET] },
 	{ "kWarmReset",  Key_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_WARMRESET] },
@@ -353,6 +361,7 @@ static const struct Config_Tag configs_ShortCutWithoutMod[] =
 {
 	{ "kOptions",    Key_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_OPTIONS] },
 	{ "kFullScreen", Key_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_FULLSCREEN] },
+	{ "kBorders",    Key_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_BORDERS] },
 	{ "kMouseMode",  Key_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_MOUSEGRAB] },
 	{ "kColdReset",  Key_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_COLDRESET] },
 	{ "kWarmReset",  Key_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_WARMRESET] },
@@ -393,8 +402,8 @@ static const struct Config_Tag configs_Sound[] =
 /* Used to load/save memory options */
 static const struct Config_Tag configs_Memory[] =
 {
-	{ "nMemorySize", Int_Tag, &ConfigureParams.Memory.nMemorySize },
-	{ "nTTRamSize", Int_Tag, &ConfigureParams.Memory.nTTRamSize },
+	{ "nMemorySize", Int_Tag, &ConfigureParams.Memory.STRamSize_KB },
+	{ "nTTRamSize", Int_Tag, &ConfigureParams.Memory.TTRamSize_KB },
 	{ "bAutoSave", Bool_Tag, &ConfigureParams.Memory.bAutoSave },
 	{ "szMemoryCaptureFileName", String_Tag, ConfigureParams.Memory.szMemoryCaptureFileName },
 	{ "szAutoSaveFileName", String_Tag, ConfigureParams.Memory.szAutoSaveFileName },
@@ -430,6 +439,7 @@ static const struct Config_Tag configs_HardDisk[] =
 	{ "nGemdosCase", Int_Tag, &ConfigureParams.HardDisk.nGemdosCase },
 	{ "nWriteProtection", Int_Tag, &ConfigureParams.HardDisk.nWriteProtection },
 	{ "bFilenameConversion", Bool_Tag, &ConfigureParams.HardDisk.bFilenameConversion },
+	{ "bGemdosHostTime", Bool_Tag, &ConfigureParams.HardDisk.bGemdosHostTime },
 	{ "bUseHardDiskImage", Bool_Tag, &ConfigureParams.Acsi[0].bUseDevice },
 	{ "szHardDiskImage", String_Tag, ConfigureParams.Acsi[0].sDeviceFile },
 	{ "bUseIdeMasterHardDiskImage", Bool_Tag, &ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage },
@@ -513,6 +523,8 @@ static const struct Config_Tag configs_Midi[] =
 	{ "bEnableMidi", Bool_Tag, &ConfigureParams.Midi.bEnableMidi },
 	{ "sMidiInFileName", String_Tag, ConfigureParams.Midi.sMidiInFileName },
 	{ "sMidiOutFileName", String_Tag, ConfigureParams.Midi.sMidiOutFileName },
+	{ "sMidiInPortName", String_Tag, ConfigureParams.Midi.sMidiInPortName },
+	{ "sMidiOutPortName", String_Tag, ConfigureParams.Midi.sMidiOutPortName },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -543,6 +555,7 @@ static const struct Config_Tag configs_System[] =
 	{ "bCycleExactCpu", Bool_Tag, &ConfigureParams.System.bCycleExactCpu },
 	{ "n_FPUType", Int_Tag, &ConfigureParams.System.n_FPUType },
 /* JIT	{ "bCompatibleFPU", Bool_Tag, &ConfigureParams.System.bCompatibleFPU }, */
+	{ "bSoftFloatFPU", Bool_Tag, &ConfigureParams.System.bSoftFloatFPU },
 	{ "bMMU", Bool_Tag, &ConfigureParams.System.bMMU },
 #endif
 	{ "VideoTiming", Int_Tag, &ConfigureParams.System.VideoTimingMode },
@@ -578,7 +591,6 @@ void Configuration_SetDefault(void)
 	/* Set defaults for logging and tracing */
 	strcpy(ConfigureParams.Log.sLogFileName, "stderr");
 	strcpy(ConfigureParams.Log.sTraceFileName, "stderr");
-	ConfigureParams.Log.nExceptionDebugMask = DEFAULT_EXCEPTIONS;
 	ConfigureParams.Log.nTextLogLevel = LOG_INFO;
 	ConfigureParams.Log.nAlertDlgLogLevel = LOG_ERROR;
 	ConfigureParams.Log.bConfirmQuit = true;
@@ -587,10 +599,14 @@ void Configuration_SetDefault(void)
 
 	/* Set defaults for debugger */
 	ConfigureParams.Debugger.nNumberBase = 10;
-	ConfigureParams.Debugger.nDisasmLines = 8;
-	ConfigureParams.Debugger.nMemdumpLines = 8;
+	ConfigureParams.Debugger.nSymbolLines = -1; /* <0: use terminal size */
+	ConfigureParams.Debugger.nMemdumpLines = -1; /* <0: use terminal size */
+	ConfigureParams.Debugger.nDisasmLines = -1; /* <0: use terminal size */
+	ConfigureParams.Debugger.nExceptionDebugMask = DEFAULT_EXCEPTIONS;
 	/* external one has nicer output, but isn't as complete as UAE one */
 	ConfigureParams.Debugger.bDisasmUAE = false;
+	ConfigureParams.Debugger.bSymbolsResident = false;
+	ConfigureParams.Debugger.bMatchAllSymbols = false;
 	ConfigureParams.Debugger.nDisasmOptions = Disasm_GetOptions();
 
 	/* Set defaults for floppy disk images */
@@ -619,6 +635,7 @@ void Configuration_SetDefault(void)
 	/* Set defaults for hard disks */
 	ConfigureParams.HardDisk.bBootFromHardDisk = false;
 	ConfigureParams.HardDisk.bFilenameConversion = false;
+	ConfigureParams.HardDisk.bGemdosHostTime = false;
 	ConfigureParams.HardDisk.nGemdosCase = GEMDOS_NOP;
 	ConfigureParams.HardDisk.nWriteProtection = WRITEPROT_OFF;
 	ConfigureParams.HardDisk.nGemdosDrive = DRIVE_C;
@@ -682,6 +699,7 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_DEBUG] = SDLK_PAUSE;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_OPTIONS] = SDLK_o;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_FULLSCREEN] = SDLK_f;
+	ConfigureParams.Shortcut.withModifier[SHORTCUT_BORDERS] = SDLK_b;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_MOUSEGRAB] = SDLK_m;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_COLDRESET] = SDLK_c;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_WARMRESET] = SDLK_r;
@@ -702,8 +720,8 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_PAD_B] = SDLK_F4;
 
 	/* Set defaults for Memory */
-	ConfigureParams.Memory.nMemorySize = 1;     /* 1 MiB */
-	ConfigureParams.Memory.nTTRamSize = 0;     /* disabled */
+	ConfigureParams.Memory.STRamSize_KB = 1024;	/* 1 MiB */
+	ConfigureParams.Memory.TTRamSize_KB = 0;	/* disabled */
 	ConfigureParams.Memory.bAutoSave = false;
 	sprintf(ConfigureParams.Memory.szMemoryCaptureFileName, "%s%chatari.sav",
 	        psHomeDir, PATHSEP);
@@ -724,11 +742,15 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Midi.bEnableMidi = false;
 	strcpy(ConfigureParams.Midi.sMidiInFileName, "/dev/snd/midiC1D0");
 	strcpy(ConfigureParams.Midi.sMidiOutFileName, "/dev/snd/midiC1D0");
+	strcpy(ConfigureParams.Midi.sMidiInPortName, "Off");
+	strcpy(ConfigureParams.Midi.sMidiOutPortName, "Off");
 
 	/* Set defaults for Screen */
 	ConfigureParams.Screen.bFullScreen = false;
 	ConfigureParams.Screen.bKeepResolution = true;
-#if !WITH_SDL2
+#if WITH_SDL2
+	ConfigureParams.Screen.bResizable = true;
+#else
 	ConfigureParams.Screen.bKeepResolutionST = false;
 #endif
 	ConfigureParams.Screen.nFrameSkips = AUTO_FRAMESKIP_LIMIT;
@@ -749,7 +771,9 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Screen.nMaxWidth = 2*NUM_VISIBLE_LINE_PIXELS;
 	ConfigureParams.Screen.nMaxHeight = 2*NUM_VISIBLE_LINES+STATUSBAR_MAX_HEIGHT;
 	ConfigureParams.Screen.bForceMax = false;
+	ConfigureParams.Screen.DisableVideo = false;
 #if WITH_SDL2
+	ConfigureParams.Screen.bUseSdlRenderer = true;
 	ConfigureParams.Screen.nRenderScaleQuality = 0;
 	ConfigureParams.Screen.bUseVsync = false;
 #endif
@@ -773,12 +797,13 @@ void Configuration_SetDefault(void)
 	/* Set defaults for System */
 	ConfigureParams.System.nMachineType = MACHINE_ST;
 	ConfigureParams.System.nCpuLevel = 0;
-	ConfigureParams.System.nCpuFreq = 8;
+	ConfigureParams.System.nCpuFreq = 8;	nCpuFreqShift = 0;
 	ConfigureParams.System.nDSPType = DSP_TYPE_NONE;
 	ConfigureParams.System.bAddressSpace24 = true;
 #if ENABLE_WINUAE_CPU
 	ConfigureParams.System.n_FPUType = FPU_NONE;
 	ConfigureParams.System.bCompatibleFPU = true; /* JIT */
+	ConfigureParams.System.bSoftFloatFPU = false;
 	ConfigureParams.System.bMMU = false;
 	ConfigureParams.System.bCycleExactCpu = true;
 #endif
@@ -819,6 +844,7 @@ void Configuration_SetDefault(void)
 void Configuration_Apply(bool bReset)
 {
 	int i;
+	int size;
 
 	if (bReset)
 	{
@@ -847,6 +873,16 @@ void Configuration_Apply(bool bReset)
 		ConfigureParams.Screen.nForceBpp = 0;
 	}
 
+	/* Check/convert ST RAM size in KB */
+	size = STMemory_RAM_Validate_Size_KB ( ConfigureParams.Memory.STRamSize_KB );
+	if ( size < 0 )
+		size = 1024;
+	ConfigureParams.Memory.STRamSize_KB = size;
+	STMemory_Init ( ConfigureParams.Memory.STRamSize_KB * 1024 );
+
+	/* Update variables depending on the new CPU Freq (to do before other ClocksTimings_xxx functions) */
+	Configuration_ChangeCpuFreq ( ConfigureParams.System.nCpuFreq );
+
 	/* Init clocks for this machine */
 	ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 
@@ -1096,8 +1132,8 @@ void Configuration_MemorySnapShot_Capture(bool bSave)
 	MemorySnapShot_Store(ConfigureParams.Rom.szTosImageFileName, sizeof(ConfigureParams.Rom.szTosImageFileName));
 	MemorySnapShot_Store(ConfigureParams.Rom.szCartridgeImageFileName, sizeof(ConfigureParams.Rom.szCartridgeImageFileName));
 
-	MemorySnapShot_Store(&ConfigureParams.Memory.nMemorySize, sizeof(ConfigureParams.Memory.nMemorySize));
-	MemorySnapShot_Store(&ConfigureParams.Memory.nTTRamSize, sizeof(ConfigureParams.Memory.nTTRamSize));
+	MemorySnapShot_Store(&ConfigureParams.Memory.STRamSize_KB, sizeof(ConfigureParams.Memory.STRamSize_KB));
+	MemorySnapShot_Store(&ConfigureParams.Memory.TTRamSize_KB, sizeof(ConfigureParams.Memory.TTRamSize_KB));
 
 	MemorySnapShot_Store(&ConfigureParams.DiskImage.szDiskFileName[0], sizeof(ConfigureParams.DiskImage.szDiskFileName[0]));
 	MemorySnapShot_Store(&ConfigureParams.DiskImage.szDiskZipPath[0], sizeof(ConfigureParams.DiskImage.szDiskZipPath[0]));
@@ -1149,3 +1185,48 @@ void Configuration_MemorySnapShot_Capture(bool bSave)
 	if (!bSave)
 		Configuration_Apply(true);
 }
+
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * This function should be called each time the CPU freq is changed.
+ * It will update the main configuration, as well as the corresponding
+ * value for nCpuFreqShift
+ *
+ * In case the new CPU freq is different from the current CPU freq, we
+ * also call MClocksTimings_UpdateCpuFreqEmul and 68000_ChangeCpuFreq
+ * to update some low level hardware related values
+ */
+void Configuration_ChangeCpuFreq ( int CpuFreq_new )
+{
+	int	CpuFreq_old = ConfigureParams.System.nCpuFreq;
+
+//fprintf ( stderr , "changing cpu freq %d -> %d\n" , ConfigureParams.System.nCpuFreq , CpuFreq_new );
+
+	/* In case value is not exactly 8, 16 or 32, then we change it so */
+	if ( CpuFreq_new < 12 )
+	{
+		ConfigureParams.System.nCpuFreq = 8;
+		nCpuFreqShift = 0;
+	}
+	else if ( CpuFreq_new > 26 )
+	{
+		ConfigureParams.System.nCpuFreq = 32;
+		nCpuFreqShift = 2;
+	}
+	else
+	{
+		ConfigureParams.System.nCpuFreq = 16;
+		nCpuFreqShift = 1;
+	}
+
+	ClocksTimings_UpdateCpuFreqEmul ( ConfigureParams.System.nMachineType , nCpuFreqShift );
+
+	if ( CpuFreq_old != CpuFreq_new )
+	{
+		M68000_ChangeCpuFreq();
+	}
+}
+
+
