@@ -23,6 +23,7 @@
 #include "debugcpu.h"
 #ifdef WINUAE_FOR_HATARI
 #include "debug.h"
+#include "blitter.h"
 #endif
 
 #define WRITE_LOG_BUF_SIZE 4096
@@ -61,6 +62,7 @@ typedef struct _LARGE_INTEGER
 
 #ifdef CPUEMU_13
 
+#ifndef WINUAE_FOR_HATARI
 uae_u8 cycle_line[256 + 1];
 
 static void sync_ce020 (void)
@@ -76,7 +78,6 @@ static void sync_ce020 (void)
 	}
 }
 
-#ifndef WINUAE_FOR_HATARI
 #define SETIFCHIP \
 	if (addr < 0xd80000) \
 		last_custom_value1 = v;
@@ -186,8 +187,19 @@ uae_u32 wait_cpu_cycle_read_ce020 (uaecptr addr, int mode)
 	SETIFCHIP
 
 #else						/* WINUAE_FOR_HATARI */
-	sync_ce020 ();
-	x_do_cycles_pre (CYCLE_UNIT);
+
+//fprintf ( stderr , "wait read ce020 glob %lu\n" , CyclesGlobalClockCounter );
+//fprintf ( stderr , "wait read ce020 %lu %lu\n" , currcycle / cpucycleunit , currcycle );
+	int bus_pos = ( CyclesGlobalClockCounter + currcycle*2/CYCLE_UNIT ) & 3;
+	if ( ( bus_pos & 2 ) == 2 )
+//	if ( bus_pos )
+	{
+//		fprintf ( stderr , "mem wait read %x %d %lu %lu\n" , addr , mode , currcycle / cpucycleunit , currcycle );
+		x_do_cycles ((4-bus_pos)*cpucycleunit);
+//		fprintf ( stderr , "mem wait read after %x %d %lu %lu\n" , addr , mode , currcycle / cpucycleunit , currcycle );
+	}
+
+//fprintf ( stderr , "wait read2 ce020 %lu %lu\n" , currcycle / cpucycleunit , currcycle );
 
 	if (mode < 0)
 		v = get_long (addr);
@@ -196,8 +208,10 @@ uae_u32 wait_cpu_cycle_read_ce020 (uaecptr addr, int mode)
 	else if (mode == 0)
 		v = get_byte (addr);
 
-	if (currprefs.cpu_model == 68020)
-		x_do_cycles_post (CYCLE_UNIT / 2, v);
+//fprintf ( stderr , "wait read3 ce020 %lu %lu\n" , currcycle / cpucycleunit , currcycle );
+	x_do_cycles_post (3*cpucycleunit, v);
+//fprintf ( stderr , "wait read4 ce020 %lu %lu\n" , currcycle / cpucycleunit , currcycle );
+
 #endif						/* WINUAE_FOR_HATARI */
 
 	return v;
@@ -294,8 +308,18 @@ void wait_cpu_cycle_write_ce020 (uaecptr addr, int mode, uae_u32 v)
 	SETIFCHIP
 
 #else						/* WINUAE_FOR_HATARI */
-	sync_ce020 ();
-	x_do_cycles_pre (CYCLE_UNIT);
+
+//fprintf ( stderr , "wait read ce020 %lu %lu\n" , currcycle / cpucycleunit , currcycle );
+	int bus_pos = ( CyclesGlobalClockCounter + currcycle*2/CYCLE_UNIT ) & 3;
+	if ( ( bus_pos & 2 ) == 2 )
+//	if ( bus_pos )
+	{
+//		fprintf ( stderr , "mem wait read %x %d %lu %lu\n" , addr , mode , currcycle / cpucycleunit , currcycle );
+		x_do_cycles ((4-bus_pos)*cpucycleunit);
+//		fprintf ( stderr , "mem wait read after %x %d %lu %lu\n" , addr , mode , currcycle / cpucycleunit , currcycle );
+	}
+
+//fprintf ( stderr , "wait read2 ce020 %lu %lu\n" , currcycle / cpucycleunit , currcycle );
 
 	if (mode < 0)
 		put_long (addr, v);
@@ -304,8 +328,10 @@ void wait_cpu_cycle_write_ce020 (uaecptr addr, int mode, uae_u32 v)
 	else if (mode == 0)
 		put_byte (addr, v);
 
-	if (currprefs.cpu_model == 68020)
-		x_do_cycles_post (CYCLE_UNIT / 2, v);
+//fprintf ( stderr , "wait read3 ce020 %lu %lu\n" , currcycle / cpucycleunit , currcycle );
+	x_do_cycles_post (3*cpucycleunit, v);
+//fprintf ( stderr , "wait read4 ce020 %lu %lu\n" , currcycle / cpucycleunit , currcycle );
+
 #endif						/* WINUAE_FOR_HATARI */
 }
 
@@ -328,6 +354,31 @@ void do_cycles_ce (unsigned long cycles)
 }
 
 
+#ifdef WINUAE_FOR_HATARI
+/* Same as do_cycles_ce() with cycle exact blitter support */
+void do_cycles_ce_hatari_blitter (unsigned long cycles)
+{
+	cycles += extra_cycle;
+	while (cycles >= CYCLE_UNIT) {
+#ifndef WINUAE_FOR_HATARI
+		int hpos = current_hpos () + 1;
+		decide_line (hpos);
+		sync_copper (hpos);
+		decide_fetch_ce (hpos);
+		if (bltstate != BLT_done)
+			decide_blitter (hpos);
+#endif						/* WINUAE_FOR_HATARI */
+		if ( Blitter_Check_Simultaneous_CPU() == 0 )
+			do_cycles (1 * CYCLE_UNIT);
+		Blitter_HOG_CPU_do_cycles_after ( 2 );
+
+		cycles -= CYCLE_UNIT;
+	}
+	extra_cycle = cycles;
+}
+#endif
+
+
 #ifndef WINUAE_FOR_HATARI
 void do_cycles_ce020 (unsigned long cycles)
 #else
@@ -502,12 +553,12 @@ void uae_reset (int hardreset)
 {
 	currprefs.quitstatefile[0] = changed_prefs.quitstatefile[0] = 0;
 
-	if (quit_program == 0) {
-		quit_program = -2;
+	if (uae_quit_program == 0) {
+		uae_quit_program = -UAE_RESET;
+		if (keyboardreset)
+			uae_quit_program = -UAE_RESET_KEYBOARD;
 		if (hardreset)
-			quit_program = -3;
-	}
-
+			uae_quit_program = -UAE_RESET_HARD;
 }
 #endif
 
