@@ -104,13 +104,29 @@ int Init680x0(void)
 		default: fprintf (stderr, "Init680x0() : Error, cpu_level unknown\n");
 	}
 
+	/* Only 68040/60 can have 'internal' FPU */
+	if ( ( ConfigureParams.System.n_FPUType == FPU_CPU ) && ( changed_prefs.cpu_model < 68040 ) )
+		ConfigureParams.System.n_FPUType = FPU_NONE;
+
+	/* 68000/10 can't have an FPU */
+	if ( ( ConfigureParams.System.n_FPUType != FPU_NONE ) && ( changed_prefs.cpu_model < 68020 ) )
+	{
+		Log_Printf(LOG_WARN, "FPU is not supported in 68000/010 configurations, disabling FPU\n");
+		ConfigureParams.System.n_FPUType = FPU_NONE;
+	}
+
 	changed_prefs.int_no_unimplemented = true;
 	changed_prefs.fpu_no_unimplemented = true;
 	changed_prefs.cpu_compatible = ConfigureParams.System.bCompatibleCpu;
 	changed_prefs.address_space_24 = ConfigureParams.System.bAddressSpace24;
 	changed_prefs.cpu_cycle_exact = ConfigureParams.System.bCycleExactCpu;
+	changed_prefs.cpu_memory_cycle_exact = ConfigureParams.System.bCycleExactCpu;
 	changed_prefs.fpu_model = ConfigureParams.System.n_FPUType;
 	changed_prefs.fpu_strict = ConfigureParams.System.bCompatibleFPU;
+	changed_prefs.fpu_softfloat = ConfigureParams.System.bSoftFloatFPU;
+
+        /* Always emulate instr/data caches for cpu >= 68020 */
+        changed_prefs.cpu_data_cache = true;
 
 	/* Set the MMU model by taking the same value as CPU model */
 	/* MMU is only supported for CPU >=68030 */
