@@ -19,13 +19,10 @@ extern int intlev (void);
 extern int Init680x0(void);
 extern void Exit680x0(void);
 
-extern uae_u32 OpCode_GemDos(uae_u32 opcode);
-extern uae_u32 OpCode_SysInit(uae_u32 opcode);
-extern uae_u32 OpCode_VDI(uae_u32 opcode);
-extern uae_u32 OpCode_NatFeat_ID(uae_u32 opcode);
-extern uae_u32 OpCode_NatFeat_Call(uae_u32 opcode);
-
-#define write_log printf
-
+extern uae_u32 REGPARAM3 OpCode_GemDos(uae_u32 opcode);
+extern uae_u32 REGPARAM3 OpCode_SysInit(uae_u32 opcode);
+extern uae_u32 REGPARAM3 OpCode_VDI(uae_u32 opcode);
+extern uae_u32 REGPARAM3 OpCode_NatFeat_ID(uae_u32 opcode);
+extern uae_u32 REGPARAM3 OpCode_NatFeat_Call(uae_u32 opcode);
 
 #endif /* HATARI_GLUE_H */
