@@ -101,6 +101,9 @@ static bool needmman (void)
 
 static void nocanbang (void)
 {
+	if (canbang) {
+		write_log(_T("Switching JIT direct off!\n"));
+	}
 	canbang = 0;
 }
 #endif
@@ -229,6 +232,7 @@ void dummy_put (uaecptr addr, int size, uae_u32 val)
 	if (addr >= 0xf00000 && addr < 0xf80000 && size < 2)
 		flash_write(addr, val);
 #endif
+
 	if (gary_nonrange(addr) || (size > 1 && gary_nonrange(addr + size - 1))) {
 		if (gary_timeout)
 			gary_wait (addr, size, true);
@@ -253,7 +257,27 @@ uae_u32 dummy_get (uaecptr addr, int size, bool inst, uae_u32 defvalue)
 		return 8;
 	}
 #endif
-
+#if ACA500x_DEVELOPMENT
+	if (addr == 0xb03000) {
+		return 0xffff;
+	}
+	if (addr == 0xb07000) {
+		return 0x0000;
+	}
+	if (addr == 0xb2f800) {
+		return 0xffff;
+	}
+	if (addr == 0xb3b800) {
+		return 0x0000;
+	}
+	if (addr == 0xb3f800) {
+		return currprefs.cpu_model > 68000 ? 0x0000 : 0xffff;
+	}
+	if (addr == 0xb0b000) {
+		extern bool isideint(void);
+		return isideint() ? 0xffff : 0x0000;
+	}
+#endif
 	if (gary_nonrange(addr) || (size > 1 && gary_nonrange(addr + size - 1))) {
 		if (gary_timeout)
 			gary_wait (addr, size, false);
@@ -320,12 +344,6 @@ static uae_u32 REGPARAM2 dummy_wget (uaecptr addr)
 {
 #ifdef JIT
 	special_mem |= S_READ;
-#endif
-#if 0
-	if (addr == 0xb0b000) {
-		extern uae_u16 isideint(void);
-		return isideint();
-	}
 #endif
 	if (illegal_mem)
 		dummylog (0, addr, 2, 0, 0);
@@ -613,6 +631,56 @@ static uae_u8 REGPARAM3 *STmem_xlate(uaecptr addr)
     return STmemory + addr;
 }
 
+/* Same functions for ST RAM but with MMU/MCU enabled to translate addresses */
+
+static uae_u32 REGPARAM3 STmem_lget_MMU(uaecptr addr)
+{
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    return do_get_mem_long(STmemory + addr);
+}
+
+static uae_u32 REGPARAM3 STmem_wget_MMU(uaecptr addr)
+{
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    return do_get_mem_word(STmemory + addr);
+}
+
+static uae_u32 REGPARAM3 STmem_bget_MMU(uaecptr addr)
+{
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    return STmemory[addr];
+}
+
+static void REGPARAM3 STmem_lput_MMU(uaecptr addr, uae_u32 l)
+{
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    do_put_mem_long(STmemory + addr, l);
+}
+
+static void REGPARAM3 STmem_wput_MMU(uaecptr addr, uae_u32 w)
+{
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    do_put_mem_word(STmemory + addr, w);
+}
+
+static void REGPARAM3 STmem_bput_MMU(uaecptr addr, uae_u32 b)
+{
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    STmemory[addr] = b;
+}
+
 
 /*
  * **** ST RAM system memory ****
@@ -623,12 +691,14 @@ static uae_u8 REGPARAM3 *STmem_xlate(uaecptr addr)
  */
 static uae_u32 REGPARAM3 SysMem_lget(uaecptr addr)
 {
+    uaecptr addr_in = addr;
+
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
 
     if(addr < 0x800 && !regs.s)
     {
-      M68000_BusError(addr, 1, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+      M68000_BusError(addr_in, 1, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
       return 0;
     }
 
@@ -637,13 +707,15 @@ static uae_u32 REGPARAM3 SysMem_lget(uaecptr addr)
 
 static uae_u32 REGPARAM3 SysMem_wget(uaecptr addr)
 {
+    uaecptr addr_in = addr;
+
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
 
     /* Only CPU will trigger bus error if bit S=0, not the blitter */
     if(addr < 0x800 && !regs.s && BusMode == BUS_MODE_CPU)
     {
-      M68000_BusError(addr, 1, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+      M68000_BusError(addr_in, 1, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
       return 0;
     }
 
@@ -652,12 +724,14 @@ static uae_u32 REGPARAM3 SysMem_wget(uaecptr addr)
 
 static uae_u32 REGPARAM3 SysMem_bget(uaecptr addr)
 {
+    uaecptr addr_in = addr;
+
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
 
     if(addr < 0x800 && !regs.s)
     {
-      M68000_BusError(addr, 1, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+      M68000_BusError(addr_in, 1, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
       return 0;
     }
 
@@ -666,12 +740,14 @@ static uae_u32 REGPARAM3 SysMem_bget(uaecptr addr)
 
 static void REGPARAM3 SysMem_lput(uaecptr addr, uae_u32 l)
 {
+    uaecptr addr_in = addr;
+
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
 
     if(addr < 0x8 || (addr < 0x800 && !regs.s))
     {
-      M68000_BusError(addr, 0, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+      M68000_BusError(addr_in, 0, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
       return;
     }
 
@@ -680,6 +756,8 @@ static void REGPARAM3 SysMem_lput(uaecptr addr, uae_u32 l)
 
 static void REGPARAM3 SysMem_wput(uaecptr addr, uae_u32 w)
 {
+    uaecptr addr_in = addr;
+
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
 
@@ -688,7 +766,7 @@ static void REGPARAM3 SysMem_wput(uaecptr addr, uae_u32 w)
     {
       if ( BusMode == BUS_MODE_CPU )
       {
-	M68000_BusError(addr, 0, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+	M68000_BusError(addr_in, 0, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
 	return;
       }
       /* If blitter writes < 0x8 then it should be ignored, else the write should be made */
@@ -701,42 +779,160 @@ static void REGPARAM3 SysMem_wput(uaecptr addr, uae_u32 w)
 
 static void REGPARAM3 SysMem_bput(uaecptr addr, uae_u32 b)
 {
+    uaecptr addr_in = addr;
+
     addr -= STmem_start & STmem_mask;
     addr &= STmem_mask;
 
     if(addr < 0x8 || (addr < 0x800 && !regs.s))
     {
-      M68000_BusError(addr, 0, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+      M68000_BusError(addr_in, 0, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
       return;
     }
 
     STmemory[addr] = b;
 }
 
+/* Same functions for ST RAM system but with MMU/MCU enabled to translate addresses */
+
+static uae_u32 REGPARAM3 SysMem_lget_MMU(uaecptr addr)
+{
+    uaecptr addr_in = addr;
+
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+
+    if(addr < 0x800 && !regs.s)
+    {
+      M68000_BusError(addr_in, 1, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+      return 0;
+    }
+
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    return do_get_mem_long(STmemory + addr);
+}
+
+static uae_u32 REGPARAM3 SysMem_wget_MMU(uaecptr addr)
+{
+    uaecptr addr_in = addr;
+
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+
+    /* Only CPU will trigger bus error if bit S=0, not the blitter */
+    if(addr < 0x800 && !regs.s && BusMode == BUS_MODE_CPU)
+    {
+      M68000_BusError(addr_in, 1, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+      return 0;
+    }
+
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    return do_get_mem_word(STmemory + addr);
+}
+
+static uae_u32 REGPARAM3 SysMem_bget_MMU(uaecptr addr)
+{
+    uaecptr addr_in = addr;
+
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+
+    if(addr < 0x800 && !regs.s)
+    {
+      M68000_BusError(addr_in, 1, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+      return 0;
+    }
+
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    return STmemory[addr];
+}
+
+static void REGPARAM3 SysMem_lput_MMU(uaecptr addr, uae_u32 l)
+{
+    uaecptr addr_in = addr;
+
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+
+    if(addr < 0x8 || (addr < 0x800 && !regs.s))
+    {
+      M68000_BusError(addr_in, 0, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
+      return;
+    }
+
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    do_put_mem_long(STmemory + addr, l);
+}
+
+static void REGPARAM3 SysMem_wput_MMU(uaecptr addr, uae_u32 w)
+{
+    uaecptr addr_in = addr;
+
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+
+    /* Only CPU will trigger bus error if bit S=0, not the blitter */
+    if(addr < 0x8 || (addr < 0x800 && !regs.s))
+    {
+      if ( BusMode == BUS_MODE_CPU )
+      {
+	M68000_BusError(addr_in, 0, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
+	return;
+      }
+      /* If blitter writes < 0x8 then it should be ignored, else the write should be made */
+      else if ( ( BusMode == BUS_MODE_BLITTER ) && ( addr < 0x8 ) )
+	return;
+    }
+
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    do_put_mem_word(STmemory + addr, w);
+}
+
+static void REGPARAM3 SysMem_bput_MMU(uaecptr addr, uae_u32 b)
+{
+    uaecptr addr_in = addr;
+
+    addr -= STmem_start & STmem_mask;
+    addr &= STmem_mask;
+
+    if(addr < 0x8 || (addr < 0x800 && !regs.s))
+    {
+      M68000_BusError(addr_in, 0, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
+      return;
+    }
+
+    addr = STMemory_MMU_Translate_Addr ( addr );
+    STmemory[addr] = b;
+}
+
 
 /*
  * **** Void memory ****
  * Between the ST-RAM end and the 4 MB barrier, there is a void memory space:
- * Reading always returns the same value and writing does nothing at all.
- * [NP] : this is not correct, reading does not always return 0, when there's
- * no memory, it will return the latest data that was read on the bus.
+ * Reading depends on current activity on the bus and writing does nothing at all.
+ *
+ * [NP] : When there's no memory, it will return the latest data that was read on the bus.
  * In many cases, this will return the word that was just read in the 68000's
  * prefetch register to decode the next opcode (tested on a real STF)
+ *
+ * STF : this seems to always return the last data word on the bus read by the CPU (tested on real STF)
+ * STE : result seems also related to what the shifter displays, it can be data read by the CPU or by
+ *       the shifter ; for now do the same as STF [TODO improve this ? No program are known to depend on it]
  */
 
 static uae_u32 REGPARAM3 VoidMem_lget(uaecptr addr)
 {
-    return 0;
+    return ( regs.db<<16 ) | regs.db;
 }
 
 static uae_u32 REGPARAM3 VoidMem_wget(uaecptr addr)
 {
-    return 0;
+    return regs.db;
 }
 
 static uae_u32 REGPARAM3 VoidMem_bget(uaecptr addr)
 {
-    return 0;
+    return regs.db;
 }
 
 static void REGPARAM3 VoidMem_lput(uaecptr addr, uae_u32 l)
@@ -933,7 +1129,7 @@ static uae_u8 REGPARAM3 *IOmem_xlate(uaecptr addr)
 
 /* **** Address banks **** */
 
-static addrbank dummy_bank =
+addrbank dummy_bank =
 {
     dummy_lget, dummy_wget, dummy_bget,
     dummy_lput, dummy_wput, dummy_bput,
@@ -966,6 +1162,22 @@ static addrbank SysMem_bank =
     SysMem_lget, SysMem_wget, ABFLAG_RAM
 };
 
+static addrbank STmem_bank_MMU =			/* similar to STmem_bank with MMU/MCU enabled */
+{
+    STmem_lget_MMU, STmem_wget_MMU, STmem_bget_MMU,
+    STmem_lput_MMU, STmem_wput_MMU, STmem_bput_MMU,
+    STmem_xlate, STmem_check, NULL, "st_mem" , "ST memory + MMU",
+    STmem_lget_MMU, STmem_wget_MMU, ABFLAG_RAM
+};
+
+static addrbank SysMem_bank_MMU =			/* similar to STmem_bank with MMU/MCU enabled */
+{
+    SysMem_lget_MMU, SysMem_wget_MMU, SysMem_bget_MMU,
+    SysMem_lput_MMU, SysMem_wput_MMU, SysMem_bput_MMU,
+    STmem_xlate, STmem_check, NULL, "sys_mem" , "Sys memory + MMU",
+    SysMem_lget_MMU, SysMem_wget_MMU, ABFLAG_RAM
+};
+
 static addrbank VoidMem_bank =
 {
     VoidMem_lget, VoidMem_wget, VoidMem_bget,
@@ -1017,13 +1229,15 @@ static addrbank IOmem_bank =
 bool mapped_malloc (addrbank *ab)
 {
 	ab->startmask = ab->start;
-	ab->baseaddr = xcalloc (uae_u8, ab->allocated + 4);
+	ab->baseaddr = xcalloc (uae_u8, ab->reserved_size + 4);
+	ab->allocated_size =  ab->baseaddr != NULL ? ab->reserved_size : 0;
 	return ab->baseaddr != NULL; 
 }
 
 void mapped_free (addrbank *ab)
 {
 	xfree(ab->baseaddr);
+	ab->allocated_size = 0;
 	ab->baseaddr = NULL; 
 }
 
@@ -1061,7 +1275,7 @@ static shmpiece *find_shmpiece (uae_u8 *base, bool safe)
 		if (safe)
 #endif
 			return 0;
-		write_log (_T("NATMEM: Failure to find mapping at %08X, %p\n"), base - NATMEM_OFFSET, base);
+		write_log (_T("NATMEM: Failure to find mapping at %08lx, %p\n"), base - NATMEM_OFFSET, base);
 		nocanbang ();
 		return 0;
 	}
@@ -1089,7 +1303,7 @@ static void delete_shmmaps (uae_u32 start, uae_u32 size)
 				size = x->size;
 			}
 
-			shmdt (x->native_address);
+			uae_shmdt (x->native_address);
 			size -= x->size;
 			start += x->size;
 			if (x->next)
@@ -1118,6 +1332,9 @@ static void add_shmmaps (uae_u32 start, addrbank *what)
 	if (!base)
 		return;
 
+	if (what->jit_read_flag && what->jit_write_flag)
+		return;
+
 	x = find_shmpiece (base, false);
 	if (!x)
 		return;
@@ -1125,7 +1342,7 @@ static void add_shmmaps (uae_u32 start, addrbank *what)
 	y = xmalloc (shmpiece, 1);
 	*y = *x;
 	base = ((uae_u8 *) NATMEM_OFFSET) + start;
-	y->native_address = (uae_u8*)shmat (y->id, base, 0);
+	y->native_address = (uae_u8*)uae_shmat (what, y->id, base, 0, NULL);
 	if (y->native_address == (void *) -1) {
 		write_log (_T("NATMEM: Failure to map existing at %08x (%p)\n"), start, base);
 		dumplist ();
@@ -1149,24 +1366,49 @@ bool mapped_malloc (addrbank *ab)
 	bool rtgmem = (ab->flags & ABFLAG_RTG) != 0;
 	static int recurse;
 
-	ab->startmask = ab->start;
-	if (!needmman () && (!rtgmem || currprefs.cpu_model < 68020)) {
-		nocanbang ();
-		ab->flags &= ~ABFLAG_DIRECTMAP;
-		if (ab->flags & ABFLAG_NOALLOC) {
+	if (ab->allocated_size) {
+		write_log(_T("mapped_malloc with memory bank '%s' already allocated!?\n"), ab->name);
+	}
+	ab->allocated_size = 0;
+
+	if (ab->label && ab->label[0] == '*') {
+		if (ab->start == 0 || ab->start == 0xffffffff) {
+			write_log(_T("mapped_malloc(*) without start address!\n"));
+			return false;
+		}
+	}
+
+	struct uae_mman_data md = { 0 };
+	uaecptr start = ab->start;
+	if (uae_mman_info(ab, &md)) {
+		start = md.start;
+	}
+	ab->startmask = start;
+        if (!md.directsupport || (ab->flags & ABFLAG_ALLOCINDIRECT)) {
+                if (!(ab->flags & ABFLAG_ALLOCINDIRECT)) {
+                        if (canbang) {
+                                write_log(_T("JIT direct switched off: %s\n"), ab->name);
+                        }
+                        nocanbang();
+                }
+                ab->flags &= ~ABFLAG_DIRECTMAP;
+                if (ab->flags & ABFLAG_NOALLOC) {
+                        ab->allocated_size = ab->reserved_size;
 #if MAPPED_MALLOC_DEBUG
 			write_log(_T("mapped_malloc noalloc %s\n"), ab->name);
 #endif
 			return true;
 		}
-		ab->baseaddr = xcalloc (uae_u8, ab->allocated + 4);
+		ab->baseaddr = xcalloc (uae_u8, ab->reserved_size + 4);
+		if (ab->baseaddr)
+			ab->allocated_size = ab->reserved_size;
 #if MAPPED_MALLOC_DEBUG
 		write_log(_T("mapped_malloc nodirect %s %p\n"), ab->name, ab->baseaddr);
 #endif
 		return ab->baseaddr != NULL;
 	}
 
-	id = shmget (IPC_PRIVATE, ab->allocated, 0x1ff, ab->label);
+	id = uae_shmget (UAE_IPC_PRIVATE, ab, 0x1ff);
 	if (id == -1) {
 		nocanbang ();
 		if (recurse)
@@ -1177,16 +1419,17 @@ bool mapped_malloc (addrbank *ab)
 		return ab->baseaddr != NULL;
 	}
 	if (!(ab->flags & ABFLAG_NOALLOC)) {
-		answer = shmat (ab, id, 0, 0);
-		shmctl (id, IPC_RMID, NULL);
+		answer = uae_shmat (iab, id, NULL, 0, &md));
+		uae_shmctl (id, UAE_IPC_RMID, NULL);
 	} else {
+		write_log(_T("MMAN: mapped_malloc using existing baseaddr %p\n"), ab->baseaddr);
 		answer = ab->baseaddr;
 	}
 	if (answer != (void *) -1) {
 		x = xmalloc (shmpiece, 1);
 		x->native_address = (uae_u8*)answer;
 		x->id = id;
-		x->size = ab->allocated;
+		x->size = ab->reserved_size;
 		x->name = ab->label;
 		x->next = shm_start;
 		x->prev = NULL;
@@ -1194,6 +1437,8 @@ bool mapped_malloc (addrbank *ab)
 			x->next->prev = x;
 		shm_start = x;
 		ab->baseaddr = x->native_address;
+		if (ab->baseaddr)
+			ab->allocated_size = ab->reserved_size;
 		ab->flags |= ABFLAG_DIRECTMAP;
 #if MAPPED_MALLOC_DEBUG
 		write_log(_T("mapped_malloc direct %s %p\n"), ab->name, ab->baseaddr);
@@ -1210,14 +1455,15 @@ bool mapped_malloc (addrbank *ab)
 	write_log(_T("mapped_malloc indirect %s %p\n"), ab->name, ab->baseaddr);
 #endif
 	return ab->baseaddr != NULL;}
+}
 
 #endif
 
 static void init_mem_banks (void)
 {
-	int i;
-
-	for (i = 0; i < MEMORY_BANKS; i++)
+	unsigned int i;
+        // unsigned so i << 16 won't overflow to negative when i >= 32768
+        for (i = 0; i < MEMORY_BANKS; i++)
 		put_mem_bank (i << 16, &dummy_bank, 0);
 #ifdef NATMEM_OFFSET
 	delete_shmmaps (0, 0xFFFF0000);
@@ -1250,9 +1496,10 @@ bool memory_region_bus_error ( uaecptr addr )
  *  CE_MEMBANK_CIA	Amiga only, for CIA chips
  *
  * Possible values for ce_cachable :
- *  bit 0 : cachable yes/no (for 68030 data cache)
- *  bit 1 : burst mode allowed when caching yes/no (for 68030 data cache)
- *	(not used, check for CE_MEMBANK_FAST32 instead)
+ *  bit 0 : cachable yes/no for data
+ *  bit 1 : burst mode allowed when caching data yes/no
+ *  bit 7 : cachable yes/no for instructions
+ *  bit 6 : burst mode allowed when caching instructions yes/no
  */
 static void init_ce_banks (void)
 {
@@ -1280,19 +1527,89 @@ static void fill_ce_banks (int start, int size, int banktype, int cachable )
 
 
 /*
- * Initialize the memory banks
+ * Initialize the standard RAM memory banks
+ *   - Unmodified STF/STE can have a max of 4 MB, but we can allow up to 14 MB
+ *     if RAM detection code is bypassed in the ROM (see tos.c)
+ *   - Falcon/TT can have up to 14 MB
+ * Depending on the current config, we enable MMU/MCU to translate addresses on
+ * each access (eg during boot when TOS checks RAM size)
+ *
+ * Most of the time, MMU bank size will be the same as RAM bank size and we use the "usual" non-MMU version
+ * which considers logical address == physical address and gives faster result
+ *
+ * See stMemory.c for the address translation used by the MMU/MCU
  */
-void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMemStart)
+void memory_map_Standard_RAM ( Uint32 MMU_Bank0_Size , Uint32 MMU_Bank1_Size )
+{
+	Log_Printf(LOG_DEBUG, "memory_map_Standard_RAM total=%d ram0=%d ram1=%d mmu0=%d mmu1=%d\n", STmem_size, RAM_Bank0_Size, RAM_Bank1_Size, MMU_Bank0_Size, MMU_Bank1_Size);
+
+	/* Between 0 and 4MB barrier, we default to void space */
+	map_banks_ce(&VoidMem_bank, 0x00, 0x400000 >> 16, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
+
+	/* Space between 4MB barrier and TOS ROM causes a bus error */
+	map_banks_ce(&BusErrMem_bank, 0x400000 >> 16, 0xA0, 0 , CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
+
+	/* Now map main ST RAM, overwriting the void and bus error regions if necessary */
+	/*  - Map the ST system RAM from 0 to 0x10000 (required for supervisor check between 0x0 and 0x800) */
+	/*  - Map rest of the ST RAM from 0x10000 to STmem_size */
+	/* If possible/needed we enable MMU/MCU translation for maximum accuracy (when sizes of banks differ between MMU and RAM) */
+	if ( ( Config_IsMachineST() || Config_IsMachineSTE() )
+	  && !ConfigureParams.System.bFastBoot
+	  && !ConfigureParams.Screen.bUseExtVdiResolutions
+	  && STmem_size <= 0x400000
+	  && ( ( MMU_Bank0_Size != RAM_Bank0_Size ) || ( ( RAM_Bank1_Size > 0 ) && ( MMU_Bank1_Size != RAM_Bank1_Size ) ) )
+	  )
+	{
+		/* We map memory according to the logical MMU configuration and we will translate addresses on each memory access. */
+		/* RAM bank 0 can never be empty, but RAM bank 1 can be empty. If there's no RAM bank 1, then we must */
+		/* use 'Void' region directly, we don't map it to STmem_bank_MMU */
+		Log_Printf(LOG_DEBUG, "memory_map_Standard_RAM - enable MMU %d %d %d\n",
+		           STmem_size, MMU_Bank0_Size, MMU_Bank1_Size);
+
+		/* Map RAM bank 0 to MMU bank 0 */
+		map_banks_ce(&SysMem_bank_MMU, 0x00, 0x10000 >> 16, 0, CE_MEMBANK_CHIP16, CACHE_ENABLE_BOTH);
+		map_banks_ce(&STmem_bank_MMU, 0x10000 >> 16, ( MMU_Bank0_Size - 0x10000 ) >> 16,
+			0, CE_MEMBANK_CHIP16, CACHE_ENABLE_BOTH);
+
+		/* If RAM bank 1 exists, we map it after MMU bank 0 ; else we keep void region */
+		if ( RAM_Bank1_Size > 0 )
+			map_banks_ce(&STmem_bank_MMU, MMU_Bank0_Size >> 16, MMU_Bank1_Size >> 16,
+				0, CE_MEMBANK_CHIP16, CACHE_ENABLE_BOTH);
+
+		/* [NP] There's a special case when bank0=128 and bank1=2048 : addresses between $40000 and $80000 */
+		/* are returning 'void' region too, which creates a "hole" in the memory mapping */
+		/* (this might be a bug / forgotten case by Atari in the STF non-IMP MMU as this memory */
+		/* configuration was certainly never used in real machines) */
+		if ( ( MMU_Bank0_Size == MEM_BANK_SIZE_128 ) && ( MMU_Bank1_Size == MEM_BANK_SIZE_2048 ) )
+		{
+			map_banks_ce(&VoidMem_bank, 0x40000 >> 16, 0x40000 >> 16, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
+		}
+	}
+	else
+	{
+		/* Don't enable MMU address translation */
+		map_banks_ce(&SysMem_bank, 0x00, 0x10000 >> 16, 0, CE_MEMBANK_CHIP16, CACHE_ENABLE_BOTH);
+		map_banks_ce(&STmem_bank, 0x10000 >> 16, ( STmem_size - 0x10000 ) >> 16, 0, CE_MEMBANK_CHIP16, CACHE_ENABLE_BOTH);
+	}
+}
+
+
+/*
+ * Initialize all the memory banks
+ */
+void memory_init(uae_u32 NewSTMemSize, uae_u32 NewTTMemSize, uae_u32 NewRomMemStart)
 {
     int 	addr;
 
     last_address_space_24 = currprefs.address_space_24;
 
-    STmem_size = (nNewSTMemSize + 65535) & 0xFFFF0000;
-    TTmem_size = (nNewTTMemSize + 65535) & 0xFFFF0000;
+    /* Round to next multiple of 65536 bytes */
+    STmem_size = (NewSTMemSize + 65535) & 0xFFFF0000;
+    TTmem_size = (NewTTMemSize + 65535) & 0xFFFF0000;
 
+//fprintf ( stderr , "memory_init: STmem_size=$%x, TTmem_size=$%x, ROM-Start=$%x,\n", STmem_size, TTmem_size, NewRomMemStart);
     /*write_log("memory_init: STmem_size=$%x, TTmem_size=$%x, ROM-Start=$%x,\n",
-              STmem_size, TTmem_size, nNewRomMemStart);*/
+              STmem_size, TTmem_size, NewRomMemStart);*/
 
 #if ENABLE_SMALL_MEM
 
@@ -1342,19 +1659,21 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
     SysMem_bank.mask = STmem_mask;
     SysMem_bank.start = STmem_start;
 
+    STmem_bank_MMU.baseaddr = STmemory;
+    STmem_bank_MMU.mask = STmem_mask;
+    STmem_bank_MMU.start = STmem_start;
+
+    SysMem_bank_MMU.baseaddr = STmemory;
+    SysMem_bank_MMU.mask = STmem_mask;
+    SysMem_bank_MMU.start = STmem_start;
+
     dummy_bank.baseaddr = NULL;				/* No real memory allocated for this region */
     VoidMem_bank.baseaddr = NULL;			/* No real memory allocated for this region */
     BusErrMem_bank.baseaddr = NULL;			/* No real memory allocated for this region */
 
 
-    /* Map the ST system RAM: */
-    map_banks_ce(&SysMem_bank, 0x00, 1, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_CACHABLE);
-    /* Between STRamEnd and 4MB barrier, there is void space: */
-    map_banks_ce(&VoidMem_bank, 0x08, 0x38, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
-    /* Space between 4MB barrier and TOS ROM causes a bus error: */
-    map_banks_ce(&BusErrMem_bank, 0x400000 >> 16, 0xA0, 0 , CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
-    /* Now map main ST RAM, overwriting the void and bus error regions if necessary: */
-    map_banks_ce(&STmem_bank, 0x01, (STmem_size >> 16) - 1, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_CACHABLE);
+    /* Map the standard RAM (Max is 4 MB on unmodified STF/STE) */
+    memory_map_Standard_RAM ( MMU_Bank0_Size , MMU_Bank1_Size );
 
 
     /* Handle extra RAM on TT and Falcon starting at 0x1000000 and up to 0x80000000 */
@@ -1372,7 +1691,7 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
 	    if ( TTmemory != NULL )
 	    {
 		/* 32 bit RAM for CPU only + cache/burst allowed */
-		map_banks_ce ( &TTmem_bank, TTmem_start >> 16, TTmem_size >> 16, 0, CE_MEMBANK_FAST32, CE_MEMBANK_CACHABLE_BURST );
+		map_banks_ce ( &TTmem_bank, TTmem_start >> 16, TTmem_size >> 16, 0, CE_MEMBANK_FAST32, CACHE_ENABLE_ALL);
 		TTmem_mask = 0xffffffff;
 		TTmem_bank.baseaddr = TTmemory;
 		TTmem_bank.mask = TTmem_mask;
@@ -1389,14 +1708,14 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
 
     /* ROM memory: */
     /* Depending on which ROM version we are using, the other ROM region is illegal! */
-    if(nNewRomMemStart == 0xFC0000)
+    if(NewRomMemStart == 0xFC0000)
     {
-        map_banks_ce(&ROMmem_bank, 0xFC0000 >> 16, 0x3, 0, CE_MEMBANK_FAST16, CE_MEMBANK_CACHABLE);	/* [NP] tested on real STF, no bus wait from ROM */
+        map_banks_ce(&ROMmem_bank, 0xFC0000 >> 16, 0x3, 0, CE_MEMBANK_FAST16, CACHE_ENABLE_BOTH);	/* [NP] tested on real STF, no bus wait from ROM */
         map_banks_ce(&BusErrMem_bank, 0xE00000 >> 16, 0x10, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
     }
-    else if(nNewRomMemStart == 0xE00000)
+    else if(NewRomMemStart == 0xE00000)
     {
-        map_banks_ce(&ROMmem_bank, 0xE00000 >> 16, 0x10, 0, CE_MEMBANK_FAST16, CE_MEMBANK_CACHABLE);	/* [NP] tested on real STF, no bus wait from ROM */
+        map_banks_ce(&ROMmem_bank, 0xE00000 >> 16, 0x10, 0, CE_MEMBANK_FAST16, CACHE_ENABLE_BOTH);	/* [NP] tested on real STF, no bus wait from ROM */
         map_banks_ce(&BusErrMem_bank, 0xFC0000 >> 16, 0x3, 0, CE_MEMBANK_CHIP16, CE_MEMBANK_NOT_CACHABLE);
     }
     else
@@ -1405,7 +1724,7 @@ void memory_init(uae_u32 nNewSTMemSize, uae_u32 nNewTTMemSize, uae_u32 nNewRomMe
     }
 
     /* Cartridge memory: */
-    map_banks_ce(&ROMmem_bank, 0xFA0000 >> 16, 0x2, 0, CE_MEMBANK_FAST16, CE_MEMBANK_CACHABLE);		/* [NP] tested on real STF, no bus wait from cartridge */
+    map_banks_ce(&ROMmem_bank, 0xFA0000 >> 16, 0x2, 0, CE_MEMBANK_FAST16, CACHE_ENABLE_BOTH);		/* [NP] tested on real STF, no bus wait from cartridge */
     ROMmem_bank.baseaddr = ROMmemory;
     ROMmem_bank.mask = ROMmem_mask;
     ROMmem_bank.start = ROMmem_start;
@@ -1485,7 +1804,7 @@ static void map_banks2 (addrbank *bank, int start, int size, int realsize, int q
 #else
 	int bnr;
 	unsigned long int hioffs = 0, endhioffs = 0x100;
-	uae_u32 realstart = start;
+	uae_u32 realstart __attribute__((unused)) = start;
 #endif
 
 //printf ( "map %x %x 24=%d\n" , start<<16 , size<<16 , currprefs.address_space_24 );
@@ -1493,7 +1812,7 @@ static void map_banks2 (addrbank *bank, int start, int size, int realsize, int q
 	if (quick <= 0)
 		old = debug_bankchange (-1);
 #endif
-	flush_icache_hard (0, 3); /* Sure don't want to keep any old mappings around! */
+	flush_icache_hard (3); /* Sure don't want to keep any old mappings around! */
 #ifdef NATMEM_OFFSET
 	if (!quick)
 		delete_shmmaps (start << 16, size << 16);
