@@ -26,7 +26,7 @@
 #define M68000_EXC_SRC_INT_DSP	4	/* DSP interrupt exception */
 
 
-/* Special flags */
+/* Special flags (from custom.h) */
 #define SPCFLAG_DEBUGGER 1
 #define SPCFLAG_STOP 2
 #define SPCFLAG_BUSERROR 4
@@ -41,9 +41,7 @@
 #define SPCFLAG_MODE_CHANGE 0x800
 #define SPCFLAG_DSP 0x1000
 
-#ifdef WITH_SOFTFLOAT
-#include <softfloat.h>
-#endif
+#include <softfloat/softfloat.h>
 
 #ifndef SET_CFLG
 
@@ -151,14 +149,17 @@ struct cache030
 	uae_u32 data[4];
 	bool valid[4];
 	uae_u32 tag;
+	uae_u8 fc;
 };
 
 #define CACHESETS040 64
+#define CACHESETS060 128
 #define CACHELINES040 4
 struct cache040
 {
 	uae_u32 data[CACHELINES040][4];
 	bool dirty[CACHELINES040][4];
+	bool gdirty[CACHELINES040];
 	bool valid[CACHELINES040];
 	uae_u32 tag[CACHELINES040];
 };
@@ -172,10 +173,8 @@ extern struct mmufixup mmufixup[2];
 
 typedef struct
 {
-	fptype fp;
-#ifdef WITH_SOFTFLOAT
 	floatx80 fpx;
-#endif
+	fptype fp;
 } fpdata;
 
 struct regstruct
@@ -212,15 +211,17 @@ struct regstruct
 
 #ifdef FPUEMU
 	fpdata fp[8];
+#ifdef JIT
 	fpdata fp_result;
-	uae_u32 fp_result_status;
+#endif
 	uae_u32 fpcr, fpsr, fpiar;
 	uae_u32 fpu_state;
 	uae_u32 fpu_exp_state;
-	fpdata exp_src1, exp_src2;
-	uae_u32 exp_pack[3];
-	uae_u16 exp_opcode, exp_extra, exp_type;
-	uae_u16 exp_size;
+	uae_u16 fp_opword;
+	uaecptr fp_ea;
+	uae_u32 fp_exp_pend, fp_unimp_pend;
+	bool fpu_exp_pre;
+	bool fp_unimp_ins;
 	bool fp_exception;
 	bool fp_branch;
 #endif
@@ -241,14 +242,23 @@ struct regstruct
 	uae_u32 pcr;
 	uae_u32 address_space_mask;
 
-	uae_u32 prefetch020[CPU_PIPELINE_MAX];
+	uae_u16 prefetch020[CPU_PIPELINE_MAX];
+	uae_u8 prefetch020_valid[CPU_PIPELINE_MAX];
 	uae_u32 prefetch020addr;
 	uae_u32 cacheholdingdata020;
 	uae_u32 cacheholdingaddr020;
+	uae_u8 cacheholdingdata_valid;
 	int pipeline_pos;
 	int pipeline_r8[2];
 	int pipeline_stop;
-	int ce020memcycles;
+	uae_u8 fc030;
+
+	uae_u32 prefetch040[CPU_PIPELINE_MAX];
+
+	int ce020endcycle;
+	int ce020startcycle;
+	int ce020prefetchendcycle;
+
 	int ce020extracycles;
 	bool ce020memcycle_data;
 	int ce020_tail;
@@ -275,11 +285,15 @@ struct cputracestruct
 
 	uae_u32 msp, vbr;
 	uae_u32 cacr, caar;
-	uae_u32 prefetch020[CPU_PIPELINE_MAX];
+	uae_u16 prefetch020[CPU_PIPELINE_MAX];
+	uae_u8 prefetch020_valid[CPU_PIPELINE_MAX];
 	uae_u32 prefetch020addr;
 	uae_u32 cacheholdingdata020;
 	uae_u32 cacheholdingaddr020;
 	struct cache020 caches020[CACHELINES020];
+	int pipeline_pos;
+	int pipeline_r8[2];
+	int pipeline_stop;
 
 	uae_u32 startcycles;
 	int needendcycles;
@@ -353,6 +367,23 @@ extern void(*x_cp_put_long)(uaecptr addr, uae_u32 v);
 extern uae_u32(*x_cp_next_iword)(void);
 extern uae_u32(*x_cp_next_ilong)(void);
 
+void mem_access_delay_long_write_ce020 (uaecptr addr, uae_u32 v);
+void mem_access_delay_word_write_ce020 (uaecptr addr, uae_u32 v);
+void mem_access_delay_byte_write_ce020 (uaecptr addr, uae_u32 v);
+uae_u32 mem_access_delay_byte_read_ce020 (uaecptr addr);
+uae_u32 mem_access_delay_word_read_ce020 (uaecptr addr);
+uae_u32 mem_access_delay_long_read_ce020 (uaecptr addr);
+uae_u32 mem_access_delay_longi_read_ce020 (uaecptr addr);
+uae_u32 mem_access_delay_wordi_read_ce020 (uaecptr addr);
+
+void mem_access_delay_long_write_c040 (uaecptr addr, uae_u32 v);
+void mem_access_delay_word_write_c040 (uaecptr addr, uae_u32 v);
+void mem_access_delay_byte_write_c040 (uaecptr addr, uae_u32 v);
+uae_u32 mem_access_delay_byte_read_c040 (uaecptr addr);
+uae_u32 mem_access_delay_word_read_c040 (uaecptr addr);
+uae_u32 mem_access_delay_long_read_c040 (uaecptr addr);
+uae_u32 mem_access_delay_longi_read_c040 (uaecptr addr);
+
 extern uae_u32(REGPARAM3 *x_cp_get_disp_ea_020)(uae_u32 base, int idx) REGPARAM;
 
 /* direct (regs.pc_p) access */
@@ -476,12 +507,12 @@ STATIC_INLINE uae_u32 next_iilongi (void)
 STATIC_INLINE void m68k_do_bsri(uaecptr oldpc, uae_s32 offset)
 {
 	m68k_areg(regs, 7) -= 4;
-	put_long(m68k_areg(regs, 7), oldpc);
+	x_put_long(m68k_areg(regs, 7), oldpc);
 	m68k_incpci(offset);
 }
 STATIC_INLINE void m68k_do_rtsi(void)
 {
-	uae_u32 newpc = get_long(m68k_areg(regs, 7));
+	uae_u32 newpc = x_get_long(m68k_areg(regs, 7));
 	m68k_setpci(newpc);
 	m68k_areg(regs, 7) += 4;
 }
@@ -545,8 +576,37 @@ STATIC_INLINE void m68k_setpc_normal(uaecptr pc)
 	}
 }
 
-extern void write_dcache030(uaecptr, uae_u32, int);
-extern uae_u32 read_dcache030(uaecptr, int);
+extern uae_u32(*read_data_030_bget)(uaecptr);
+extern uae_u32(*read_data_030_wget)(uaecptr);
+extern uae_u32(*read_data_030_lget)(uaecptr);
+extern void(*write_data_030_bput)(uaecptr,uae_u32);
+extern void(*write_data_030_wput)(uaecptr,uae_u32);
+extern void(*write_data_030_lput)(uaecptr,uae_u32);
+
+extern uae_u32(*read_data_030_fc_bget)(uaecptr, uae_u32);
+extern uae_u32(*read_data_030_fc_wget)(uaecptr, uae_u32);
+extern uae_u32(*read_data_030_fc_lget)(uaecptr, uae_u32);
+extern void(*write_data_030_fc_bput)(uaecptr, uae_u32, uae_u32);
+extern void(*write_data_030_fc_wput)(uaecptr, uae_u32, uae_u32);
+extern void(*write_data_030_fc_lput)(uaecptr, uae_u32, uae_u32);
+
+extern void write_dcache030_bput(uaecptr, uae_u32, uae_u32);
+extern void write_dcache030_wput(uaecptr, uae_u32, uae_u32);
+extern void write_dcache030_lput(uaecptr, uae_u32, uae_u32);
+extern uae_u32 read_dcache030_bget(uaecptr, uae_u32);
+extern uae_u32 read_dcache030_wget(uaecptr, uae_u32);
+extern uae_u32 read_dcache030_lget(uaecptr, uae_u32);
+
+extern void write_dcache030_mmu_bput(uaecptr, uae_u32);
+extern void write_dcache030_mmu_wput(uaecptr, uae_u32);
+extern void write_dcache030_mmu_lput(uaecptr, uae_u32);
+extern uae_u32 read_dcache030_mmu_bget(uaecptr);
+extern uae_u32 read_dcache030_mmu_wget(uaecptr);
+extern uae_u32 read_dcache030_mmu_lget(uaecptr);
+extern void write_dcache030_lrmw_mmu(uaecptr, uae_u32, uae_u32);
+extern uae_u32 read_dcache030_lrmw_mmu(uaecptr, uae_u32);
+
+extern void check_t0_trace(void);
 extern uae_u32 get_word_icache030(uaecptr addr);
 extern uae_u32 get_long_icache030(uaecptr addr);
 
@@ -567,6 +627,9 @@ extern uae_u32 get_long_icache040(uaecptr addr);
 extern void (*x_do_cycles)(unsigned long);
 extern void (*x_do_cycles_pre)(unsigned long);
 extern void (*x_do_cycles_post)(unsigned long, uae_u32);
+#ifdef WINUAE_FOR_HATARI
+void set_x_funcs_hatari_blitter (int flag);
+#endif
 
 extern uae_u32 REGPARAM3 x_get_disp_ea_020 (uae_u32 base, int idx) REGPARAM;
 extern uae_u32 REGPARAM3 x_get_disp_ea_ce020 (uae_u32 base, int idx) REGPARAM;
@@ -577,6 +640,7 @@ extern void REGPARAM3 x_put_bitfield (uae_u32 dst, uae_u32 bdata[2], uae_u32 val
 
 extern void m68k_setstopped (void);
 extern void m68k_resumestopped (void);
+extern void m68k_cancel_idle(void);
 
 extern uae_u32 REGPARAM3 get_disp_ea_020 (uae_u32 base, int idx) REGPARAM;
 extern uae_u32 REGPARAM3 get_bitfield (uae_u32 src, uae_u32 bdata[2], uae_s32 offset, int width) REGPARAM;
@@ -586,17 +650,23 @@ extern void m68k_disasm_ea (uaecptr addr, uaecptr *nextpc, int cnt, uae_u32 *sea
 extern void m68k_disasm (uaecptr addr, uaecptr *nextpc, int cnt);
 extern void m68k_disasm_2 (TCHAR *buf, int bufsize, uaecptr addr, uaecptr *nextpc, int cnt, uae_u32 *seaddr, uae_u32 *deaddr, int safemode);
 extern void sm68k_disasm (TCHAR*, TCHAR*, uaecptr addr, uaecptr *nextpc);
+extern int m68k_asm(TCHAR *buf, uae_u16 *out, uaecptr pc);
 extern int get_cpu_model (void);
 #ifdef WINUAE_FOR_HATARI
 extern void m68k_disasm_file (FILE *f, uaecptr addr, uaecptr *nextpc, int cnt);
 #endif
 
 extern void set_cpu_caches (bool flush);
+#ifdef WINUAE_FOR_HATARI
+extern void invalidate_cpu_data_caches(void);
+#endif
 extern void flush_cpu_caches(bool flush);
 extern void flush_cpu_caches_040(uae_u16 opcode);
 extern void REGPARAM3 MakeSR (void) REGPARAM;
-extern void REGPARAM3 MakeFromSR (void) REGPARAM;
+extern void REGPARAM3 MakeFromSR(void) REGPARAM;
+extern void REGPARAM3 MakeFromSR_T0(void) REGPARAM;
 extern void REGPARAM3 Exception (int) REGPARAM;
+extern void REGPARAM3 Exception_cpu(int) REGPARAM;
 extern void REGPARAM3 ExceptionL (int, uaecptr) REGPARAM;
 extern void NMI (void);
 extern void NMI_delayed (void);
@@ -608,16 +678,16 @@ extern int m68k_movec2 (int, uae_u32 *);
 extern bool m68k_divl (uae_u32, uae_u32, uae_u16);
 extern bool m68k_mull (uae_u32, uae_u32, uae_u16);
 extern void init_m68k (void);
-extern void init_m68k_full (void);
 extern void m68k_go (int);
 extern void m68k_dumpstate (uaecptr *);
 extern void m68k_dumpstate_2 (uaecptr, uaecptr *);
 extern void m68k_dumpstate_file (FILE *f, uaecptr *);
-extern void m68k_dumpcache (void);
+extern void m68k_dumpcache (bool);
 extern int getDivu68kCycles (uae_u32 dividend, uae_u16 divisor);
 extern int getDivs68kCycles (uae_s32 dividend, uae_s16 divisor);
-extern void divbyzero_special (bool issigned, uae_s32 dst);
-extern void m68k_do_rte (void);
+extern void divbyzero_special(bool issigned, uae_s32 dst);
+extern void setdivuoverflowflags(uae_u32 dividend, uae_u16 divisor);
+extern void setdivsoverflowflags(uae_s32 dividend, uae_s16 divisor);
 extern void protect_roms (bool);
 extern void unprotect_maprom (void);
 extern bool is_hardreset(void);
@@ -654,12 +724,11 @@ extern void cpu_change(int newmodel);
 extern void cpu_fallback(int mode);
 
 extern void fill_prefetch (void);
-extern void fill_prefetch_020 (void);
-extern void fill_prefetch_030 (void);
-
-#ifdef WINUAE_FOR_HATARI
-//extern void m68k_reset (bool hardreset);	// TODO remove
-#endif
+extern void fill_prefetch_020_ntx(void);
+extern void fill_prefetch_030_ntx(void);
+extern void fill_prefetch_030_ntx_continue(void);
+extern void fill_prefetch_020(void);
+extern void fill_prefetch_030(void);
 
 #define CPU_OP_NAME(a) op ## a
 
@@ -682,6 +751,8 @@ extern const struct cputbl op_smalltbl_52_ff[];
 extern const struct cputbl op_smalltbl_22_ff[]; // prefetch
 extern const struct cputbl op_smalltbl_23_ff[]; // CE
 extern const struct cputbl op_smalltbl_32_ff[]; // MMU
+extern const struct cputbl op_smalltbl_34_ff[]; // MMU + cache
+extern const struct cputbl op_smalltbl_35_ff[]; // MMU + CE + cache
 /* 68020 */
 extern const struct cputbl op_smalltbl_3_ff[];
 extern const struct cputbl op_smalltbl_43_ff[];
@@ -704,15 +775,14 @@ extern const struct cputbl op_smalltbl_14_ff[]; // CE
 extern cpuop_func *cpufunctbl[65536] ASM_SYM_FOR_FUNC ("cpufunctbl");
 
 #ifdef JIT
-extern void flush_icache(uaecptr, int);
-extern void flush_icache_hard(uaecptr, int);
+extern void flush_icache(int);
+extern void flush_icache_hard(int);
 extern void compemu_reset(void);
 #else
-#define flush_icache(uaecptr, int) do {} while (0)
-#define flush_icache_hard(uaecptr, int) do {} while (0)
+#define flush_icache(int) do {} while (0)
+#define flush_icache_hard(int) do {} while (0)
 #endif
 bool check_prefs_changed_comp (bool);
-extern void flush_dcache (uaecptr, int);
 #ifdef WINUAE_FOR_HATARI
 extern void flush_instr_cache (uaecptr, int);
 #endif
@@ -742,12 +812,26 @@ extern bool can_cpu_tracer (void);
 #define CPU_HALT_PCI_CONFLICT 8
 #define CPU_HALT_CPU_STUCK 9
 #define CPU_HALT_SSP_IN_NON_EXISTING_ADDRESS 10
+#define CPU_HALT_INVALID_START_ADDRESS 11
 
 void cpu_semaphore_get(void);
 void cpu_semaphore_release(void);
 bool execute_other_cpu(int until);
 void execute_other_cpu_single(void);
 
+
+/* From uae.h */
+#define UAE_QUIT 1
+#define UAE_RESET 2
+#define UAE_RESET_KEYBOARD 3
+#define UAE_RESET_HARD 4
+
+extern int uae_quit_program;
+/* From uae.h */
+
+
+
+
 #ifdef WINUAE_FOR_HATARI
 /*** Hatari ***/
 
