@@ -17,7 +17,7 @@
 
 
 #define UAEMAJOR 3
-#define UAEMINOR 4
+#define UAEMINOR 6
 #define UAESUBREV 0
 
 typedef enum { KBD_LANG_US, KBD_LANG_DK, KBD_LANG_DE, KBD_LANG_SE, KBD_LANG_FR, KBD_LANG_IT, KBD_LANG_ES } KbdLang;
@@ -52,6 +52,17 @@ struct strlist {
 
 #define INTERNALEVENT_COUNT 1
 
+#if 0
+struct uae_input_device_event
+{
+	uae_s16 eventid[MAX_INPUT_SUB_EVENT_ALL];
+	TCHAR *custom[MAX_INPUT_SUB_EVENT_ALL];
+	uae_u64 flags[MAX_INPUT_SUB_EVENT_ALL];
+	uae_u8 port[MAX_INPUT_SUB_EVENT_ALL];
+	uae_s16 extra;
+};
+#endif
+
 struct uae_input_device {
 	TCHAR *name;
 	TCHAR *configname;
@@ -81,6 +92,7 @@ struct jport {
 	int autofire;
 	struct inputdevconfig idc;
 	bool nokeyboardoverride;
+	bool changed;
 };
 #define JPORT_UNPLUGGED -2
 #define JPORT_NONE -1
@@ -103,6 +115,11 @@ struct jport {
 #define CONFIG_TYPE_NORESET 4
 #define CONFIG_BLEN 2560
 
+#define MOUSEUNTRAP_NONE 0
+#define MOUSEUNTRAP_MIDDLEBUTTON 1
+#define MOUSEUNTRAP_MAGIC 2
+#define MOUSEUNTRAP_BOTH 3
+
 #define TABLET_OFF 0
 #define TABLET_MOUSEHACK 1
 #define TABLET_REAL 2
@@ -175,8 +192,10 @@ struct uaedev_config_info {
 	TCHAR volname[MAX_DPATH];
 	TCHAR rootdir[MAX_DPATH];
 	bool readonly;
+	bool lock;
 	int bootpri;
 	TCHAR filesys[MAX_DPATH];
+	TCHAR geometry[MAX_DPATH];
 	int lowcyl;
 	int highcyl; // zero if detected from size
 	int cyls; // calculated/corrected highcyl
@@ -184,6 +203,7 @@ struct uaedev_config_info {
 	int sectors;
 	int reserved;
 	int blocksize;
+	uae_u64 max_lba;
 	int controller_type;
 	int controller_type_unit;
 	int controller_unit;
@@ -251,6 +271,8 @@ enum { CP_GENERIC = 1, CP_CDTV, CP_CDTVCR, CP_CD32, CP_A500, CP_A500P, CP_A600,
 #define MONITOREMU_AVIDEO24 8
 #define MONITOREMU_FIRECRACKER24 9
 #define MONITOREMU_DCTV 10
+#define MONITOREMU_OPALVISION 11
+#define MONITOREMU_COLORBURST 12
 
 #define MAX_FILTERSHADERS 4
 
@@ -332,6 +354,7 @@ struct gfx_filterdata
 
 #define MAX_DUPLICATE_EXPANSION_BOARDS 4
 #define MAX_EXPANSION_BOARDS 20
+#define ROMCONFIG_CONFIGTEXT_LEN 256
 struct boardromconfig;
 struct romconfig
 {
@@ -343,6 +366,10 @@ struct romconfig
 	int device_settings;
 	int subtype;
 	void *unitdata;
+	TCHAR configtext[ROMCONFIG_CONFIGTEXT_LEN];
+	uae_u16 manufacturer;
+	uae_u8 product;
+	uae_u8 autoconfig[16];
 	struct boardromconfig *back;
 };
 #define MAX_BOARD_ROMS 2
@@ -369,7 +396,12 @@ struct ramboard
 	uae_u8 product;
 	uae_u8 autoconfig[16];
 	bool autoconfig_inuse;
+	bool manual_config;
+	bool no_reset_unmap;
 	int device_order;
+	uae_u32 start_address;
+	uae_u32 end_address;
+	uae_u32 write_address;
 };
 struct expansion_params
 {
@@ -428,6 +460,8 @@ struct uae_prefs {
 	int sound_volume_paula;
 	int sound_volume_cd;
 	int sound_volume_board;
+	int sound_volume_midi;
+	int sound_volume_genlock;
 	bool sound_stereo_swap_paula;
 	bool sound_stereo_swap_ahi;
 	bool sound_auto;
@@ -448,7 +482,6 @@ struct uae_prefs {
 	bool comp_constjump;
 	int cachesize;
 	bool fpu_strict;
-
 	bool fpu_softfloat;
 
 #ifndef WINUAE_FOR_HATARI
@@ -481,6 +514,8 @@ struct uae_prefs {
 	bool lightboost_strobo;
 	int lightboost_strobo_ratio;
 	bool gfx_grayscale;
+	bool lightpen_crosshair;
+	int lightpen_offset[2];
 
 	struct gfx_filterdata gf[2];
 
@@ -489,14 +524,20 @@ struct uae_prefs {
 
 	bool immediate_blits;
 	int waiting_blits;
+	double blitter_speed_throttle;
 	unsigned int chipset_mask;
+	bool keyboard_connected;
 #endif
 	bool ntscmode;
 #ifndef WINUAE_FOR_HATARI
 	bool genlock;
 	int genlock_image;
 	int genlock_mix;
+	int genlock_scale;
+	int genlock_aspect;
+	bool genlock_alpha;
 	TCHAR genlock_image_file[MAX_DPATH];
+	TCHAR genlock_video_file[MAX_DPATH];
 	int monitoremu;
 	double chipset_refreshrate;
 	struct chipset_refresh cr[MAX_CHIPSET_REFRESH + 2];
@@ -583,7 +624,11 @@ struct uae_prefs {
 	bool cs_cia6526;
 	bool cs_bytecustomwritebug;
 	bool cs_color_burst;
+	bool cs_romisslow;
+	bool cs_toshibagary;
+	int cs_unmapped_space;
 	int cs_hacks;
+	int cs_ciatype[2];
 
 	struct boardromconfig expansionboard[MAX_EXPANSION_BOARDS];
 
@@ -624,6 +669,7 @@ struct uae_prefs {
 	int cpu_level;				/* Hatari */
 	int cpu_model;
 	int mmu_model;
+	bool mmu_ec;
 	int cpu060_revision;
 	int fpu_model;
 	int fpu_revision;
@@ -634,6 +680,7 @@ struct uae_prefs {
 	bool int_no_unimplemented;
 	bool fpu_no_unimplemented;
 	bool address_space_24;
+	bool cpu_data_cache;
 	bool picasso96_nocustom;
 	int picasso96_modeflags;
 	int cpu_model_fallback;
@@ -663,6 +710,7 @@ struct uae_prefs {
 	uae_u32 custom_memory_sizes[MAX_CUSTOM_MEMORY_ADDRS];
 	uae_u32 custom_memory_mask[MAX_CUSTOM_MEMORY_ADDRS];
 	int uaeboard;
+	bool uaeboard_nodiag;
 	int uaeboard_order;
 
 	bool kickshifter;
@@ -696,7 +744,6 @@ struct uae_prefs {
 
 	/* Target specific options */
 
-	bool win32_middle_mouse;
 	bool win32_logfile;
 	bool win32_notaskbarbutton;
 	bool win32_nonotificationicon;
@@ -775,7 +822,7 @@ struct uae_prefs {
 	int input_mouse_speed;
 	int input_tablet;
 	bool tablet_library;
-	bool input_magic_mouse;
+	int input_mouse_untrap;
 	int input_magic_mouse_cursor;
 	int input_keyboard_type;
 	int input_autoswitch;
@@ -844,12 +891,14 @@ extern void cfgfile_save_options (struct zfile *f, struct uae_prefs *p, int type
 extern int target_get_display (const TCHAR*);
 extern const TCHAR *target_get_display_name (int, bool);
 
+extern struct uae_prefs *cfgfile_open(const TCHAR *filename, int *type);
+extern void cfgfile_close(struct uae_prefs *p);
 extern int cfgfile_load (struct uae_prefs *p, const TCHAR *filename, int *type, int ignorelink, int userconfig);
 extern int cfgfile_save (struct uae_prefs *p, const TCHAR *filename, int);
 extern void cfgfile_parse_line (struct uae_prefs *p, TCHAR *, int);
 extern void cfgfile_parse_lines (struct uae_prefs *p, const TCHAR *, int);
 extern int cfgfile_parse_option (struct uae_prefs *p, const TCHAR *option, TCHAR *value, int);
-extern int cfgfile_get_description (const TCHAR *filename, TCHAR *description, TCHAR *hostlink, TCHAR *hardwarelink, int *type);
+extern int cfgfile_get_description (struct uae_prefs *p, const TCHAR *filename, TCHAR *description, TCHAR *hostlink, TCHAR *hardwarelink, int *type);
 extern void cfgfile_show_usage (void);
 extern int cfgfile_searchconfig(const TCHAR *in, int index, TCHAR *out, int outsize);
 extern uae_u32 cfgfile_uaelib(TrapContext *ctx, int mode, uae_u32 name, uae_u32 dst, uae_u32 maxlen);
@@ -864,9 +913,9 @@ extern int cfgfile_configuration_change (int);
 extern void fixup_prefs_dimensions (struct uae_prefs *prefs);
 extern void fixup_prefs (struct uae_prefs *prefs, bool userconfig);
 extern void fixup_cpu (struct uae_prefs *prefs);
-extern bool cfgfile_board_enabled(struct uae_prefs *p, int romtype, int devnum);
 extern void cfgfile_compatibility_romtype(struct uae_prefs *p);
 extern void cfgfile_compatibility_rtg(struct uae_prefs *p);
+extern bool cfgfile_detect_art(struct uae_prefs *p, TCHAR *path);
 
 extern void check_prefs_changed_custom (void);
 extern void check_prefs_changed_cpu (void);
