@@ -52,6 +52,7 @@ const char CycInt_fileid[] = "Hatari cycInt.c : " __DATE__ " " __TIME__;
 #include <assert.h>
 
 #include "main.h"
+#include "configuration.h"
 #include "blitter.h"
 #include "dmaSnd.h"
 #include "crossbar.h"
@@ -342,31 +343,6 @@ void CycInt_AddRelativeInterrupt(int CycleTime, int CycleType, interrupt_id Hand
 }
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Add interrupt to occur from now without offset
- */
-#if 0
-void CycInt_AddRelativeInterruptNoOffset(int CycleTime, int CycleType, interrupt_id Handler)
-{
-	/* Update list cycle counts before adding a new one, */
-	/* since CycInt_SetNewInterrupt can change the active int / PendingInterruptCount */
-	if ( ( ActiveInterrupt > 0 ) && ( PendingInterruptCount > 0 ) )
-		CycInt_UpdateInterrupt();
-
-//  nCyclesOver = 0;
-	InterruptHandlers[Handler].bUsed = true;
-	InterruptHandlers[Handler].Cycles = INT_CONVERT_TO_INTERNAL((Sint64)CycleTime , CycleType) + PendingInterruptCount;
-
-	/* Set new */
-	CycInt_SetNewInterrupt();
-
-	LOG_TRACE(TRACE_INT, "int add rel no_off video_cyc=%d handler=%d handler_cyc=%"PRId64" pending_count=%d\n",
-	               Cycles_GetCounter(CYCLES_COUNTER_VIDEO), Handler, InterruptHandlers[Handler].Cycles, PendingInterruptCount );
-}
-#endif
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Add interrupt to occur after CycleTime/CycleType + CycleOffset.
@@ -378,6 +354,7 @@ void CycInt_AddRelativeInterruptNoOffset(int CycleTime, int CycleType, interrupt
  */
 void CycInt_AddRelativeInterruptWithOffset(int CycleTime, int CycleType, interrupt_id Handler, int CycleOffset)
 {
+//fprintf ( stderr , "int add rel %d type %d handler %d offset %d\n" , CycleTime,CycleType,Handler,CycleOffset );
 	assert(CycleTime >= 0);
 
 	/* Update list cycle counts with current PendingInterruptCount before adding a new int, */
@@ -478,6 +455,16 @@ bool CycInt_InterruptActive(interrupt_id Handler)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Return the number of the active interrupt (0 means no active int)
+ */
+int CycInt_GetActiveInt(void)
+{
+	return ActiveInterrupt;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Return cycles passed for an interrupt handler
