@@ -45,11 +45,6 @@ const char BreakCond_fileid[] = "Hatari breakcond.c : " __DATE__ " " __TIME__;
 
 #define BC_DEFAULT_DSP_SPACE 'P'
 
-static inline bool is_register_type(value_t vtype) {
-	/* type used for CPU/DSP registers */
-	return (vtype == VALUE_TYPE_REG16 || vtype == VALUE_TYPE_REG32);
-}
-
 typedef struct {
 	bool is_indirect;
 	char dsp_space;	/* DSP has P, X, Y address spaces, zero if not DSP */
@@ -610,7 +605,8 @@ static bool BreakCond_ParseAddressModifier(parser_state_t *pstate, bc_value_t *b
 		return true;
 	}
 	if (!bc_value->is_indirect) {
-		pstate->error = "space/width modifier makes sense only for an address (register)";
+		pstate->error = "space/width modifier can be used only with an (address) expression\n"
+				"(note that you can use a mask instead of width, for example: 'd0 & 0xff')";
 		EXITFUNC(("arg:%d -> false\n", pstate->arg));
 		return false;
 	}
