@@ -63,6 +63,8 @@ static const char* History_ReasonStr(debug_reason_t reason)
 		return "CPU steps";
 	case REASON_DSP_STEPS:
 		return "DSP steps";
+	case REASON_PROGRAM:
+		return "Program break";
 	case REASON_USER:
 		return "User break";
 	default:
@@ -246,7 +248,7 @@ static void History_Save(const char *name)
 char *History_Match(const char *text, int state)
 {
 	static const char* cmds[] = { "cpu", "dsp", "off", "save" };
-	return DebugUI_MatchHelper(cmds, ARRAYSIZE(cmds), text, state);
+	return DebugUI_MatchHelper(cmds, ARRAY_SIZE(cmds), text, state);
 }
 
 /**
