@@ -12,14 +12,16 @@ const char Profiledsp_fileid[] = "Hatari profiledsp.c : " __DATE__ " " __TIME__;
 
 #include <stdio.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <assert.h>
 #include "main.h"
 #include "configuration.h"
 #include "clocks_timings.h"
 #include "dsp.h"
+#include "symbols.h"
 #include "profile.h"
 #include "profile_priv.h"
-#include "symbols.h"
+#include "debug_priv.h"
 /* for VBL info */
 #include "screen.h"
 #include "video.h"
@@ -116,9 +118,9 @@ void Profile_DspShowStats(void)
  * Show DSP instructions which execution was profiled, in the address order,
  * starting from the given address.  Return next disassembly address.
  */
-Uint16 Profile_DspShowAddresses(Uint32 addr, Uint32 upper, FILE *out)
+Uint16 Profile_DspShowAddresses(Uint32 addr, Uint32 upper, FILE *out, paging_t use_paging)
 {
-	int show, shown, active;
+	int show, shown, addrs, active;
 	dsp_profile_item_t *data;
 	Uint16 nextpc;
 	Uint32 end;
@@ -138,30 +140,37 @@ Uint16 Profile_DspShowAddresses(Uint32 addr, Uint32 upper, FILE *out)
 		}
 		show = active;
 	} else {
-		show = ConfigureParams.Debugger.nDisasmLines;
+		show = DebugUI_GetPageLines(ConfigureParams.Debugger.nDisasmLines, 0);
 		if (!show || show > active) {
 			show = active;
 		}
 	}
+	if (use_paging == PAGING_DISABLED) {
+		show = INT_MAX;
+	}
 
 	fputs("# disassembly with profile data: <instructions percentage>% (<sum of instructions>, <sum of cycles>, <max cycle difference>)\n", out);
+	shown = 2; /* first and last printf */
 
-	nextpc = 0;
-	for (shown = 0; shown < show && addr < end; addr++) {
+	addrs = nextpc = 0;
+	for (; shown < show && addr < end; addr++) {
 		if (!data[addr].count) {
 			continue;
 		}
 		if (addr != nextpc && nextpc) {
 			fputs("[...]\n", out);
+			shown++;
 		}
-		symbol = Symbols_GetByDspAddress(addr);
+		symbol = Symbols_GetByDspAddress(addr, SYMTYPE_TEXT);
 		if (symbol) {
 			fprintf(out, "%s:\n", symbol);
+			shown++;
 		}
 		nextpc = DSP_DisasmAddress(out, addr, addr);
+		addrs++;
 		shown++;
 	}
-	printf("Disassembled %d (of active %d) DSP addresses.\n", shown, active);
+	printf("Disassembled %d (of active %d) DSP addresses.\n", addrs, active);
 	return nextpc;
 }
 
@@ -270,7 +279,7 @@ void Profile_DspShowCounts(int show, bool only_symbols)
 		return;
 	}
 
-	symbols = Symbols_DspCount();
+	symbols = Symbols_DspCodeCount();
 	if (!symbols) {
 		fprintf(stderr, "ERROR: no DSP symbols loaded!\n");
 		return;
@@ -281,7 +290,7 @@ void Profile_DspShowCounts(int show, bool only_symbols)
 	for (end = sort_arr + active; sort_arr < end; sort_arr++) {
 
 		addr = *sort_arr;
-		name = Symbols_GetByDspAddress(addr);
+		name = Symbols_GetByDspAddress(addr, SYMTYPE_TEXT);
 		if (!name) {
 			continue;
 		}
@@ -303,7 +312,7 @@ void Profile_DspShowCounts(int show, bool only_symbols)
 static const char * addr2name(Uint32 addr, Uint64 *total)
 {
 	*total = dsp_profile.data[addr].count;
-	return Symbols_GetByDspAddress(addr);
+	return Symbols_GetByDspAddress(addr, SYMTYPE_TEXT);
 }
 
 /**
@@ -328,7 +337,7 @@ void Profile_DspSave(FILE *out)
 	 * p:0202  0aa980 000200  (07 cyc)  jclr #0,x:$ffe9,p:$0200  0.00% (6, 42)
 	 */
 	fputs("Field regexp:\t^p:([0-9a-f]+) .*% \\((.*)\\)$\n", out);
-	Profile_DspShowAddresses(0, DSP_PROFILE_ARR_SIZE, out);
+	Profile_DspShowAddresses(0, DSP_PROFILE_ARR_SIZE, out, PAGING_DISABLED);
 	Profile_DspShowCallers(out);
 }
 
@@ -365,7 +374,7 @@ bool Profile_DspStart(void)
 	printf("Allocated DSP profile buffer (%d KB).\n",
 	       (int)sizeof(*dsp_profile.data)*DSP_PROFILE_ARR_SIZE/1024);
 
-	Profile_AllocCallinfo(&(dsp_callinfo), Symbols_DspCount(), "DSP");
+	Profile_AllocCallinfo(&(dsp_callinfo), Symbols_DspCodeCount(), "DSP");
 
 	item = dsp_profile.data;
 	for (i = 0; i < DSP_PROFILE_ARR_SIZE; i++, item++) {
@@ -515,7 +524,7 @@ static void collect_calls(Uint16 pc, counters_t *counters)
 	}
 
 	/* address is one which we're tracking? */
-	idx = Symbols_GetDspAddressIndex(pc);
+	idx = Symbols_GetDspCodeIndex(pc);
 	if (unlikely(idx >= 0)) {
 
 		flag = dsp_opcode_type(prev_pc, pc);
@@ -718,7 +727,7 @@ void Profile_DspGetPointers(bool **enabled, Uint32 **disasm_addr)
 /**
  * Get callinfo & symbol search pointers for stack walking.
  */
-void Profile_DspGetCallinfo(callinfo_t **callinfo, const char* (**get_symbol)(Uint32))
+void Profile_DspGetCallinfo(callinfo_t **callinfo, const char* (**get_symbol)(Uint32, symtype_t))
 {
 	*callinfo = &(dsp_callinfo);
 	*get_symbol = Symbols_GetByDspAddress;
