@@ -1122,22 +1122,16 @@ void Crossbar_Recalculate_Clocks_Cycles(void)
 	crossbar.clock32_cycles_counter = 0;
 
 	/* Calculate 25 Mhz clock cycles */
-#ifdef OLD_CPU_SHIFT
-	cyclesClk = ((double)CPU_FREQ / Crossbar_DetectSampleRate(25)) / (double)(crossbar.playTracks) / 2.0;
-#else
 	/* Take nCpuFreqShift into account to keep a constant sound rate at all cpu freq */
 	cyclesClk = ((double)( ( CPU_FREQ << nCpuFreqShift ) ) / Crossbar_DetectSampleRate(25)) / (double)(crossbar.playTracks) / 2.0;
-#endif
+
 	crossbar.clock25_cycles = (int)(cyclesClk);
 	crossbar.clock25_cycles_decimal = (int)((cyclesClk - (double)(crossbar.clock25_cycles)) * (double)DECIMAL_PRECISION);
 
 	/* Calculate 32 Mhz clock cycles */
-#ifdef OLD_CPU_SHIFT
-	cyclesClk = ((double)CPU_FREQ / Crossbar_DetectSampleRate(32)) / (double)(crossbar.playTracks) / 2.0;
-#else
 	/* Take nCpuFreqShift into account to keep a constant sound rate at all cpu freq */
 	cyclesClk = ((double)( ( CPU_FREQ << nCpuFreqShift ) ) / Crossbar_DetectSampleRate(32)) / (double)(crossbar.playTracks) / 2.0;
-#endif
+
 	crossbar.clock32_cycles = (int)(cyclesClk);
 	crossbar.clock32_cycles_decimal = (int)((cyclesClk - (double)(crossbar.clock32_cycles)) * (double)DECIMAL_PRECISION);
 
