@@ -30,6 +30,7 @@ const char File_fileid[] = "Hatari file.c : " __DATE__ " " __TIME__;
 #include "createBlankImage.h"
 #include "str.h"
 #include "zip.h"
+#include "log.h"
 
 #ifdef HAVE_FLOCK
 # include <sys/file.h>
@@ -333,7 +334,6 @@ off_t File_Length(const char *pszFileName)
 	{
 		fseeko(hDiskFile, 0, SEEK_END);
 		FileSize = ftello(hDiskFile);
-		fseeko(hDiskFile, 0, SEEK_SET);
 		fclose(hDiskFile);
 		return FileSize;
 	}
@@ -532,6 +532,9 @@ char * File_MakePath(const char *pDir, const char *pName, const char *pExt)
 /**
  * Shrink a file name to a certain length and insert some dots if we cut
  * something away (useful for showing file names in a dialog).
+ * Note: maxlen is the maximum length of the destination string, _not_
+ * including the final '\0' byte! So the destination buffer has to be
+ * at least one byte bigger than maxlen.
  */
 void File_ShrinkName(char *pDestFileName, const char *pSrcFileName, int maxlen)
 {
@@ -923,3 +926,42 @@ void File_HandleDotDirs(char *path)
 		}
 	}
 }
+
+
+#if defined(WIN32)
+static TCHAR szTempFileName[MAX_PATH];
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Get temporary filename for Windows
+ */
+char* WinTmpFile(void)
+{
+	DWORD dwRetVal = 0;
+	UINT uRetVal   = 0;
+	TCHAR lpTempPathBuffer[MAX_PATH];
+
+	/* Gets the temp path env string (no guarantee it's a valid path) */
+	dwRetVal = GetTempPath(MAX_PATH,		/* length of the buffer */
+                           lpTempPathBuffer);		/* buffer for path */
+	if (dwRetVal > MAX_PATH || (dwRetVal == 0))
+	{
+		Log_Printf(LOG_ERROR, "GetTempPath failed.\n");
+		return NULL;
+	}
+
+	/* Generates a temporary file name */
+	uRetVal = GetTempFileName(lpTempPathBuffer,	/* directory for tmp files */
+                              TEXT("HATARI"),		/* temp file name prefix */
+                              0,			/* create unique name */
+                              szTempFileName);		/* buffer for name */
+	if (uRetVal == 0)
+	{
+		Log_Printf(LOG_ERROR, "GetTempFileName failed.\n");
+		return NULL;
+	}
+	return (char*)szTempFileName;
+}
+#endif
+
+
