@@ -42,11 +42,11 @@ const char Gemdos_fileid[] = "Hatari gemdos.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "cart.h"
-#include "tos.h"
 #include "configuration.h"
 #include "file.h"
 #include "floppy.h"
 #include "ide.h"
+#include "inffile.h"
 #include "hdc.h"
 #include "gemdos.h"
 #include "gemdos_defines.h"
@@ -58,6 +58,7 @@ const char Gemdos_fileid[] = "Hatari gemdos.c : " __DATE__ " " __TIME__;
 #include "scandir.h"
 #include "stMemory.h"
 #include "str.h"
+#include "tos.h"
 #include "hatari-glue.h"
 #include "maccess.h"
 #include "symbols.h"
@@ -65,6 +66,10 @@ const char Gemdos_fileid[] = "Hatari gemdos.c : " __DATE__ " " __TIME__;
 /* Maximum supported length of a GEMDOS path: */
 #define MAX_GEMDOS_PATH 256
 
+#define BASEPAGE_SIZE (0x80+0x80)  /* info + command line */
+#define BASEPAGE_OFFSET_DTA 0x20
+#define BASEPAGE_OFFSET_PARENT 0x24
+
 /* Have we re-directed GemDOS vector to our own routines yet? */
 bool bInitGemDOS;
 
@@ -81,10 +86,12 @@ EMULATEDDRIVE **emudrives = NULL;
 #define TOS_NAMELEN  14
 
 typedef struct {
+  /* GEMDOS internals */
   Uint8 index[2];
   Uint8 magic[4];
-  char dta_pat[TOS_NAMELEN];
-  char dta_sattrib;
+  char dta_pat[TOS_NAMELEN]; /* unused */
+  char dta_sattrib;          /* unused */
+  /* TOS API */
   char dta_attrib;
   Uint8 dta_time[2];
   Uint8 dta_date[2];
@@ -94,6 +101,7 @@ typedef struct {
 
 #define DTA_MAGIC_NUMBER  0x12983476
 #define MAX_DTAS_FILES    256      /* Must be ^2 */
+#define MAX_DTAS_MASK     (MAX_DTAS_FILES-1)
 #define CALL_PEXEC_ROUTINE 3       /* Call our cartridge pexec routine */
 
 #define  BASE_FILEHANDLE     64    /* Our emulation handles - MUST not be valid TOS ones, but MUST be <256 */
@@ -135,9 +143,6 @@ typedef struct
 static FILE_HANDLE  FileHandles[MAX_FILE_HANDLES];
 static INTERNAL_DTA InternalDTAs[MAX_DTAS_FILES];
 static int DTAIndex;        /* Circular index into above */
-static Uint32 DTA_Gemdos;   /* DTA address in ST memory space */
-static DTA *pDTA;           /* Our GEMDOS hard drive Disk Transfer Address structure */
-			    /* This a direct pointer to DTA_Gemdos using STMemory_STAddrToPointer() */
 static Uint16 CurrentDrive; /* Current drive (0=A,1=B,2=C etc...) */
 static Uint32 act_pd;       /* Used to get a pointer to the current basepage */
 static Uint16 nAttrSFirst;  /* File attribute for SFirst/Snext */
@@ -219,6 +224,11 @@ static bool GemDOS_SetFileInformation(int Handle, DATETIME *DateTime)
 	 * the file after it's modification time is changed.
 	 */
 	fflush(FileHandles[Handle].FileHandle);
+
+	/* use host modification times instead of Atari ones? */
+	if (ConfigureParams.HardDisk.bGemdosHostTime)
+		return true;
+
 	filename = FileHandles[Handle].szActualName;
 	
 	/* Bits: 0-4 = secs/2, 5-10 = mins, 11-15 = hours (24-hour format) */
@@ -277,7 +287,7 @@ static Uint8 GemDOS_ConvertAttribute(mode_t mode)
  * Populate the DTA buffer with file info.
  * @return   0 if entry is ok, 1 if entry should be skipped, < 0 for errors.
  */
-static int PopulateDTA(char *path, struct dirent *file)
+static int PopulateDTA(char *path, struct dirent *file, DTA *pDTA, Uint32 DTA_Gemdos)
 {
 	/* TODO: host file path can be longer than MAX_GEMDOS_PATH */
 	char tempstr[MAX_GEMDOS_PATH];
@@ -285,7 +295,12 @@ static int PopulateDTA(char *path, struct dirent *file)
 	DATETIME DateTime;
 	int nFileAttr, nAttrMask;
 
-	snprintf(tempstr, sizeof(tempstr), "%s%c%s", path, PATHSEP, file->d_name);
+	if (snprintf(tempstr, sizeof(tempstr), "%s%c%s",
+	             path, PATHSEP, file->d_name) >= (int)sizeof(tempstr))
+	{
+		fprintf(stderr, "PopulateDTA: path is too long.\n");
+		return -1;
+	}
 
 	if (stat(tempstr, &filestat) != 0)
 	{
@@ -324,22 +339,22 @@ static int PopulateDTA(char *path, struct dirent *file)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Clear a used DTA structure.
+ * Clear given DTA structure.
  */
-static void ClearInternalDTA(void)
+static void ClearInternalDTA(int idx)
 {
 	int i;
 
 	/* clear the old DTA structure */
-	if (InternalDTAs[DTAIndex].found != NULL)
+	if (InternalDTAs[idx].found != NULL)
 	{
-		for (i=0; i < InternalDTAs[DTAIndex].nentries; i++)
-			free(InternalDTAs[DTAIndex].found[i]);
-		free(InternalDTAs[DTAIndex].found);
-		InternalDTAs[DTAIndex].found = NULL;
+		for (i = 0; i < InternalDTAs[idx].nentries; i++)
+			free(InternalDTAs[idx].found[i]);
+		free(InternalDTAs[idx].found);
+		InternalDTAs[idx].found = NULL;
 	}
-	InternalDTAs[DTAIndex].nentries = 0;
-	InternalDTAs[DTAIndex].bUsed = false;
+	InternalDTAs[idx].nentries = 0;
+	InternalDTAs[idx].bUsed = false;
 }
 
 
@@ -453,6 +468,23 @@ static void GemDOS_UnforceFileHandle(int i)
 	ForcedHandles[i].Basepage = 0;
 }
 
+/**
+ * Clear & un-force all file handles
+ */
+static void GemDOS_ClearAllFileHandles(void)
+{
+	int i;
+
+	for(i = 0; i < ARRAY_SIZE(FileHandles); i++)
+	{
+		GemDOS_CloseFileHandle(i);
+	}
+	for(i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
+	{
+		GemDOS_UnforceFileHandle(i);
+	}
+}
+
 /*-----------------------------------------------------------------------*/
 
 /**
@@ -479,46 +511,24 @@ void GemDOS_Init(void)
 	int i;
 	bInitGemDOS = false;
 
-	/* Clear handles structure */
-	memset(FileHandles, 0, sizeof(FileHandles));
-	for(i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
-	{
-		GemDOS_UnforceFileHandle(i);
-	}
+	GemDOS_ClearAllFileHandles();
+
 	/* Clear DTAs */
 	for(i = 0; i < ARRAY_SIZE(InternalDTAs); i++)
 	{
-		InternalDTAs[i].bUsed = false;
-		InternalDTAs[i].nentries = 0;
-		InternalDTAs[i].found = NULL;
+		ClearInternalDTA(i);
 	}
 	DTAIndex = 0;
 }
 
-
 /*-----------------------------------------------------------------------*/
 /**
- * Reset GemDOS file system
+ * Initialize GemDOS drives current paths (to drive root)
  */
-void GemDOS_Reset(void)
+static void GemDOS_InitCurPaths(void)
 {
 	int i;
 
-	/* Init file handles table */
-	for (i = 0; i < ARRAY_SIZE(FileHandles); i++)
-	{
-		GemDOS_CloseFileHandle(i);
-	}
-	for(i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
-	{
-		GemDOS_UnforceFileHandle(i);
-	}
-	for (DTAIndex = 0; DTAIndex < MAX_DTAS_FILES; DTAIndex++)
-	{
-		ClearInternalDTA();
-	}
-	DTAIndex = 0;
-
 	if (emudrives)
 	{
 		for (i = 0; i < MAX_HARDDRIVES; i++)
@@ -531,13 +541,22 @@ void GemDOS_Reset(void)
 			}
 		}
 	}
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Reset GemDOS file system
+ */
+void GemDOS_Reset(void)
+{
+	GemDOS_Init();
+	GemDOS_InitCurPaths();
 
 	/* Reset */
-	bInitGemDOS = false;
+	act_pd = 0;
 	CurrentDrive = nBootDrive;
 	Symbols_RemoveCurrentProgram();
-	DTA_Gemdos = 0x0;
-	pDTA = NULL;
+	INF_CreateOverride();
 }
 
 /*-----------------------------------------------------------------------*/
@@ -739,12 +758,19 @@ void GemDOS_InitDrives(void)
 			emudrives[i] = NULL;
 		}
 	}
+
+	/* Set current paths in case Atari -> host GEMDOS path mapping
+	 * is needed before TOS boots GEMDOS up (at which point they're
+	 * also initialized), like happens with autostart INF file
+	 * handling.
+	 */
+	GemDOS_InitCurPaths();
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Un-init the GEMDOS drive
+ * Un-init GEMDOS drives
  */
 void GemDOS_UnInitDrives(void)
 {
@@ -776,7 +802,7 @@ void GemDOS_UnInitDrives(void)
  */
 void GemDOS_MemorySnapShot_Capture(bool bSave)
 {
-	int i;
+	int i, dummy;
 	bool bEmudrivesAvailable;
 
 	/* Save/Restore the emudrives structure */
@@ -826,35 +852,14 @@ void GemDOS_MemorySnapShot_Capture(bool bSave)
 	MemorySnapShot_Store(&DTAIndex,sizeof(DTAIndex));
 	MemorySnapShot_Store(&bInitGemDOS,sizeof(bInitGemDOS));
 	MemorySnapShot_Store(&act_pd, sizeof(act_pd));
-	if (bSave)
-	{
-		/* Store the value in ST memory space */
-		MemorySnapShot_Store ( &DTA_Gemdos , sizeof(DTA_Gemdos) );
-	}
-	else
-	{
-		/* Restore the value in ST memory space and update pDTA */
-		MemorySnapShot_Store ( &DTA_Gemdos , sizeof(DTA_Gemdos) );
-		if ( DTA_Gemdos == 0x0 )
-			pDTA = NULL;
-		else
-			pDTA = (DTA *)STMemory_STAddrToPointer( DTA_Gemdos );
-	}
+	MemorySnapShot_Store(&dummy, sizeof(dummy)); /* redundant DTA_Gemdos */
 	MemorySnapShot_Store(&CurrentDrive,sizeof(CurrentDrive));
 	/* Don't save file handles as files may have changed which makes
 	 * it impossible to get a valid handle back
 	 */
 	if (!bSave)
 	{
-		/* Clear file handles  */
-		for(i = 0; i < ARRAY_SIZE(FileHandles); i++)
-		{
-			GemDOS_CloseFileHandle(i);
-		}
-		for(i = 0; i < ARRAY_SIZE(ForcedHandles); i++)
-		{
-			GemDOS_UnforceFileHandle(i);
-		}
+		GemDOS_ClearAllFileHandles();
 	}
 }
 
@@ -887,11 +892,11 @@ static bool GemDOS_BasepageMatches(Uint32 checkbase)
 {
 	int maxparents = 12; /* prevent basepage parent loops */
 	Uint32 basepage = STMemory_ReadLong(act_pd);
-	while (maxparents-- > 0 && STMemory_CheckAreaType ( basepage, 0x100, ABFLAG_RAM ) )
+	while (maxparents-- > 0 && STMemory_CheckAreaType(basepage, BASEPAGE_SIZE, ABFLAG_RAM))
 	{
 		if (basepage == checkbase)
 			return true;
-		basepage = STMemory_ReadLong(basepage + 0x24);	/* parent */
+		basepage = STMemory_ReadLong(basepage + BASEPAGE_OFFSET_PARENT);
 	}
 	return false;
 }
@@ -1434,34 +1439,6 @@ static bool GemDOS_SetDrv(Uint32 Params)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * GEMDOS Set Disk Transfer Address (DTA)
- * Call 0x1A
- */
-static bool GemDOS_SetDTA(Uint32 Params)
-{
-	/*  Look up on stack to find where DTA is */
-	DTA_Gemdos = STMemory_ReadLong(Params);
-
-	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x1A Fsetdta(0x%x) at PC 0x%X\n", DTA_Gemdos,
-		  M68000_GetPC());
-
-	if ( STMemory_CheckAreaType ( DTA_Gemdos, sizeof(DTA), ABFLAG_RAM ) )
-	{
-		/* Store as PC pointer */
-		pDTA = (DTA *)STMemory_STAddrToPointer(DTA_Gemdos);
-	}
-	else
-	{
-		Log_Printf(LOG_WARN, "GEMDOS Fsetdta() failed due to invalid DTA address 0x%x\n", DTA_Gemdos);
-		DTA_Gemdos = 0x0;
-		pDTA = NULL;
-	}
-	/* redirect to TOS */
-	return false;
-}
-
 /*-----------------------------------------------------------------------*/
 /**
  * GEMDOS Dfree Free disk space.
@@ -1846,7 +1823,10 @@ static bool GemDOS_Create(Uint32 Params)
 		if (Mode & GEMDOS_FILE_ATTRIB_READONLY)
 		{
 			/* after closing, file should be read-only */
-			chmod(szActualFileName, S_IRUSR|S_IRGRP|S_IROTH);
+			if (chmod(szActualFileName, S_IRUSR|S_IRGRP|S_IROTH))
+			{
+				perror("Failed to set file to read-only");
+			}
 		}
 		/* Tag handle table entry as used in this process and return handle */
 		FileHandles[Index].bUsed = true;
@@ -1888,6 +1868,12 @@ static bool GemDOS_Create(Uint32 Params)
 
 
 /*-----------------------------------------------------------------------*/
+static inline bool redirect_to_TOS(void)
+{
+	LOG_TRACE(TRACE_OS_GEMDOS|TRACE_OS_BASE, "-> to TOS\n");
+	return false;
+}
+
 /**
  * GEMDOS Open file
  * Call 0x3D
@@ -1902,7 +1888,8 @@ static bool GemDOS_Open(Uint32 Params)
 		"read-only", "write-only", "read/write", "read/write"
 	};
 	int Drive, Index, Mode;
-	FILE *AutostartHandle;
+	FILE *OverrideHandle;
+	bool bToTos = false;
 
 	/* Find filename */
 	pszFileName = (char *)STMemory_STAddrToPointer(STMemory_ReadLong(Params));
@@ -1917,29 +1904,35 @@ static bool GemDOS_Open(Uint32 Params)
 
 	if (!ISHARDDRIVE(Drive))
 	{
-		/* redirect to TOS */
-		LOG_TRACE(TRACE_OS_GEMDOS|TRACE_OS_BASE, "-> to TOS\n");
-		return false;
+		if (INF_Overriding(AUTOSTART_FOPEN))
+			bToTos = true;
+		else
+			return redirect_to_TOS();
 	}
 
 	/* Find slot to store file handle, as need to return WORD handle for ST  */
 	Index = GemDOS_FindFreeFileHandle();
 	if (Index == -1)
 	{
+		if (bToTos)
+			return redirect_to_TOS();
+
 		/* No free handles, return error code */
 		Regs[REG_D0] = GEMDOS_ENHNDL;       /* No more handles */
 		return true;
 	}
 
-	if ((AutostartHandle = TOS_AutoStartOpen(pszFileName)))
+	if ((OverrideHandle = INF_OpenOverride(pszFileName)))
 	{
 		strcpy(szActualFileName, pszFileName);
-		FileHandles[Index].FileHandle = AutostartHandle;
+		FileHandles[Index].FileHandle = OverrideHandle;
 		RealMode = "read-only";
 	}
 	else
 	{
 		struct stat FileStat;
+		if (bToTos)
+			return redirect_to_TOS();
 
 		/* Convert to hard drive filename */
 		GemDOS_CreateHardDriveFileName(Drive, pszFileName,
@@ -2043,7 +2036,7 @@ static bool GemDOS_Close(Uint32 Params)
 	}
 	
 	/* Close file and free up handle table */
-	if (TOS_AutoStartClose(FileHandles[Handle].FileHandle))
+	if (INF_CloseOverride(FileHandles[Handle].FileHandle))
 	{
 		FileHandles[Handle].bUsed = false;
 	}
@@ -2145,8 +2138,8 @@ static bool GemDOS_Read(Uint32 Params)
 	
 	if (ferror(FileHandles[Handle].FileHandle))
 	{
-		Log_Printf(LOG_WARN, "GEMDOS failed to read from '%s'\n",
-			   FileHandles[Handle].szActualName );
+		Log_Printf(LOG_WARN, "GEMDOS failed to read from '%s': %s\n",
+			   FileHandles[Handle].szActualName, strerror(errno));
 		Regs[REG_D0] = errno2gemdos(errno, ERROR_FILE);
 	} else
 		/* Return number of bytes read */
@@ -2307,7 +2300,7 @@ static bool GemDOS_LSeek(Uint32 Params)
 	nOldPos = ftell(fhndl);
 
 	/* Determine the size of the file */
-	if (fseek(fhndl, 0L, SEEK_END) != 0)
+	if (fseek(fhndl, 0L, SEEK_END) != 0 || nOldPos < 0)
 	{
 		Regs[REG_D0] = GEMDOS_E_SEEK;
 		return true;
@@ -2629,17 +2622,17 @@ static bool GemDOS_SNext(void)
 	struct dirent **temp;
 	int Index;
 	int ret;
+	DTA *pDTA;
+	Uint32 DTA_Gemdos;
 
 	LOG_TRACE(TRACE_OS_GEMDOS, "GEMDOS 0x4F Fsnext() at PC 0x%X\n" , M68000_GetPC());
 
 	/* Refresh pDTA pointer (from the current basepage) */
-	DTA_Gemdos = STMemory_ReadLong(STMemory_ReadLong(act_pd)+32);
+	DTA_Gemdos = STMemory_ReadLong(STMemory_ReadLong(act_pd) + BASEPAGE_OFFSET_DTA);
 
 	if ( !STMemory_CheckAreaType ( DTA_Gemdos, sizeof(DTA), ABFLAG_RAM ) )
 	{
 		Log_Printf(LOG_WARN, "GEMDOS Fsnext() failed due to invalid DTA address 0x%x\n", DTA_Gemdos);
-		DTA_Gemdos = 0x0;
-		pDTA = NULL;
 		Regs[REG_D0] = GEMDOS_EINTRN;    /* "internal error */
 		return true;
 	}
@@ -2653,7 +2646,7 @@ static bool GemDOS_SNext(void)
 	}
 
 	/* Find index into our list of structures */
-	Index = do_get_mem_word(pDTA->index) & (MAX_DTAS_FILES-1);
+	Index = do_get_mem_word(pDTA->index) & MAX_DTAS_MASK;
 
 	if (nAttrSFirst == GEMDOS_FILE_ATTRIB_VOLUME_LABEL)
 	{
@@ -2672,7 +2665,8 @@ static bool GemDOS_SNext(void)
 		}
 
 		ret = PopulateDTA(InternalDTAs[Index].path,
-				  temp[InternalDTAs[Index].centry++]);
+				  temp[InternalDTAs[Index].centry++],
+				  pDTA, DTA_Gemdos);
 	} while (ret == 1);
 
 	if (ret < 0)
@@ -2702,6 +2696,8 @@ static bool GemDOS_SFirst(Uint32 Params)
 	int Drive;
 	DIR *fsdir;
 	int i,j,count;
+	DTA *pDTA;
+	Uint32 DTA_Gemdos;
 
 	/* Find filename to search for */
 	pszFileName = (char *)STMemory_STAddrToPointer(STMemory_ReadLong(Params));
@@ -2722,13 +2718,11 @@ static bool GemDOS_SFirst(Uint32 Params)
 		                    szActualFileName, sizeof(szActualFileName));
 
 	/* Refresh pDTA pointer (from the current basepage) */
-	DTA_Gemdos = STMemory_ReadLong(STMemory_ReadLong(act_pd)+32);
+	DTA_Gemdos = STMemory_ReadLong(STMemory_ReadLong(act_pd) + BASEPAGE_OFFSET_DTA);
 
 	if ( !STMemory_CheckAreaType ( DTA_Gemdos, sizeof(DTA), ABFLAG_RAM ) )
 	{
 		Log_Printf(LOG_WARN, "GEMDOS Fsfirst() failed due to invalid DTA address 0x%x\n", DTA_Gemdos);
-		DTA_Gemdos = 0x0;
-		pDTA = NULL;
 		Regs[REG_D0] = GEMDOS_EINTRN;    /* "internal error */
 		return true;
 	}
@@ -2744,7 +2738,7 @@ static bool GemDOS_SFirst(Uint32 Params)
 	do_put_mem_long(pDTA->magic, DTA_MAGIC_NUMBER);
 
 	if (InternalDTAs[DTAIndex].bUsed == true)
-		ClearInternalDTA();
+		ClearInternalDTA(DTAIndex);
 	InternalDTAs[DTAIndex].bUsed = true;
 
 	/* Were we looking for the volume label? */
@@ -2814,7 +2808,7 @@ static bool GemDOS_SFirst(Uint32 Params)
 	GemDOS_SNext();
 	/* increment DTA index */
 	DTAIndex++;
-	DTAIndex &= (MAX_DTAS_FILES-1);
+	DTAIndex &= MAX_DTAS_MASK;
 
 	return true;
 }
@@ -3155,9 +3149,10 @@ void GemDOS_Info(FILE *fp, Uint32 bShowOpcodes)
 	{
 		if (!emudrives[i])
 			continue;
-		fprintf(fp, "- %c: %s\n",
+		fprintf(fp, "- %c: %s\n  curpath: %s\n",
 			'A' + emudrives[i]->drive_number,
-			emudrives[i]->hd_emulation_dir);
+			emudrives[i]->hd_emulation_dir,
+			emudrives[i]->fs_currpath);
 	}
 
 	fputs("\nInternal Fsfirst() DTAs:\n", fp);
@@ -3209,11 +3204,60 @@ void GemDOS_Info(FILE *fp, Uint32 bShowOpcodes)
 		fputs("- None.\n", fp);
 }
 
+/**
+ * Show given DTA info
+ * (works also without GEMDOS HD emu)
+ */
+void GemDOS_InfoDTA(FILE *fp, Uint32 dta_addr)
+{
+	DTA *dta;
+	Uint32 magic;
+	char name[TOS_NAMELEN+1];
+
+	fprintf(fp, "DTA (0x%x):\n", dta_addr);
+	if (act_pd)
+	{
+		Uint32 basepage = STMemory_ReadLong(act_pd);
+		Uint32 dta_curr = STMemory_ReadLong(basepage + BASEPAGE_OFFSET_DTA);
+		if (dta_addr != dta_curr)
+		{
+			fprintf(fp, "- NOTE: given DTA (0x%x) is not current program one (0x%x)\n",
+				dta_addr, dta_curr);
+		}
+		if (dta_addr >= basepage && dta_addr + sizeof(DTA) < basepage + BASEPAGE_SIZE)
+		{
+			const char *msg = (dta_addr == basepage + 0x80) ? ", replacing command line" : "";
+			fprintf(fp, "- NOTE: DTA (0x%x) is within current program basepage (0x%x)%s!\n",
+				dta_addr, basepage, msg);
+		}
+	}
+	if (!STMemory_CheckAreaType(dta_addr, sizeof(DTA), ABFLAG_RAM)) {
+		fprintf(fp, "- ERROR: invalid memory address!\n");
+		return;
+	}
+	dta = (DTA *)STMemory_STAddrToPointer(dta_addr);
+	memcpy(name, dta->dta_name, TOS_NAMELEN);
+	name[TOS_NAMELEN] = '\0';
+	magic = do_get_mem_long(dta->magic);
+	fprintf(fp, "- magic: 0x%08x (GEMDOS HD = 0x%08x)\n", magic, DTA_MAGIC_NUMBER);
+	if (magic == DTA_MAGIC_NUMBER)
+		fprintf(fp, "- index: 0x%04x\n", do_get_mem_word(dta->index));
+	fprintf(fp, "- attr: 0x%x\n", dta->dta_attrib);
+	fprintf(fp, "- time: 0x%04x\n", do_get_mem_word(dta->dta_time));
+	fprintf(fp, "- date: 0x%04x\n", do_get_mem_word(dta->dta_date));
+	fprintf(fp, "- size: %d\n", do_get_mem_long(dta->dta_size));
+	fprintf(fp, "- name: '%s'\n", name);
+}
+
 #else /* !ENABLE_TRACING */
 void GemDOS_Info(FILE *fp, Uint32 bShowOpcodes)
 {
 	fputs("Hatari isn't configured with ENABLE_TRACING\n", fp);
 }
+void GemDOS_InfoDTA(FILE *fp, Uint32 addrDTA)
+{
+	fputs("Hatari isn't configured with ENABLE_TRACING\n", fp);
+}
 #endif /* !ENABLE_TRACING */
 
 
@@ -3264,9 +3308,6 @@ void GemDOS_OpCode(void)
 	 case 0x0e:
 		Finished = GemDOS_SetDrv(Params);
 		break;
-	 case 0x1a:
-		Finished = GemDOS_SetDTA(Params);
-		break;
 	 case 0x31:
 		Finished = GemDOS_Ptermres(Params);
 		break;
@@ -3370,6 +3411,7 @@ void GemDOS_OpCode(void)
 
 	case 0x09:	/* Cconws */
 	case 0x0A:	/* Cconrs */
+	case 0x1A:	/* Fsetdta */
 	case 0x20:	/* Super */
 	case 0x48:	/* Malloc */
 	case 0x49:	/* Mfree */
@@ -3462,10 +3504,12 @@ void GemDOS_Boot(void)
 
 	LOG_TRACE(TRACE_OS_GEMDOS, "Gemdos_Boot() at PC 0x%X\n", M68000_GetPC() );
 
-	/* install our gemdos handler, if -e or --harddrive option used,
-	 * or user wants to do GEMDOS tracing
+	/* install our gemdos handler, if user has enabled either
+	 * GEMDOS HD, autostarting or GEMDOS tracing
 	 */
-	if (!GEMDOS_EMU_ON && !(LogTraceFlags & (TRACE_OS_GEMDOS|TRACE_OS_BASE)))
+	if (!GEMDOS_EMU_ON &&
+	    !INF_Overriding(AUTOSTART_INTERCEPT) &&
+	    !(LogTraceFlags & (TRACE_OS_GEMDOS|TRACE_OS_BASE)))
 		return;
 
 	/* Get the address of the p_run variable that points to the actual basepage */
