@@ -36,7 +36,7 @@ static SGOBJ aboutdlg[] =
 	{ SGTEXT, 0, 0, 1,19, 38,1, "WITHOUT ANY WARRANTY. See the GNU Ge-" },
 	{ SGTEXT, 0, 0, 1,20, 38,1, "neral Public License for more details." },
 	{ SGBUTTON, SG_DEFAULT, 0, 16,23, 8,1, "OK" },
-	{ -1, 0, 0, 0,0, 0,0, NULL }
+	{ SGSTOP, 0, 0, 0,0, 0,0, NULL }
 };
 
 
