@@ -16,7 +16,6 @@ typedef struct
 {
   char sLogFileName[FILENAME_MAX];
   char sTraceFileName[FILENAME_MAX];
-  int nExceptionDebugMask;
   int nTextLogLevel;
   int nAlertDlgLogLevel;
   bool bConfirmQuit;
@@ -29,10 +28,16 @@ typedef struct
 typedef struct
 {
   int nNumberBase;
-  int nDisasmLines;
+  int nSymbolLines;
   int nMemdumpLines;
+  int nDisasmLines;
+  int nExceptionDebugMask;
   int nDisasmOptions;
   bool bDisasmUAE;
+  /* load symbols immediately on program start, and keep them after its termination */
+  bool bSymbolsResident;
+  /* whether to match all symbols or just types relevant for given command */
+  bool bMatchAllSymbols;
 } CNF_DEBUGGER;
 
 
@@ -87,6 +92,7 @@ typedef struct
 typedef enum {
   SHORTCUT_OPTIONS,
   SHORTCUT_FULLSCREEN,
+  SHORTCUT_BORDERS,
   SHORTCUT_MOUSEGRAB,
   SHORTCUT_COLDRESET,
   SHORTCUT_WARMRESET,
@@ -120,8 +126,8 @@ typedef struct
 
 typedef struct
 {
-  int nMemorySize;
-  int nTTRamSize;
+  int STRamSize_KB;
+  int TTRamSize_KB;
   bool bAutoSave;
   char szMemoryCaptureFileName[FILENAME_MAX];
   char szAutoSaveFileName[FILENAME_MAX];
@@ -210,6 +216,7 @@ typedef struct
   WRITEPROTECTION nWriteProtection;
   GEMDOS_CHR_CONV nGemdosCase;
   bool bFilenameConversion;
+  bool bGemdosHostTime;
   bool bBootFromHardDisk;
   char szHardDiskDirectories[MAX_HARDDRIVES][FILENAME_MAX];
   char szIdeMasterHardDiskImage[FILENAME_MAX];
@@ -252,7 +259,9 @@ typedef struct
   int nFrameSkips;
   bool bFullScreen;
   bool bKeepResolution;
-#if !WITH_SDL2
+#if WITH_SDL2
+  bool bResizable;
+#else
   bool bKeepResolutionST;
 #endif
   bool bAllowOverscan;
@@ -271,9 +280,11 @@ typedef struct
   int nMaxWidth;
   int nMaxHeight;
 #if WITH_SDL2
+  bool bUseSdlRenderer;
   int nRenderScaleQuality;
   bool bUseVsync;
 #endif
+  bool DisableVideo;
 } CNF_SCREEN;
 
 
@@ -291,6 +302,8 @@ typedef struct
   bool bEnableMidi;
   char sMidiInFileName[FILENAME_MAX];
   char sMidiOutFileName[FILENAME_MAX];
+  char sMidiInPortName[FILENAME_MAX];
+  char sMidiOutPortName[FILENAME_MAX];
 } CNF_MIDI;
 
 
@@ -349,6 +362,7 @@ typedef struct
   bool bCycleExactCpu;
   FPUTYPE n_FPUType;
   bool bCompatibleFPU;            /* More compatible FPU */
+  bool bSoftFloatFPU;
   bool bMMU;                      /* TRUE if MMU is enabled */
 #endif
 } CNF_SYSTEM;
@@ -416,5 +430,6 @@ extern void Configuration_Apply(bool bReset);
 extern void Configuration_Load(const char *psFileName);
 extern void Configuration_Save(void);
 extern void Configuration_MemorySnapShot_Capture(bool bSave);
+extern void Configuration_ChangeCpuFreq ( int CpuFreq_new );
 
 #endif
