@@ -13,11 +13,12 @@
 /* prototypes for gui-sdl/dlg*.c functions: */
 extern int Dialog_MainDlg(bool *bReset, bool *bLoadedSnapshot);
 extern void Dialog_AboutDlg(void);
-extern void Dialog_HaltDlg(void);
+extern void DlgCpu_Main(void);
 extern int DlgAlert_Notice(const char *text);
 extern int DlgAlert_Query(const char *text);
 extern void Dialog_DeviceDlg(void);
 extern void DlgFloppy_Main(void);
+extern void Dialog_HaltDlg(void);
 extern void DlgHardDisk_Main(void);
 extern void Dialog_JoyDlg(void);
 extern void Dialog_KeyboardDlg(void);
@@ -26,7 +27,7 @@ extern char* DlgNewDisk_Main(void);
 extern void Dialog_MonitorDlg(void);
 extern void Dialog_WindowDlg(void);
 extern void Dialog_SoundDlg(void);
-extern void Dialog_SystemDlg(void);
+extern void DlgSystem_Main(void);
 extern void DlgRom_Main(void);
 /* and dialog.c */
 extern bool Dialog_DoProperty(void);
