@@ -113,23 +113,10 @@ enum {
 
 
 # define M68000_GetPC()     m68k_getpc()
-# define M68000_SetPC(val)  m68k_setpc(val)
 
 # define M68000_InstrPC		regs.instruction_pc
 # define M68000_CurrentOpcode	regs.opcode
 
-
-static inline Uint16 M68000_GetSR(void)
-{
-	MakeSR();
-	return regs.sr;
-}
-static inline void M68000_SetSR(Uint16 v)
-{
-	regs.sr = v;
-	MakeFromSR();
-}
-
 # define M68000_SetSpecial(flags)   set_special(flags)
 # define M68000_UnsetSpecial(flags) unset_special(flags)
 
@@ -247,14 +234,7 @@ extern const char *OpcodeName[];
 static inline void M68000_AddCycles(int cycles)
 {
 	cycles = (cycles + 3) & ~3;
-#ifdef OLD_CPU_SHIFT
-	cycles = cycles >> nCpuFreqShift;
-
 	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL ( cycles , INT_CPU_CYCLE );
-#else
-//	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL_NO_FREQSHIFT ( cycles , INT_CPU_CYCLE );
-	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL ( cycles , INT_CPU_CYCLE );
-#endif
 	nCyclesMainCounter += cycles;
 	CyclesGlobalClockCounter += cycles;
 }
@@ -345,14 +325,7 @@ static inline void M68000_AddCyclesWithPairing(int cycles)
 		cycles = (cycles + 3) & ~3;		/* no pairing, round current instr to 4 cycles */
 	}
 
-#ifdef OLD_CPU_SHIFT
-	cycles = cycles >> nCpuFreqShift;
-
 	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL ( cycles , INT_CPU_CYCLE );
-#else
-//	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL_NO_FREQSHIFT ( cycles , INT_CPU_CYCLE );
-	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL ( cycles , INT_CPU_CYCLE );
-#endif
 
 	nCyclesMainCounter += cycles;
 	CyclesGlobalClockCounter += cycles;
@@ -372,14 +345,7 @@ static inline void M68000_AddCyclesWithPairing(int cycles)
  */
 static inline void M68000_AddCycles_CE(int cycles)
 {
-#ifdef OLD_CPU_SHIFT
-	cycles = cycles >> nCpuFreqShift;
-
-	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL(cycles, INT_CPU_CYCLE);
-#else
-//	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL_NO_FREQSHIFT ( cycles , INT_CPU_CYCLE );
 	PendingInterruptCount -= INT_CONVERT_TO_INTERNAL ( cycles , INT_CPU_CYCLE );
-#endif
 
 	nCyclesMainCounter += cycles;
 	CyclesGlobalClockCounter += cycles;
@@ -402,7 +368,11 @@ extern void M68000_SyncCpuBus_OnWriteAccess ( void );
 extern void M68000_Flush_Instr_Cache ( uaecptr addr , int size );
 extern void M68000_Flush_Data_Cache ( uaecptr addr , int size );
 extern void M68000_Flush_All_Caches ( uaecptr addr , int size );
+extern void M68000_SetBlitter_CE ( bool ce_mode );
 extern int DMA_MaskAddressHigh ( void );
 extern void M68000_ChangeCpuFreq ( void );
+extern Uint16 M68000_GetSR ( void );
+extern void M68000_SetSR ( Uint16 v );
+extern void M68000_SetPC ( uaecptr v );
 
 #endif
