@@ -11,7 +11,8 @@
 #include <stdio.h>
 #include <SDL.h>
 
-extern int ScreenSnapShot_SavePNG_ToFile(SDL_Surface *surface, FILE *fp, int png_compression_level, int png_filter ,
+extern int ScreenSnapShot_SavePNG_ToFile(SDL_Surface *surface, int destw,
+		int desth, FILE *fp, int png_compression_level, int png_filter,
 		int CropLeft , int CropRight , int CropTop , int CropBottom );
 extern void ScreenSnapShot_SaveScreen(void);
 
