@@ -10,6 +10,9 @@
 #include <SDL.h>
 #include "main.h"
 
+/* must correspond to max value returned by Statusbar_GetHeightForSize() */
+#define STATUSBAR_MAX_HEIGHT (2*(2*8+2))
+
 typedef enum {
 	DRIVE_LED_A,
 	DRIVE_LED_B,
