@@ -58,8 +58,9 @@
 #define VIDEO_HEIGHT_HBL_COLOR  200	/* This is usually the height of the screen */
 #define VIDEO_HEIGHT_HBL_MONO   400
 
-#define VIDEO_HEIGHT_BOTTOM_50HZ 47	/* number of lines in a 50 Hz bottom border */
-#define VIDEO_HEIGHT_BOTTOM_60HZ 29	/* number of lines in a 60 Hz bottom border */
+#define VIDEO_HEIGHT_BOTTOM_50HZ 45	/* number of extra pixel lines in a 50 Hz screen when removing bottom border */
+					/* (2 extra lines can be displayed by doing a 2nd 60/50 Hz switch at the end of the removed bottom border) */
+#define VIDEO_HEIGHT_BOTTOM_60HZ 26	/* number of extra pixel lines in a 60 Hz screen when removing bottom border */
 
 #define VIDEO_END_HBL_50HZ	( VIDEO_START_HBL_50HZ + VIDEO_HEIGHT_HBL_COLOR )	/* 263 */
 #define VIDEO_END_HBL_60HZ	( VIDEO_START_HBL_60HZ + VIDEO_HEIGHT_HBL_COLOR )	/* 234 */
@@ -116,12 +117,12 @@
 #define HBL_VIDEO_CYCLE_OFFSET		0			/* cycles after end of current line (ie on every 512 cycles in 50 Hz) */
 #define TIMERB_VIDEO_CYCLE_OFFSET	24			/* cycles after last displayed pixels : 376+24 in 50 Hz or 372+24 in 60 Hz */
 
-/* This is when ff8205/07/09 are reloaded with the content of ff8201/03 : on line 310 cycle 48/52 in 50 Hz and on line 260 cycle 48/52 in 60 Hz */
+/* This is when ff8205/07/09 are reloaded with the content of ff8201/03 : on line 310 cycle 56/60 in 50 Hz and on line 260 cycle 56/60 in 60 Hz */
 /* (values were measured on real STF/STE) */
 #define RESTART_VIDEO_COUNTER_LINE_50HZ		( SCANLINES_PER_FRAME_50HZ-3 )
 #define RESTART_VIDEO_COUNTER_LINE_60HZ		( SCANLINES_PER_FRAME_60HZ-3 )
-#define RESTART_VIDEO_COUNTER_CYCLE_STF		( 48 )
-#define RESTART_VIDEO_COUNTER_CYCLE_STE		( 48 + 4 )	/* 4 cycles later than STF */
+#define RESTART_VIDEO_COUNTER_CYCLE_STF		( 56 )
+#define RESTART_VIDEO_COUNTER_CYCLE_STE		( 56 + 4 )	/* 4 cycles later than STF */
 
 /* anything above 4 uses automatic frameskip */
 #define AUTO_FRAMESKIP_LIMIT	5
