@@ -27,6 +27,7 @@ const char IoMemTabFalc_fileid[] = "Hatari ioMemTabFalcon.c : " __DATE__ " " __T
 #include "falcon/videl.h"
 #include "configuration.h"
 #include "statusbar.h"
+#include "stMemory.h"
 #if ENABLE_DSP_EMU
 #include "falcon/dsp.h"
 #endif
@@ -110,7 +111,7 @@ void IoMemTabFalcon_DSPemulation(void (**readtab)(void), void (**writetab)(void)
 /**
  * Take into account the Falcon Bus Control register $ff8007.b
 	$FFFF8007 Falcon Bus Control
-		BIT 6 : F30 Start (0=Cold, 1=Warm) 
+		BIT 6 : F30 Start (0=Cold, 1=Warm)
 		BIT 5 : STe Bus Emulation (0=on)
 		BIT 3 : Blitter Flag (0=on, 1=off)
 		BIT 2 : Blitter (0=8mhz, 1=16mhz)
@@ -126,26 +127,19 @@ static void IoMemTabFalcon_BusCtrl_WriteByte(void)
 	else
 		IoMem_Init_FalconInSTeBuscompatibilityMode(1);
 
-	/* 68030 Frequency changed ? */
-	/* We change freq only in 68030 mode for a normal Falcon, */
-	/* not if CPU is 68040 or 68060 is used */
-	if ( ConfigureParams.System.nCpuLevel == 3 )
+	/* 68030 Frequency changed ? We change freq only in 68030 mode
+	 * for a normal Falcon, not if CPU is 68040 or 68060 is used,
+	 * or if the user requested a faster frequency manually */
+	if (ConfigureParams.System.nCpuLevel == 3 && ConfigureParams.System.nCpuFreq <= 16)
 	{
-		int Freq_old = nCpuFreqShift;
-
 		if ((busCtrl & 0x1) == 1) {
 			/* 16 Mhz bus for 68030 */
-			nCpuFreqShift = 1;
-			ConfigureParams.System.nCpuFreq = 16;
+			Configuration_ChangeCpuFreq ( 16 );
 		}
 		else {
 			/* 8 Mhz bus for 68030 */
-			nCpuFreqShift = 0;
-			ConfigureParams.System.nCpuFreq = 8;
+			Configuration_ChangeCpuFreq ( 8 );
 		}
-
-		if ( Freq_old != nCpuFreqShift )
-			M68000_ChangeCpuFreq();
 	}
 	Statusbar_UpdateInfo();			/* Update clock speed in the status bar */
 }
@@ -178,7 +172,7 @@ static void IoMemTabFalc_Compatible_ReadByte(void)
 {
 	if (nIoMemAccessSize != SIZE_BYTE || (IoMem_ReadByte(0xff8007) & 0x20) != 0)
 	{
-		M68000_BusError(IoAccessBaseAddress, BUS_ERROR_READ,
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ,
 		                nIoMemAccessSize, BUS_ERROR_ACCESS_DATA);
 	}
 }
@@ -187,7 +181,7 @@ static void IoMemTabFalc_Compatible_WriteByte(void)
 {
 	if (nIoMemAccessSize != SIZE_BYTE || (IoMem_ReadByte(0xff8007) & 0x20) != 0)
 	{
-		M68000_BusError(IoAccessBaseAddress, BUS_ERROR_WRITE,
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE,
 		                nIoMemAccessSize, BUS_ERROR_ACCESS_DATA);
 	}
 }
@@ -200,7 +194,7 @@ static void IoMemTabFalc_Compatible_ReadWord(void)
 {
 	if (nIoMemAccessSize == SIZE_BYTE || (IoMem_ReadByte(0xff8007) & 0x20) != 0)
 	{
-		M68000_BusError(IoAccessBaseAddress, BUS_ERROR_READ,
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_READ,
 		                nIoMemAccessSize, BUS_ERROR_ACCESS_DATA);
 	}
 }
@@ -209,7 +203,7 @@ static void IoMemTabFalc_Compatible_WriteWord(void)
 {
 	if (nIoMemAccessSize == SIZE_BYTE || (IoMem_ReadByte(0xff8007) & 0x20) != 0)
 	{
-		M68000_BusError(IoAccessBaseAddress, BUS_ERROR_WRITE,
+		M68000_BusError(IoAccessFullAddress, BUS_ERROR_WRITE,
 		                nIoMemAccessSize, BUS_ERROR_ACCESS_DATA);
 	}
 }
@@ -221,7 +215,7 @@ static void IoMemTabFalc_Compatible_WriteWord(void)
 const INTERCEPT_ACCESS_FUNC IoMemTable_Falcon[] =
 {
 	{ 0xff8000, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
-	{ 0xff8001, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* Memory configuration */
+	{ 0xff8001, SIZE_BYTE, STMemory_MMU_Config_ReadByte, STMemory_MMU_Config_WriteByte },	/* Memory configuration */
 	{ 0xff8006, SIZE_BYTE, IoMem_ReadWithoutInterception, VIDEL_Monitor_WriteByte },        /* Falcon monitor and memory configuration */
 	{ 0xff8007, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMemTabFalcon_BusCtrl_WriteByte }, /* Falcon bus configuration */
 	{ 0xff800C, SIZE_WORD, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
@@ -241,7 +235,7 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_Falcon[] =
 	{ 0xff820b, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here : return 0 not ff */
 	{ 0xff820c, SIZE_BYTE, IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus error here : return 0 not ff */
 	{ 0xff820d, SIZE_BYTE, IoMem_ReadWithoutInterception, VIDEL_ScreenBase_WriteByte },     /* Video base low byte */
-	{ 0xff820e, SIZE_WORD, IoMem_ReadWithoutInterception, VIDEL_LineOffset_WriteWord },     /* Falcon line offset */
+	{ 0xff820e, SIZE_WORD, VIDEL_LineOffset_ReadWord, VIDEL_LineOffset_WriteWord },         /* Falcon line offset */
 	{ 0xff8210, SIZE_WORD, IoMem_ReadWithoutInterception, VIDEL_Line_Width_WriteWord },     /* Falcon line width */
 	{ 0xff8212, 46       , IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
 
