@@ -26,6 +26,7 @@ const char IoMemTabSTE_fileid[] = "Hatari ioMemTabSTE.c : " __DATE__ " " __TIME_
 #include "video.h"
 #include "blitter.h"
 #include "statusbar.h"
+#include "stMemory.h"
 
 
 /**
@@ -40,19 +41,17 @@ void IoMemTabMegaSTE_CacheCpuCtrl_WriteByte(void)
 {
 	Uint8 busCtrl = IoMem_ReadByte(0xff8e21);
 
-	/* 68000 Frequency changed ? */
-	/* We change freq only in 68000 mode for a normal MegaSTE */
-	if ( ConfigureParams.System.nCpuLevel == 0 )
+	/* 68000 Frequency changed ? We change freq only in 68000 mode for a
+	 * normal MegaSTE, if the user did not request a faster one manually */
+	if (ConfigureParams.System.nCpuLevel == 0 && ConfigureParams.System.nCpuFreq <= 16)
 	{
 		if ((busCtrl & 0x1) == 1) {
 			/* 16 Mhz bus for 68000 */
-			nCpuFreqShift = 1;
-			ConfigureParams.System.nCpuFreq = 16;
+			Configuration_ChangeCpuFreq ( 16 );
 		}
 		else {
 			/* 8 Mhz bus for 68000 */
-			nCpuFreqShift = 0;
-			ConfigureParams.System.nCpuFreq = 8;
+			Configuration_ChangeCpuFreq ( 8 );
 		}
 	}
 	Statusbar_UpdateInfo();			/* Update clock speed in the status bar */
@@ -66,7 +65,7 @@ void IoMemTabMegaSTE_CacheCpuCtrl_WriteByte(void)
 const INTERCEPT_ACCESS_FUNC IoMemTable_STE[] =
 {
 	{ 0xff8000, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
-	{ 0xff8001, SIZE_BYTE, IoMem_ReadWithoutInterception, IoMem_WriteWithoutInterception }, /* Memory configuration */
+	{ 0xff8001, SIZE_BYTE, STMemory_MMU_Config_ReadByte, STMemory_MMU_Config_WriteByte },	/* Memory configuration */
 	{ 0xff8002, 14,        IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus errors here */
 
 	{ 0xff8200, SIZE_BYTE, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
