@@ -293,10 +293,12 @@ void M68000_Start(void)
  *	cpu_level : not used anymore
  *	cpu_compatible : 0/false (no prefetch for 68000/20/30)  1/true (prefetch opcode for 68000/20/30)
  *	cpu_cycle_exact : 0/false   1/true (most accurate, implies cpu_compatible)
- *	cpu_memory_cycle_exact : 0/false   1/true (most accurate, implies cpu_compatible)
+ *	cpu_memory_cycle_exact : 0/false   1/true (less accurate than cpu_cycle_exact)
+ *	cpu_data_cache : 0/false (don't emulate caches)   1/true (emulate instr/data caches for 68020/30/40/60)
  *	address_space_24 : 1 (68000/10 and 68030 LC for Falcon), 0 (68020/30/40/60)
  *	fpu_model : 0, 68881 (external), 68882 (external), 68040 (cpu) , 68060 (cpu)
  *	fpu_strict : true/false (more accurate rounding)
+ *	fpu_softfloat : 0/false (faster but less accurate, use host's cpu/fpu)   1/true (most accurate but slower)
  *	mmu_model : 0, 68030, 68040, 68060
  *
  *	m68k_speed : -1=don't adjust cycle  >=0 use m68k_speed_throttle to precisely adjust cycles
@@ -309,27 +311,6 @@ void M68000_Start(void)
  */
 void M68000_CheckCpuSettings(void)
 {
-	int	Freq_old = nCpuFreqShift;
-
-	if (ConfigureParams.System.nCpuFreq < 12)
-	{
-		ConfigureParams.System.nCpuFreq = 8;
-		nCpuFreqShift = 0;
-	}
-	else if (ConfigureParams.System.nCpuFreq > 26)
-	{
-		ConfigureParams.System.nCpuFreq = 32;
-		nCpuFreqShift = 2;
-	}
-	else
-	{
-		ConfigureParams.System.nCpuFreq = 16;
-		nCpuFreqShift = 1;
-	}
-
-	if ( Freq_old != nCpuFreqShift )
-		M68000_ChangeCpuFreq();
-
 	changed_prefs.cpu_level = ConfigureParams.System.nCpuLevel;
 	changed_prefs.cpu_compatible = ConfigureParams.System.bCompatibleCpu;
 
@@ -347,11 +328,23 @@ void M68000_CheckCpuSettings(void)
 	}
 	currprefs.cpu_level = changed_prefs.cpu_level;
 
+	/* Only 68040/60 can have 'internal' FPU */
+	if ( ( ConfigureParams.System.n_FPUType == FPU_CPU ) && ( changed_prefs.cpu_model < 68040 ) )
+		ConfigureParams.System.n_FPUType = FPU_NONE;
+
+	/* 68000/10 can't have an FPU */
+	if ( ( ConfigureParams.System.n_FPUType != FPU_NONE ) && ( changed_prefs.cpu_model < 68020 ) )
+	{
+		Log_Printf(LOG_WARN, "FPU is not supported in 68000/010 configurations, disabling FPU\n");
+		ConfigureParams.System.n_FPUType = FPU_NONE;
+	}
+
 	changed_prefs.address_space_24 = ConfigureParams.System.bAddressSpace24;
 	changed_prefs.cpu_cycle_exact = ConfigureParams.System.bCycleExactCpu;
 	changed_prefs.cpu_memory_cycle_exact = ConfigureParams.System.bCycleExactCpu;
 	changed_prefs.fpu_model = ConfigureParams.System.n_FPUType;
 	changed_prefs.fpu_strict = ConfigureParams.System.bCompatibleFPU;
+	changed_prefs.fpu_softfloat = ConfigureParams.System.bSoftFloatFPU;
 
 	/* Update the MMU model by taking the same value as CPU model */
 	/* MMU is only supported for CPU >=68030, this is later checked in custom.c fixup_cpu() */
@@ -366,6 +359,9 @@ void M68000_CheckCpuSettings(void)
 
 	/* We don't use JIT */
 	currprefs.cachesize = changed_prefs.cachesize = 0;
+
+	/* Always emulate instr/data caches for cpu >= 68020 */
+	changed_prefs.cpu_data_cache = true;
 #else
 	if (ConfigureParams.System.nCpuLevel > 4)
 		ConfigureParams.System.nCpuLevel = 4;
@@ -741,32 +737,61 @@ void	M68000_SyncCpuBus_OnWriteAccess ( void )
  */
 void	M68000_Flush_All_Caches ( uaecptr addr , int size )
 {
+//fprintf ( stderr , "M68000_Flush_All_Caches\n" );
 #ifdef WINUAE_FOR_HATARI
-	M68000_Flush_Instr_Cache ( addr , size );
-	M68000_Flush_Data_Cache ( addr , size );
+	flush_cpu_caches(true);
+	invalidate_cpu_data_caches();
 #endif
 }
 
 
 void	M68000_Flush_Instr_Cache ( uaecptr addr , int size )
 {
+//fprintf ( stderr , "M68000_Flush_Instr_Cache\n" );
 #ifdef WINUAE_FOR_HATARI
 	/* Instruction cache for cpu >= 68020 */
-	flush_instr_cache ( addr , size );
+	flush_cpu_caches(true);
 #endif
 }
 
 
 void	M68000_Flush_Data_Cache ( uaecptr addr , int size )
 {
+//fprintf ( stderr , "M68000_Flush_Data_Cache\n" );
 #ifdef WINUAE_FOR_HATARI
-	/* Data cache for cpu >= 68030 is only emulated with WinUAE */
-	flush_dcache ( addr , size );
+	/* Data cache for cpu >= 68030 */
+	invalidate_cpu_data_caches();
 #endif
 }
 
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * When running in 68000 CE mode, allow to change the "do_cycles" functions
+ * in the cpu emulation depending on the blitter state.
+ *  - if the blitter is not busy, we keep the 'normal' 68000 CE "do_cycles" functions
+ *  - if the blitter is busy, we use a slightly slower "do_cycles" to accurately
+ *    count bus accesses made by the blitter and the CPU
+ *
+ * This limits the overhead of emulating cycle exact blitter bus accesses when blitter is OFF.
+ */
+void	M68000_SetBlitter_CE ( bool state )
+{
+#ifdef WINUAE_FOR_HATARI
+//fprintf ( stderr , "M68000_SetBlitter_CE state=%s\n" , state ? "on" : "off" );
+	if ( state )
+	{
+		set_x_funcs_hatari_blitter ( 1 );		/* on */
+	}
+	else
+	{
+		set_x_funcs_hatari_blitter ( 0 );		/* off */
+	}
+#endif
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * On real STF/STE hardware, DMA accesses are restricted to 4 MB (video addresses,
@@ -793,7 +818,7 @@ int	DMA_MaskAddressHigh ( void )
 	if (Config_IsMachineTT() || Config_IsMachineFalcon())
 		return 0xff;					/* Falcon / TT can access 24 bits with DMA */
 
-	else if ( ConfigureParams.Memory.nMemorySize > 4 )	/* ST/STE with more than 4 MB */
+	else if (ConfigureParams.Memory.STRamSize_KB > 4*1024)	/* ST/STE with more than 4 MB */
 		return 0xff;					/* Allow 'fake' 24 bits for DMA */
 
 	else							/* ST/STE with <= 4 MB */
@@ -817,3 +842,35 @@ void	M68000_ChangeCpuFreq ( void )
 		Crossbar_Recalculate_Clocks_Cycles();
 	}
 }
+
+
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Some CPU registers can't be read or modified directly, some additional
+ * actions are required.
+ */
+Uint16	M68000_GetSR ( void )
+{
+	MakeSR();
+	return regs.sr;
+}
+
+void	M68000_SetSR ( Uint16 v )
+{
+	regs.sr = v;
+	MakeFromSR();
+}
+
+void	M68000_SetPC ( uaecptr v )
+{
+	m68k_setpc ( v );
+#ifdef WINUAE_FOR_HATARI
+	fill_prefetch();
+#else
+	refill_prefetch (m68k_getpc(), 0);
+#endif
+}
+
+
