@@ -289,9 +289,9 @@ bool Main_SetVBLSlowdown(int factor)
  * This function waits on each emulated VBL to synchronize the real time
  * with the emulated ST.
  * Unfortunately SDL_Delay and other sleep functions like usleep or nanosleep
- * are very inaccurate on some systems like Linux 2.4 or Mac OS X (they can only
- * wait for a multiple of 10ms due to the scheduler on these systems), so we have
- * to "busy wait" there to get an accurate timing.
+ * are very inaccurate on some systems like Linux 2.4 or macOS (they can only
+ * wait for a multiple of 10ms due to the scheduler on these systems), so we
+ * have to "busy wait" there to get an accurate timing.
  * All times are expressed as micro seconds, to avoid too much rounding error.
  */
 void Main_WaitOnVbl(void)
@@ -324,10 +324,14 @@ void Main_WaitOnVbl(void)
 	nDelay = DestTicks - CurrentTicks;
 
 	/* Do not wait if we are in fast forward mode or if we are totally out of sync */
+	/* or if we are in benchmark mode */
 	if (ConfigureParams.System.bFastForward == true
-	    || nDelay < -4*FrameDuration_micro || nDelay > 50*FrameDuration_micro)
+	    || nDelay < -4*FrameDuration_micro || nDelay > 50*FrameDuration_micro
+	    || BenchmarkMode )
+
 	{
-		if (ConfigureParams.System.bFastForward == true)
+		if ( ( ConfigureParams.System.bFastForward == true )
+		  || ( BenchmarkMode == true ) )
 		{
 			if (!nFirstMilliTick)
 				nFirstMilliTick = Main_GetTicks();
@@ -341,6 +345,7 @@ void Main_WaitOnVbl(void)
 		DestTicks = CurrentTicks + FrameDuration_micro;
 		return;
 	}
+
 	/* If automatic frameskip is enabled and delay's more than twice
 	 * the effect of single frameskip, decrease frameskip
 	 */
@@ -517,6 +522,12 @@ void Main_EventHandler(void)
 			break;
 
 		 case SDL_KEYDOWN:
+#if WITH_SDL2
+			if (event.key.repeat) {
+				bContinueProcessing = true;
+				break;
+			}
+#endif
 			Keymap_KeyDown(&event.key.keysym);
 			break;
 
@@ -607,11 +618,23 @@ void Main_EventHandler(void)
 			break;
 
 		 case SDL_WINDOWEVENT:
+			/* Note: any changes here should most likely be done
+			 * also in sdlgui.c::SDLGui_DoDialog()
+			 */
 			if (event.window.event == SDL_WINDOWEVENT_SIZE_CHANGED
-			    || event.window.event == SDL_WINDOWEVENT_RESTORED)
+			    || event.window.event == SDL_WINDOWEVENT_RESTORED
+			    || event.window.event == SDL_WINDOWEVENT_EXPOSED)
 			{
 				SDL_UpdateRect(sdlscrn, 0, 0, 0, 0);
-				break;
+			}
+			if (event.window.event == SDL_WINDOWEVENT_EXPOSED
+			    && !ConfigureParams.Screen.bUseSdlRenderer)
+			{
+				/* Hack: Redraw screen here when going into
+				 * fullscreen mode without SDL renderer */
+				sdlscrn = SDL_GetWindowSurface(sdlWindow);
+				Screen_SetFullUpdate();
+				Statusbar_Init(sdlscrn);
 			}
 			bContinueProcessing = true;
 			break;
@@ -701,6 +724,8 @@ static void Main_Init(void)
 	Screen_Init();
 	Main_SetTitle(NULL);
 
+	STMemory_Init ( ConfigureParams.Memory.STRamSize_KB * 1024 );
+
 	ACIA_Init( ACIA_Array , MachineClocks.ACIA_Freq , MachineClocks.ACIA_Freq );
 	IKBD_Init();			/* After ACIA_Init */
 
@@ -728,7 +753,8 @@ static void Main_Init(void)
 	}
 	if (!bTosImageLoaded || bQuitProgram)
 	{
-		fprintf(stderr, "Failed to load TOS image!\n");
+		if (!bTosImageLoaded)
+			fprintf(stderr, "Failed to load TOS image!\n");
 		SDL_Quit();
 		exit(-2);
 	}
