@@ -36,16 +36,20 @@ const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
 #include "vdi.h"
 #include "joy.h"
 #include "log.h"
-#include "tos.h"
+#include "inffile.h"
 #include "paths.h"
 #include "avi_record.h"
 #include "hatari-glue.h"
 #include "68kDisass.h"
 #include "xbios.h"
+#include "stMemory.h"
+
 
 bool bLoadAutoSave;        /* Load autosave memory snapshot at startup */
 bool bLoadMemorySave;      /* Load memory snapshot provided via option at startup */
 bool AviRecordOnStartup;   /* Start avi recording at startup */
+bool BenchmarkMode;	   /* Start in benchmark mode (try to run at maximum emulation */
+			   /* speed allowed by the CPU). Disable audio/video for best results */
 
 int ConOutDevice = CONOUT_DEVICE_NONE; /* device number for xconout device to track */
 
@@ -60,11 +64,14 @@ enum {
 	OPT_CONFIGFILE,
 	OPT_KEYMAPFILE,
 	OPT_FASTFORWARD,
+	OPT_AUTOSTART,
 	OPT_MONO,		/* common display options */
 	OPT_MONITOR,
+	OPT_TOS_RESOLUTION,
 	OPT_FULLSCREEN,
 	OPT_WINDOW,
 	OPT_GRAB,
+	OPT_RESIZABLE,
 	OPT_FRAMESKIPS,
 	OPT_SLOWDOWN,
 	OPT_MOUSE_WARP,
@@ -73,6 +80,7 @@ enum {
 	OPT_MAXWIDTH,
 	OPT_MAXHEIGHT,
 	OPT_FORCEBPP,
+	OPT_DISABLE_VIDEO,
 	OPT_BORDERS,		/* ST/STE display options */
 	OPT_RESOLUTION_ST,
 	OPT_SPEC512,
@@ -99,8 +107,12 @@ enum {
 	OPT_JOYSTICK4,
 	OPT_JOYSTICK5,
 	OPT_PRINTER,
+#ifdef HAVE_PORTMIDI
+	OPT_MIDI,
+#else
 	OPT_MIDI_IN,
 	OPT_MIDI_OUT,
+#endif
 	OPT_RS232_IN,
 	OPT_RS232_OUT,
 	OPT_DRIVEA,		/* disk options */
@@ -114,6 +126,7 @@ enum {
 	OPT_HARDDRIVE,
 	OPT_WRITEPROT_HD,
 	OPT_GEMDOS_CASE,
+	OPT_GEMDOS_HOSTTIME,
 	OPT_GEMDOS_CONVERT,
 	OPT_GEMDOS_DRIVE,
 	OPT_ACSIHDIMAGE,
@@ -135,6 +148,7 @@ enum {
 	OPT_CPU_ADDR24,
 	OPT_FPU_TYPE,
 /*	OPT_FPU_JIT_COMPAT, */
+	OPT_FPU_SOFTFLOAT,
 	OPT_MMU,
 #endif
 	OPT_MACHINE,		/* system options */
@@ -166,6 +180,7 @@ enum {
 	OPT_LOGLEVEL,
 	OPT_ALERTLEVEL,
 	OPT_RUNVBLS,
+	OPT_BENCHMARK,
 	OPT_ERROR,
 	OPT_CONTINUE
 };
@@ -194,18 +209,24 @@ static const opt_t HatariOptions[] = {
 	  "<file>", "Read (additional) keyboard mappings from <file>" },
 	{ OPT_FASTFORWARD, NULL, "--fast-forward",
 	  "<bool>", "Help skipping stuff on fast machine" },
+	{ OPT_AUTOSTART, NULL, "--auto",
+	  "<x>", "Atari program autostarting with Atari path" },
 
 	{ OPT_HEADER, NULL, NULL, NULL, "Common display" },
 	{ OPT_MONO,      "-m", "--mono",
 	  NULL, "Start in monochrome mode instead of color" },
 	{ OPT_MONITOR,      NULL, "--monitor",
 	  "<x>", "Select monitor type (x = mono/rgb/vga/tv)" },
+	{ OPT_TOS_RESOLUTION,   NULL, "--tos-res",
+	  "<x>", "TOS resolution (x = low/med/high/ttlow/ttmed)" },
 	{ OPT_FULLSCREEN,"-f", "--fullscreen",
 	  NULL, "Start emulator in fullscreen mode" },
 	{ OPT_WINDOW,    "-w", "--window",
 	  NULL, "Start emulator in windowed mode" },
 	{ OPT_GRAB, NULL, "--grab",
 	  NULL, "Grab mouse (also) in windowed mode" },
+	{ OPT_RESIZABLE,    NULL, "--resizable",
+	  "<bool>", "Allow window resizing" },
 	{ OPT_FRAMESKIPS, NULL, "--frameskips",
 	  "<x>", "Skip <x> frames after each shown frame (0=off, >4=auto/max)" },
 	{ OPT_SLOWDOWN, NULL, "--slowdown",
@@ -222,6 +243,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Maximum window height for borders & zooming" },
 	{ OPT_FORCEBPP, NULL, "--bpp",
 	  "<x>", "Force internal bitdepth (x = 15/16/32, 0=disable)" },
+	{ OPT_DISABLE_VIDEO,   NULL, "--disable-video",
+	  "<bool>", "Run emulation without displaying video (audio only)" },
 
 	{ OPT_HEADER, NULL, NULL, NULL, "ST/STE specific display" },
 	{ OPT_BORDERS, NULL, "--borders",
@@ -289,10 +312,15 @@ static const opt_t HatariOptions[] = {
 	  "<type>", "Set joystick type (none/keys/real) for given port" },
 	{ OPT_PRINTER,   NULL, "--printer",
 	  "<file>", "Enable printer support and write data to <file>" },
+#ifdef HAVE_PORTMIDI
+	{ OPT_MIDI,   NULL, "--midi",
+	  "<bool>", "Whether to use MIDI (with PortMidi devices)" },
+#else
 	{ OPT_MIDI_IN,   NULL, "--midi-in",
 	  "<file>", "Enable MIDI and use <file> as the input device" },
 	{ OPT_MIDI_OUT,  NULL, "--midi-out",
 	  "<file>", "Enable MIDI and use <file> as the output device" },
+#endif
 	{ OPT_RS232_IN,  NULL, "--rs232-in",
 	  "<file>", "Enable serial port and use <file> as the input device" },
 	{ OPT_RS232_OUT, NULL, "--rs232-out",
@@ -323,6 +351,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Write protect harddrive <dir> contents (on/off/auto)" },
 	{ OPT_GEMDOS_CASE, NULL, "--gemdos-case",
 	  "<x>", "Forcibly up/lowercase new GEMDOS dir/filenames (off/upper/lower)" },
+	{ OPT_GEMDOS_HOSTTIME, NULL, "--gemdos-time",
+	  "<x>", "Which timestamps to use for GEMDOS files (atari/host)" },
 	{ OPT_GEMDOS_CONVERT, NULL, "--gemdos-conv",
 	  "<bool>", "Atari GEMDOS <-> host (UTF-8) file name conversion" },
 	{ OPT_GEMDOS_DRIVE, NULL, "--gemdos-drive",
@@ -336,7 +366,7 @@ static const opt_t HatariOptions[] = {
 	
 	{ OPT_HEADER, NULL, NULL, NULL, "Memory" },
 	{ OPT_MEMSIZE,   "-s", "--memsize",
-	  "<x>", "ST RAM size (x = size in MiB from 0 to 14, 0 = 512KiB)" },
+	  "<x>", "ST RAM size (x = size in MiB from 0 to 14, 0 = 512KiB ; else size in KiB)" },
 #if ENABLE_WINUAE_CPU
 	{ OPT_TT_RAM,   NULL, "--ttram",
 	  "<x>", "TT RAM size (x = size in MiB from 0 to 256)" },
@@ -372,6 +402,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "FPU type (x=none/68881/68882/internal)" },
 	/*{ OPT_FPU_JIT_COMPAT, NULL, "--fpu-compatible",
 	  "<bool>", "Use more compatible, but slower FPU JIT emulation" },*/
+	{ OPT_FPU_SOFTFLOAT, NULL, "--fpu-softfloat",
+	  "<bool>", "Use full software FPU emulation" },
 	{ OPT_MMU, NULL, "--mmu",
 	  "<bool>", "Use MMU emulation" },
 #endif
@@ -439,6 +471,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Show dialog for log messages above given level" },
 	{ OPT_RUNVBLS, NULL, "--run-vbls",
 	  "<x>", "Exit after x VBLs" },
+	{ OPT_BENCHMARK, NULL, "--benchmark",
+	  NULL, "Start in benchmark mode (use with --run-vbls)" },
 
 	{ OPT_ERROR, NULL, NULL, NULL, NULL }
 };
@@ -585,7 +619,7 @@ static void Opt_ShowHelp(void)
  * otherwise 'value' is show as the option user gave.
  * Return false if error string was given, otherwise true
  */
-static bool Opt_ShowError(unsigned int optid, const char *value, const char *error)
+bool Opt_ShowError(unsigned int optid, const char *value, const char *error)
 {
 	const opt_t *opt;
 
@@ -850,6 +884,24 @@ Uint32 Opt_GetNoParachuteFlag(void)
 }
 
 
+/**
+ * Do final validation for the earlier + parsed options
+ *
+ * Return false if they fail validation.
+ */
+static bool Opt_ValidateOptions(void)
+{
+	const char *err, *val;
+	int opt_id;
+
+	if ((opt_id = INF_ValidateAutoStart(&val, &err)))
+	{
+		return Opt_ShowError(opt_id, val, err);
+	}
+	return true;
+}
+
+
 /**
  * Return true if given path points to an Atari program, false otherwise.
  */
@@ -896,7 +948,7 @@ static bool Opt_HandleArgument(const char *path)
 		 * then make sure that given program from that
 		 * dir will be started.
 		 */
-		TOS_AutoStart(prgname);
+		INF_SetAutoStart(prgname, OPT_AUTOSTART);
 	}
 	if (dir) {
 		path = dir;
@@ -957,7 +1009,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 	{
 		/* last argument can be a non-option */
 		if (argv[i][0] != '-' && i+1 == argc)
-			return Opt_HandleArgument(argv[i]);
+			return Opt_HandleArgument(argv[i]) && Opt_ValidateOptions();
 
 		/* WhichOption() checks also that there is an argument,
 		 * so we don't need to check that below
@@ -982,6 +1034,13 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ok = Opt_Bool(argv[++i], OPT_FASTFORWARD, &ConfigureParams.System.bFastForward);
 			break;
 
+		case OPT_AUTOSTART:
+			if (!(ok = INF_SetAutoStart(argv[++i], OPT_AUTOSTART)))
+			{
+				return Opt_ShowError(OPT_AUTOSTART, argv[i], "Invalid drive and/or path specified for autostart program");
+			}
+			break;
+
 		case OPT_CONFIGFILE:
 			i += 1;
 			/* true -> file needs to exist */
@@ -1025,6 +1084,14 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			bLoadAutoSave = false;
 			break;
 
+		case OPT_TOS_RESOLUTION:
+			i += 1;
+			if (!INF_SetResolution(argv[i], OPT_TOS_RESOLUTION))
+			{
+				return Opt_ShowError(OPT_TOS_RESOLUTION, argv[i], "Invalid resolution");
+			}
+			break;
+
 		case OPT_FULLSCREEN:
 			ConfigureParams.Screen.bFullScreen = true;
 			break;
@@ -1037,6 +1104,15 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			bGrabMouse = true;
 			break;
 
+		case OPT_RESIZABLE:
+#if WITH_SDL2
+			ok = Opt_Bool(argv[++i], OPT_RESIZABLE, &ConfigureParams.Screen.bResizable);
+#else
+			fprintf(stderr, "The --resizable option is supported only in SDL2 build!\n");
+			i++;
+#endif
+			break;
+
 		case OPT_FRAMESKIPS:
 			skips = atoi(argv[++i]);
 			if (skips < 0)
@@ -1088,6 +1164,10 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ConfigureParams.Screen.nForceBpp = planes;
 			break;
 
+		case OPT_DISABLE_VIDEO:
+			ok = Opt_Bool(argv[++i], OPT_DISABLE_VIDEO, &ConfigureParams.Screen.DisableVideo);
+			break;
+
 			/* ST/STE display options */
 		case OPT_BORDERS:
 			ok = Opt_Bool(argv[++i], OPT_BORDERS, &ConfigureParams.Screen.bAllowOverscan);
@@ -1095,7 +1175,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 
 		case OPT_RESOLUTION_ST:
 #if WITH_SDL2
-			fprintf(stderr, "The --desktop-st option is not supported in this version (with SDL2)!\n");
+			fprintf(stderr, "The --desktop-st option is not supported in SDL2 build!\n");
 			i++;
 #else
 			ok = Opt_Bool(argv[++i], OPT_RESOLUTION_ST, &ConfigureParams.Screen.bKeepResolutionST);
@@ -1305,6 +1385,11 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 					&ConfigureParams.Printer.bEnablePrinting);
 			break;
 
+#ifdef HAVE_PORTMIDI
+		case OPT_MIDI:
+			ok = Opt_Bool(argv[++i], OPT_MIDI, &ConfigureParams.Midi.bEnableMidi);
+			break;
+#else
 		case OPT_MIDI_IN:
 			i += 1;
 			ok = Opt_StrCpy(OPT_MIDI_IN, true, ConfigureParams.Midi.sMidiInFileName,
@@ -1318,7 +1403,8 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 					argv[i], sizeof(ConfigureParams.Midi.sMidiOutFileName),
 					&ConfigureParams.Midi.bEnableMidi);
 			break;
-      
+#endif
+
 		case OPT_RS232_IN:
 			i += 1;
 			ok = Opt_StrCpy(OPT_RS232_IN, true, ConfigureParams.RS232.szInFileName,
@@ -1419,6 +1505,16 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				return Opt_ShowError(OPT_GEMDOS_CASE, argv[i], "Unknown option value");
 			break;
 
+		case OPT_GEMDOS_HOSTTIME:
+			i += 1;
+			if (strcasecmp(argv[i], "atari") == 0)
+				ConfigureParams.HardDisk.bGemdosHostTime = false;
+			else if (strcasecmp(argv[i], "host") == 0)
+				ConfigureParams.HardDisk.bGemdosHostTime = true;
+			else
+				return Opt_ShowError(OPT_GEMDOS_HOSTTIME, argv[i], "Unknown option value");
+			break;
+
 		case OPT_GEMDOS_CONVERT:
 			ok = Opt_Bool(argv[++i], OPT_GEMDOS_CONVERT, &ConfigureParams.HardDisk.bFilenameConversion);
 			break;
@@ -1466,7 +1562,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			if (strlen(str) > 2 && isdigit(str[0]) && str[1] == '=')
 			{
 				drive = str[0] - '0';
-				if (drive < 0 || drive > 7)
+				if (drive < 0 || drive >= MAX_ACSI_DEVS)
 					return Opt_ShowError(OPT_ACSIHDIMAGE, str, "Invalid ACSI drive <id>, must be 0-7");
 				str += 2;
 			}
@@ -1508,18 +1604,19 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			/* Memory options */
 		case OPT_MEMSIZE:
 			memsize = atoi(argv[++i]);
-			if (memsize < 0 || memsize > 14)
+			memsize = STMemory_RAM_Validate_Size_KB ( memsize );
+			if (memsize < 0)
 			{
 				return Opt_ShowError(OPT_MEMSIZE, argv[i], "Invalid memory size");
 			}
-			ConfigureParams.Memory.nMemorySize = memsize;
+			ConfigureParams.Memory.STRamSize_KB = memsize;
 			bLoadAutoSave = false;
 			break;
 
 #if ENABLE_WINUAE_CPU
 		case OPT_TT_RAM:
 			memsize = atoi(argv[++i]);
-			ConfigureParams.Memory.nTTRamSize = Opt_ValueAlignMinMax(memsize+3, 4, 0, 256);
+			ConfigureParams.Memory.TTRamSize_KB = Opt_ValueAlignMinMax(memsize+3, 4, 0, 256) * 1024;
 			bLoadAutoSave = false;
 			break;
 #endif
@@ -1586,7 +1683,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				return Opt_ShowError(OPT_CPUCLOCK, argv[i], "Invalid CPU clock");
 			}
-			ConfigureParams.System.nCpuFreq = cpuclock;
+			Configuration_ChangeCpuFreq ( cpuclock );
 			bLoadAutoSave = false;
 			break;
 
@@ -1637,6 +1734,10 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			ok = Opt_Bool(argv[++i], OPT_FPU_COMPATIBLE, &ConfigureParams.System.bCompatibleFPU);
 			break;
 */
+		case OPT_FPU_SOFTFLOAT:
+			ok = Opt_Bool(argv[++i], OPT_FPU_SOFTFLOAT, &ConfigureParams.System.bSoftFloatFPU);
+			break;
+
 		case OPT_MMU:
 			ok = Opt_Bool(argv[++i], OPT_MMU, &ConfigureParams.System.bMMU);
 			bLoadAutoSave = false;
@@ -1650,31 +1751,31 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				ConfigureParams.System.nMachineType = MACHINE_ST;
 				ConfigureParams.System.nCpuLevel = 0;
-				ConfigureParams.System.nCpuFreq = 8;
+				Configuration_ChangeCpuFreq ( 8 );
 			}
 			else if (strcasecmp(argv[i], "megast") == 0)
 			{
 				ConfigureParams.System.nMachineType = MACHINE_MEGA_ST;
 				ConfigureParams.System.nCpuLevel = 0;
-				ConfigureParams.System.nCpuFreq = 8;
+				Configuration_ChangeCpuFreq ( 8 );
 			}
 			else if (strcasecmp(argv[i], "ste") == 0)
 			{
 				ConfigureParams.System.nMachineType = MACHINE_STE;
 				ConfigureParams.System.nCpuLevel = 0;
-				ConfigureParams.System.nCpuFreq = 8;
+				Configuration_ChangeCpuFreq ( 8 );
 			}
 			else if (strcasecmp(argv[i], "megaste") == 0)
 			{
 				ConfigureParams.System.nMachineType = MACHINE_MEGA_STE;
 				ConfigureParams.System.nCpuLevel = 0;
-				ConfigureParams.System.nCpuFreq = 16;
+				Configuration_ChangeCpuFreq ( 16 );
 			}
 			else if (strcasecmp(argv[i], "tt") == 0)
 			{
 				ConfigureParams.System.nMachineType = MACHINE_TT;
 				ConfigureParams.System.nCpuLevel = 3;
-				ConfigureParams.System.nCpuFreq = 32;
+				Configuration_ChangeCpuFreq ( 32 );
 			}
 			else if (strcasecmp(argv[i], "falcon") == 0)
 			{
@@ -1683,7 +1784,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 #endif
 				ConfigureParams.System.nMachineType = MACHINE_FALCON;
 				ConfigureParams.System.nCpuLevel = 3;
-				ConfigureParams.System.nCpuFreq = 16;
+				Configuration_ChangeCpuFreq ( 16 );
 			}
 			else
 			{
@@ -1834,14 +1935,14 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			else
 			{
-				ExceptionDebugMask = ConfigureParams.Log.nExceptionDebugMask;
+				ExceptionDebugMask = ConfigureParams.Debugger.nExceptionDebugMask;
 				fprintf(stderr, "Exception debugging enabled (0x%x).\n", ExceptionDebugMask);
 			}
 			break;
 
 		case OPT_EXCEPTIONS:
 			i += 1;
-			/* sets ConfigureParams.Log.nExceptionDebugMask */
+			/* sets ConfigureParams.Debugger.nExceptionDebugMask */
 			errstr = Log_SetExceptionDebugMask(argv[i]);
 			if (errstr)
 			{
@@ -1855,7 +1956,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				/* already enabled, change run-time config */
 				int oldmask = ExceptionDebugMask;
-				ExceptionDebugMask = ConfigureParams.Log.nExceptionDebugMask;
+				ExceptionDebugMask = ConfigureParams.Debugger.nExceptionDebugMask;
 				fprintf(stderr, "Exception debugging changed (0x%x -> 0x%x).\n",
 					oldmask, ExceptionDebugMask);
 			}
@@ -1966,6 +2067,10 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			Main_SetRunVBLs(atol(argv[++i]));
 			break;
 		       
+		case OPT_BENCHMARK:
+			BenchmarkMode = true;
+			break;
+
 		case OPT_ERROR:
 			/* unknown option or missing option parameter */
 			return false;
@@ -1980,7 +2085,7 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		}
 	}
 
-	return true;
+	return Opt_ValidateOptions();
 }
 
 /**
