@@ -23,8 +23,10 @@ const char Paths_fileid[] = "Hatari paths.c : " __DATE__ " " __TIME__;
 
 #if defined(__MACOSX__)
 	#define HATARI_HOME_DIR "Library/Application Support/Hatari"
+#elif defined(WIN32)
+	#define HATARI_HOME_DIR "AppData\\Local\\Hatari"
 #else
-	#define HATARI_HOME_DIR ".hatari"
+	#define HATARI_HOME_DIR ".config/hatari"
 #endif
 
 static char sWorkingDir[FILENAME_MAX];    /* Working directory */
@@ -112,7 +114,7 @@ static void Paths_GetExecDirFromPATH(const char *argv0, char *pExecDir, int nMax
 			strlcpy(pExecDir, pAct, nMaxLen);
 			break;
 		}
-		pAct = strtok (0, pToken);
+		pAct = strtok (NULL, pToken);
   	}
 
 	free(pPathEnv);
@@ -162,7 +164,7 @@ static char *Paths_InitExecDir(const char *argv0)
 	/* If we do not have the execdir yet, analyze argv[0] and the PATH: */
 	if (psExecDir[0] == 0)
 	{
-		if (strchr(argv0, PATHSEP) == 0)
+		if (strchr(argv0, PATHSEP) == NULL)
 		{
 			/* No separator in argv[0], we have to explore PATH... */
 			Paths_GetExecDirFromPATH(argv0, psExecDir, FILENAME_MAX);
@@ -217,25 +219,47 @@ static void Paths_InitHomeDirs(void)
 		/* $HOME not set, so let's use current working dir as home */
 		strcpy(sUserHomeDir, sWorkingDir);
 		strcpy(sHatariHomeDir, sWorkingDir);
+		return;
 	}
-	else
-	{
-		sUserHomeDir[FILENAME_MAX-1] = 0;
 
-		/* Try to use a .hatari directory in the users home directory */
-		snprintf(sHatariHomeDir, FILENAME_MAX, "%s%c%s", sUserHomeDir,
-		         PATHSEP, HATARI_HOME_DIR);
-		if (!File_DirExists(sHatariHomeDir))
+	sUserHomeDir[FILENAME_MAX-1] = 0;
+
+	/* Try to use a private hatari directory in the users home directory */
+	snprintf(sHatariHomeDir, FILENAME_MAX, "%s%c%s", sUserHomeDir,
+	         PATHSEP, HATARI_HOME_DIR);
+	if (File_DirExists(sHatariHomeDir))
+	{
+		return;
+	}
+	/* Try legacy location ~/.hatari */
+	snprintf(sHatariHomeDir, FILENAME_MAX, "%s%c.hatari", sUserHomeDir,
+	         PATHSEP);
+	if (File_DirExists(sHatariHomeDir))
+	{
+		return;
+	}
+
+	/* Hatari home directory does not exists yet...
+	 * ... so let's try to create it: */
+#if !defined(__MACOSX__) && !defined(WIN32)
+	snprintf(sHatariHomeDir, FILENAME_MAX, "%s%c.config", sUserHomeDir,
+	         PATHSEP);
+	if (!File_DirExists(sHatariHomeDir))
+	{
+		/* ~/.config does not exist yet, create it first */
+		if (mkdir(sHatariHomeDir, 0700) != 0)
 		{
-			/* Hatari home directory does not exists yet...
-			 * ...so let's try to create it: */
-			if (mkdir(sHatariHomeDir, 0755) != 0)
-			{
-				/* Failed to create, so use user's home dir instead */
-				strcpy(sHatariHomeDir, sUserHomeDir);
-			}
+			perror("Failed to create ~/.config directory");
 		}
 	}
+#endif
+	snprintf(sHatariHomeDir, FILENAME_MAX, "%s%c%s", sUserHomeDir,
+	         PATHSEP, HATARI_HOME_DIR);
+	if (mkdir(sHatariHomeDir, 0750) != 0)
+	{
+		/* Failed to create, so use user's home dir instead */
+		strcpy(sHatariHomeDir, sUserHomeDir);
+	}
 }
 
 
