@@ -11,6 +11,7 @@ const char Resolution_fileid[] = "Hatari resolution.c : " __DATE__ " " __TIME__;
 #include <SDL.h>
 #include "main.h"
 #include "configuration.h"
+#include "log.h"
 #include "resolution.h"
 #include "statusbar.h"
 #include "screen.h"
@@ -68,7 +69,7 @@ void Resolution_Init(void)
 		ConfigureParams.Screen.nMaxHeight = DesktopHeight;
 	}
 	DEBUGPRINT(("Desktop resolution: %dx%d\n",DesktopWidth, DesktopHeight));
-	fprintf(stderr, "Configured max Hatari resolution = %dx%d, optimal for ST = %dx%d\n",
+	Log_Printf(LOG_DEBUG, "Configured max Hatari resolution = %dx%d, optimal for ST = %dx%d\n",
 		ConfigureParams.Screen.nMaxWidth, ConfigureParams.Screen.nMaxHeight,
 		2*NUM_VISIBLE_LINE_PIXELS, 2*NUM_VISIBLE_LINES+STATUSBAR_MAX_HEIGHT);
 }
@@ -100,7 +101,8 @@ static void Resolution_GetMaxSize(int *width, int *height)
  * - Otherwise select the largest available mode
  * return true for success and false if no matching mode was found.
  */
-static inline bool Resolution_Select(SDL_Rect **modes, int *width, int *height)
+#if !WITH_SDL2
+static bool Resolution_Select(SDL_Rect **modes, int *width, int *height)
 {
 #define TOO_LARGE 0x7fff
 	int i, bestw, besth;
@@ -137,6 +139,7 @@ static inline bool Resolution_Select(SDL_Rect **modes, int *width, int *height)
 	return true;
 #undef TOO_LARGE
 }
+#endif
 
 
 /**
