@@ -93,6 +93,7 @@ static void (*ScreenDrawFunctionsNormal[3])(void); /* Screen draw functions */
 static bool bScreenContentsChanged;     /* true if buffer changed and requires blitting */
 static bool bScrDoubleY;                /* true if double on Y */
 static int ScrUpdateFlag;               /* Bit mask of how to update screen */
+static bool bRGBTableInSync;            /* Is RGB table up to date? */
 
 /* These are used for the generic screen conversion functions */
 static int genconv_width_req, genconv_height_req, genconv_bpp;
@@ -109,15 +110,30 @@ static SDL_Texture *sdlTexture;
 
 void SDL_UpdateRects(SDL_Surface *screen, int numrects, SDL_Rect *rects)
 {
-	SDL_UpdateTexture(sdlTexture, NULL, screen->pixels, screen->pitch);
-	SDL_RenderClear(sdlRenderer);
-	SDL_RenderCopy(sdlRenderer, sdlTexture, NULL, NULL);
-	SDL_RenderPresent(sdlRenderer);
+	if (ConfigureParams.Screen.bUseSdlRenderer)
+	{
+		SDL_UpdateTexture(sdlTexture, NULL, screen->pixels, screen->pitch);
+		SDL_RenderCopy(sdlRenderer, sdlTexture, NULL, NULL);
+		SDL_RenderPresent(sdlRenderer);
+	}
+	else
+	{
+		SDL_UpdateWindowSurfaceRects(sdlWindow, rects, numrects);
+	}
 }
 
 void SDL_UpdateRect(SDL_Surface *screen, Sint32 x, Sint32 y, Sint32 w, Sint32 h)
 {
-	SDL_Rect rect = { x, y, w, h };
+	SDL_Rect rect;
+
+	if (w == 0 && h == 0) {
+		x = y = 0;
+		w = screen->w;
+		h = screen->h;
+	}
+
+	rect.x = x; rect.y = y;
+	rect.w = w; rect.h = h;
 	SDL_UpdateRects(screen, 1, &rect);
 }
 
@@ -284,7 +300,8 @@ static void Screen_FreeSDL2Resources(void)
 	}
 	if (sdlscrn)
 	{
-		SDL_FreeSurface(sdlscrn);
+		if (ConfigureParams.Screen.bUseSdlRenderer)
+			SDL_FreeSurface(sdlscrn);
 		sdlscrn = NULL;
 	}
 	if (sdlRenderer)
@@ -311,16 +328,11 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 	if (bitdepth == 0 || bitdepth == 24)
 		bitdepth = 32;
 #endif
-
 	/* Check if we really have to change the video mode: */
 	if (sdlscrn != NULL && sdlscrn->w == width && sdlscrn->h == height
 	    && sdlscrn->format->BitsPerPixel == bitdepth && !bForceChange)
 		return false;
 
-	/* We can not continue recording with a different resolution */
-	if (Avi_AreWeRecording())
-		Avi_StopRecording();
-
 #ifdef _MUDFLAP
 	if (sdlscrn)
 	{
@@ -349,7 +361,13 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 	else
 	{
 		int deskw, deskh;
-		sdlVideoFlags  = SDL_WINDOW_RESIZABLE;
+		if (getenv("PARENT_WIN_ID") != NULL)	/* Embedded window? */
+			sdlVideoFlags = SDL_WINDOW_BORDERLESS;
+		else if (ConfigureParams.Screen.bResizable
+		         && ConfigureParams.Screen.bUseSdlRenderer)
+			sdlVideoFlags = SDL_WINDOW_RESIZABLE;
+		else
+			sdlVideoFlags = 0;
 		/* Make sure that window is not bigger than current desktop */
 		Resolution_GetDesktopSize(&deskw, &deskh);
 		if (win_width > deskw)
@@ -359,8 +377,11 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 	}
 
 	Screen_FreeSDL2Resources();
-	if (((bInFullScreen && !ConfigureParams.Screen.bKeepResolution)
-	     || bPrevInFullScreen != bInFullScreen) && sdlWindow)
+	if (sdlWindow &&
+	    ((bInFullScreen && !ConfigureParams.Screen.bKeepResolution) ||
+	     (bPrevInFullScreen != bInFullScreen) ||
+	     bForceChange
+	    ))
 	{
 		SDL_DestroyWindow(sdlWindow);
 		sdlWindow = NULL;
@@ -381,6 +402,11 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 		bPrevUseVsync = ConfigureParams.Screen.bUseVsync;
 	}
 
+#ifdef SDL_HINT_WINDOWS_NO_CLOSE_ON_ALT_F4		/* requires sdl >= 2.0.4 */
+	/* Disable closing Hatari with alt+F4 under Windows as alt+F4 can be used by some emulated programs */
+	SDL_SetHintWithPriority(SDL_HINT_WINDOWS_NO_CLOSE_ON_ALT_F4, "1", SDL_HINT_OVERRIDE);
+#endif
+
 	/* Set new video mode */
 	DEBUGPRINT(("SDL screen request: %d x %d @ %d (%s)\n", width, height,
 	        bitdepth, bInFullScreen?"fullscreen":"windowed"));
@@ -396,16 +422,24 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 		                             SDL_WINDOWPOS_UNDEFINED,
 		                             win_width, win_height, sdlVideoFlags);
 	}
-	sdlRenderer = SDL_CreateRenderer(sdlWindow, -1, 0);
-	if (!sdlWindow || !sdlRenderer)
+	if (!sdlWindow)
 	{
-		fprintf(stderr,"Failed to create window or renderer!\n");
+		fprintf(stderr, "Failed to create %dx%d window!\n",
+		        win_width, win_height);
 		exit(-1);
 	}
-	else
+	if (ConfigureParams.Screen.bUseSdlRenderer)
 	{
 		int rm, bm, gm, pfmt;
 
+		sdlRenderer = SDL_CreateRenderer(sdlWindow, -1, 0);
+		if (!sdlRenderer)
+		{
+			fprintf(stderr,"Failed to create %dx%d renderer!\n",
+			        win_width, win_height);
+			exit(1);
+		}
+
 		if (bInFullScreen)
 			SDL_RenderSetLogicalSize(sdlRenderer, width, height);
 
@@ -430,10 +464,15 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 		                               width, height);
 		if (!sdlTexture)
 		{
-			fprintf(stderr,"Failed to create texture!\n");
+			fprintf(stderr,"Failed to create %dx%d@%d texture!\n",
+			       width, height, bitdepth);
 			exit(-3);
 		}
 	}
+	else
+	{
+		sdlscrn = SDL_GetWindowSurface(sdlWindow);
+	}
 
 #else	/* WITH_SDL2 */
 
@@ -489,6 +528,10 @@ bool Screen_SetSDLVideoSize(int width, int height, int bitdepth, bool bForceChan
 		Control_ReparentWindow(width, height, bInFullScreen);
 	}
 
+	Avi_SetSurface(sdlscrn);
+
+	bRGBTableInSync = false;
+
 	return true;
 }
 
@@ -591,7 +634,6 @@ static void Screen_SetSTResolution(bool bForceChange)
 
 	if (Screen_SetSDLVideoSize(Width, Height, BitCount, bForceChange))
 	{
-		Screen_SetupRGBTable();         /* Create color conversion table */
 		Statusbar_Init(sdlscrn);
 
 		/* screen area without the statusbar */
@@ -601,6 +643,12 @@ static void Screen_SetSTResolution(bool bForceChange)
 		STScreenRect.h = sdlscrn->h - Statusbar_GetHeight();
 	}
 
+	if (!bRGBTableInSync)
+	{
+		Screen_SetupRGBTable();   /* Create color conversion table */
+		bRGBTableInSync = true;
+	}
+
 	/* Set drawing functions */
 	Screen_SetDrawFunctions(sdlscrn->format->BitsPerPixel, bDoubleLowRes);
 
@@ -622,7 +670,7 @@ static void Screen_ChangeResolution(bool bForceChange)
 	}
 	else if (Config_IsMachineFalcon())
 	{
-		VIDEL_ZoomModeChanged(bForceChange);
+		Videl_ScreenModeChanged(bForceChange);
 	}
 	else if (Config_IsMachineTT())
 	{
@@ -684,6 +732,13 @@ void Screen_Init(void)
 	}
 	pFrameBuffer = &FrameBuffers[0];
 
+	/* Set initial window resolution */
+	bInFullScreen = ConfigureParams.Screen.bFullScreen;
+	Screen_ChangeResolution(false);
+	ScreenDrawFunctionsNormal[ST_HIGH_RES] = Screen_ConvertHighRes;
+
+	Video_SetScreenRasters();                       /* Set rasters ready for first screen */
+
 	/* Load and set icon */
 	snprintf(sIconFileName, sizeof(sIconFileName), "%s%chatari-icon.bmp",
 	         Paths_GetDataDir(), PATHSEP);
@@ -700,13 +755,6 @@ void Screen_Init(void)
 		SDL_FreeSurface(pIconSurf);
 	}
 
-	/* Set initial window resolution */
-	bInFullScreen = ConfigureParams.Screen.bFullScreen;
-	Screen_ChangeResolution(false);
-	ScreenDrawFunctionsNormal[ST_HIGH_RES] = Screen_ConvertHighRes;
-
-	Video_SetScreenRasters();                       /* Set rasters ready for first screen */
-
 	/* Configure some SDL stuff: */
 	SDL_ShowCursor(SDL_DISABLE);
 }
@@ -1166,6 +1214,10 @@ static void Screen_Blit(SDL_Rect *sbar_rect)
 {
 	unsigned char *pTmpScreen;
 
+	/* Don't update anything on screen if video output is disabled */
+	if ( ConfigureParams.Screen.DisableVideo )
+		return;
+
 #if 0	/* double buffering cannot be used with partial screen updates */
 # if NUM_FRAMEBUFFERS > 1
 	if (bInFullScreen && (sdlscrn->flags & SDL_DOUBLEBUF))
@@ -1301,7 +1353,7 @@ static bool Screen_DrawFrame(bool bForceFlip)
  */
 bool Screen_Draw(void)
 {
-	if (bQuitProgram || !VideoBase)
+	if (bQuitProgram)
 	{
 		return false;
 	}
@@ -1456,6 +1508,10 @@ void Screen_GenConvUpdate(SDL_Rect *extra, bool forced)
 	SDL_Rect rects[2];
 	int count = 1;
 
+	/* Don't update anything on screen if video output is disabled */
+	if ( ConfigureParams.Screen.DisableVideo )
+		return;
+
 	if (!forced && !genconv_do_update) // the HW surface is available
 		return;
 
