@@ -25,6 +25,7 @@ const char ST_fileid[] = "Hatari st.c : " __DATE__ " " __TIME__;
 
 #include "main.h"
 #include "file.h"
+#include "floppy.h"
 #include "st.h"
 
 #define SAVE_TO_ST_IMAGES
@@ -54,7 +55,7 @@ bool ST_FileNameIsST(const char *pszFileName, bool bAllowGZ)
  * Load .ST file into memory, set number of bytes loaded and return a pointer
  * to the buffer.
  */
-Uint8 *ST_ReadDisk(const char *pszFileName, long *pImageSize)
+Uint8 *ST_ReadDisk(int Drive, const char *pszFileName, long *pImageSize, int *pImageType)
 {
 	Uint8 *pStFile;
 
@@ -63,8 +64,12 @@ Uint8 *ST_ReadDisk(const char *pszFileName, long *pImageSize)
 	/* Just load directly a buffer, and set ImageSize accordingly */
 	pStFile = File_Read(pszFileName, pImageSize, NULL);
 	if (!pStFile)
+	{
 		*pImageSize = 0;
+		return NULL;
+	}
 
+	*pImageType = FLOPPY_IMAGE_TYPE_ST;
 	return pStFile;
 }
 
@@ -73,7 +78,7 @@ Uint8 *ST_ReadDisk(const char *pszFileName, long *pImageSize)
 /**
  * Save .ST file from memory buffer. Returns true is all OK.
  */
-bool ST_WriteDisk(const char *pszFileName, Uint8 *pBuffer, int ImageSize)
+bool ST_WriteDisk(int Drive, const char *pszFileName, Uint8 *pBuffer, int ImageSize)
 {
 #ifdef SAVE_TO_ST_IMAGES
 
