@@ -55,56 +55,6 @@ static const char * const pszTosNameExts[] =
 	NULL
 };
 
-static struct {
-	FILE *file;          /* file pointer to contents of INF file */
-	char prgname[16];    /* TOS name of the program to auto start */
-	const char *infname; /* name of the INF file TOS will try to match */
-	int match_count;     /* how many times INF was matched after boot */
-	int match_max;       /* how many times TOS needs it to be matched */
-} TosAutoStart;
-
-/* autostarted program name will be added after first '\' character */
-static const char emudesk_inf[] =
-"#E 9A 07\r\n"
-"#Z 01 C:\\@\r\n"
-"#W 00 00 02 06 26 0C 08 C:\\*.*@\r\n"
-"#W 00 00 02 08 26 0C 00 @\r\n"
-"#W 00 00 02 0A 26 0C 00 @\r\n"
-"#W 00 00 02 0D 26 0C 00 @\r\n"
-"#M 00 00 01 FF A DISK A@ @\r\n"
-"#M 01 00 01 FF B DISK B@ @\r\n"
-"#M 02 00 01 FF C DISK C@ @\r\n"
-"#F FF 28 @ *.*@\r\n"
-"#D FF 02 @ *.*@\r\n"
-"#G 08 FF *.APP@ @\r\n"
-"#G 08 FF *.PRG@ @\r\n"
-"#P 08 FF *.TTP@ @\r\n"
-"#F 08 FF *.TOS@ @\r\n"
-"#T 00 03 03 FF   TRASH@ @\r\n";
-
-static const char desktop_inf[] =
-"#a000000\r\n"
-"#b001000\r\n"
-"#c7770007000600070055200505552220770557075055507703111302\r\n"
-"#d\r\n"
-"#Z 01 C:\\@\r\n"
-"#E D8 11\r\n"
-"#W 00 00 10 01 17 17 13 C:\\*.*@\r\n"
-"#W 00 00 08 0B 1D 0D 00 @\r\n"
-"#W 00 00 0A 0F 1A 09 00 @\r\n"
-"#W 00 00 0E 01 1A 09 00 @\r\n"
-"#M 00 00 05 FF A DISK A@ @\r\n"
-"#M 00 01 05 FF B DISK B@ @\r\n"
-"#M 00 02 05 FF C DISK C@ @\r\n"
-"#T 00 03 02 FF   TRASH@ @\r\n"
-"#F FF 04   @ *.*@\r\n"
-"#D FF 01   @ *.*@\r\n"
-"#P 03 04   @ *.*@\r\n"
-"#G 03 FF   *.APP@ @\r\n"
-"#G 03 FF   *.PRG@ @\r\n"
-"#P 03 FF   *.TTP@ @\r\n"
-"#F 03 04   *.TOS@ @\r\n";
-
 /* Flags that define if a TOS patch should be applied */
 enum
 {
@@ -113,6 +63,7 @@ enum
 	TP_ANTI_STE,          /* Apply patch only if running on plain ST */
 	TP_ANTI_PMMU,         /* Apply patch only if no PMMU is available */
 	TP_FIX_060,           /* Apply patch only if CPU is 68060 */
+	TP_VDIRES,            /* Apply patch only if VDI is used */
 };
 
 /* This structure is used for patching the TOS ROMs */
@@ -135,6 +86,8 @@ static const char pszNoSteHw[] = "disable STE hardware access";
 static const char pszNoPmmu[] = "disable PMMU access";
 static const char pszFix060[] = "replace code for 68060";
 static const char pszFalconExtraRAM[] = "enable extra TT RAM on Falcon";
+static const char pszAtariLogo[] = "draw Atari Logo";
+static const char pszSTbook[] = "disable MCU access on ST-Book";
 
 //static Uint8 pRtsOpcode[] = { 0x4E, 0x75 };  /* 0x4E75 = RTS */
 static const Uint8 pNopOpcodes[] = { 0x4E, 0x71, 0x4E, 0x71, 0x4E, 0x71, 0x4E, 0x71,
@@ -146,6 +99,45 @@ static const Uint8 pRomCheckOpcode306[] = { 0x60, 0x00, 0x00, 0xB0 };  /* BRA $e
 static const Uint8 pRomCheckOpcode404[] = { 0x60, 0x00, 0x00, 0x94 };  /* BRA $e00746 */
 static const Uint8 pBraOpcode[] = { 0x60 };  /* 0x60XX = BRA */
 
+/*
+ * Routine for drawing the Atari logo.
+ * When this function is called, A0 contains a pointer to a 96x86x1 image.
+ * We cannot use the vdi yet (the screen workstation has not yet been opened),
+ * but we can take into account extended VDI modes.
+ */
+static const Uint8 pAtariLogo[] = {
+	0x3e, 0x3c, 0x00, 0x01,     /* move.w    #planes, d7; number will be patched below */
+	0x2c, 0x3c, 0, 0, 1, 64,    /* move.l    #linewidth, d6; number will be patched below */
+	0x22, 0x78, 0x04, 0x4e,     /* movea.l   (_v_bas_ad).w,a1 */
+	0xd3, 0xc6,                 /* adda.l    d6,a1 ; start drawing at 5th line */
+	0xd3, 0xc6,                 /* adda.l    d6,a1 */
+	0xd3, 0xc6,                 /* adda.l    d6,a1 */
+	0xd3, 0xc6,                 /* adda.l    d6,a1 */
+	0xd3, 0xc6,                 /* adda.l    d6,a1 */
+	0x30, 0x3c, 0x00, 0x55,     /* move.w    #$0055,d0 ; 86 lines of data */
+/* logocol1: */
+	0x72, 0x05,                 /* moveq.l   #5,d1 ; 6 words of data per line */
+	0x24, 0x49,                 /* movea.l   a1,a2 */
+/* logocol2: */
+	0x34, 0x18,                 /* move.w    (a0)+,d2 */
+	0x36, 0x07,                 /* move.w    d7,d3 */
+	0x53, 0x43,                 /* subq.w    #1,d3 */
+/* logocol3: */
+	0x34, 0xc2,                 /* move.w    d2,(a2)+ */
+	0x51, 0xcb, 0xff, 0xfc,     /* dbf       d3,logocol3 */
+	0x51, 0xc9, 0xff, 0xf2,     /* dbf       d1,logocol2 */
+	0xd3, 0xc6,                 /* adda.l    d6,a1 */
+	0x51, 0xc8, 0xff, 0xe8,     /* dbf       d0,logocol1 */
+	0x4e, 0x71,                 /* nops to end of original routine */
+	0x4e, 0x71,
+	0x4e, 0x71,
+	0x4e, 0x71,
+	0x4e, 0x71,
+	0x4e, 0x71,
+	0x4e, 0x71,
+	0x4e, 0x71
+};
+
 static const Uint8 p060movep1[] = {	/* replace MOVEP */
 	0x70, 0x0c,			/* moveq #12,d0 */
 	0x42, 0x30, 0x08, 0x00,		/* loop: clr.b 0,(d0,a0) */
@@ -235,12 +227,20 @@ static const TOS_PATCH TosPatches[] =
   /* as we've changed bytes in the ROM! So, just skip anyway! */
   { 0x206, -1, pszRomCheck, TP_ALWAYS, 0xE007FA, 0x2E3C0001, 4, pRomCheckOpcode206 },
   { 0x206, -1, pszDmaBoot, TP_HDIMAGE_OFF, 0xE00898, 0x610000E0, 4, pNopOpcodes }, /* BSR.W $E0097A */
+  { 0x206, -1, pszAtariLogo, TP_VDIRES, 0xE0076C, 0x1038044c, sizeof( pAtariLogo ), pAtariLogo },
+
+  { 0x208, -1, pszDmaBoot, TP_HDIMAGE_OFF, 0xE00806, 0x610000E8, 4, pNopOpcodes }, /* BSR.W $E008F0 */
+  { 0x208, -1, pszAtariLogo, TP_VDIRES, 0xE006B4, 0x1038044c, sizeof( pAtariLogo ), pAtariLogo },
+  { 0x208, -1, pszSTbook, TP_ALWAYS, 0xE00066, 0x303900d0, 18, pNopOpcodes },
+  { 0x208, -1, pszSTbook, TP_ALWAYS, 0xE000D6, 0x4a7900d0, 6, pNopOpcodes },
+  { 0x208, -1, pszSTbook, TP_ALWAYS, 0xE009FE, 0x303900d0, 14, pNopOpcodes },
 
   { 0x306, -1, pszRomCheck, TP_ALWAYS, 0xE007D4, 0x2E3C0001, 4, pRomCheckOpcode306 },
   { 0x306, -1, pszNoPmmu, TP_ANTI_PMMU, 0xE00068, 0xF0394000, 24, pNopOpcodes }, /* pmove : TC=0 TT0=0 TT1=0 -> disable MMU */
   { 0x306, -1, pszNoPmmu, TP_ANTI_PMMU, 0xE01702, 0xF0394C00, 32, pNopOpcodes }, /* pmove : CRP=80000002 00000700 TC=80f04445 TT0=017e8107 TT1=807e8507 -> */
   { 0x306, -1, pszFix060, TP_FIX_060, 0xe024dc, 0x01C80000, 12, p060movep1 },
   { 0x306, -1, pszFix060, TP_FIX_060, 0xe024fa, 0x01C80000, 12, p060movep1 },
+  { 0x306, -1, pszAtariLogo, TP_VDIRES, 0xE00754, 0x1038044c, sizeof( pAtariLogo ), pAtariLogo },
 
   { 0x400, -1, pszNoPmmu, TP_ANTI_PMMU, 0xE00064, 0xF0394000, 24, pNopOpcodes }, /* pmove : TC=0 TT0=0 TT1=0 -> disable MMU */
   { 0x400, -1, pszNoPmmu, TP_ANTI_PMMU, 0xE0148A, 0xF0394C00, 32, pNopOpcodes },
@@ -313,6 +313,7 @@ static void TOS_FixRom(void)
 	int nGoodPatches, nBadPatches;
 	short TosCountry;
 	const TOS_PATCH *pPatch;
+	Uint32 logopatch_addr = 0;
 
 	/* We can't patch RAM TOS images (yet) */
 	if (bRamTosImage && TosVersion != 0x0492)
@@ -348,6 +349,7 @@ static void TOS_FixRom(void)
 				        && ConfigureParams.System.bFastBoot)
 				    || (pPatch->Flags == TP_ANTI_STE && Config_IsMachineST())
 				    || (pPatch->Flags == TP_ANTI_PMMU && !use_mmu)
+				    || (pPatch->Flags == TP_VDIRES && bUseVDIRes)
 				    || (pPatch->Flags == TP_FIX_060 && ConfigureParams.System.nCpuLevel > 4)
 				   )
 				{
@@ -355,6 +357,8 @@ static void TOS_FixRom(void)
 					Log_Printf(LOG_DEBUG, "Applying TOS patch '%s'.\n", pPatch->pszName);
 					memcpy(&RomMem[pPatch->Address], pPatch->pNewData, pPatch->Size);
 					nGoodPatches += 1;
+					if (strcmp(pPatch->pszName, pszAtariLogo) == 0)
+						logopatch_addr = pPatch->Address;
 				}
 				else
 				{
@@ -371,141 +375,18 @@ static void TOS_FixRom(void)
 		pPatch += 1;
 	}
 
+	/* patch some values into the "Draw logo" patch */
+	if (logopatch_addr != 0)
+	{
+		STMemory_WriteWord(logopatch_addr + 2, VDIPlanes);
+		STMemory_WriteLong(logopatch_addr + 6, VDIWidth * VDIPlanes / 8);
+	}
+
 	Log_Printf(LOG_DEBUG, "Applied %i TOS patches, %i patches failed.\n",
 	           nGoodPatches, nBadPatches);
 }
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Set name of program that will be auto started after TOS boots.
- * Supported only from TOS 1.04 forward.
- */
-void TOS_AutoStart(const char *prgname)
-{
-	Str_Filename2TOSname(prgname, TosAutoStart.prgname);
-}
-
-/*-----------------------------------------------------------------------*/
-/**
- * Create a temporary desktop.inf file which will start autostart program.
- * This needs to be re-created on each boot in case user changed TOS version.
- */
-static void TOS_CreateAutoInf(void)
-{
-	const char *contents, *infname, *prgname;
-	int offset, size, max;
-	FILE *fp;
-
-	/* in case TOS didn't for some reason close it on previous boot */
-	TOS_AutoStartClose(TosAutoStart.file);
-
-	prgname = TosAutoStart.prgname;
-	/* autostart not enabled? */
-	if (!*prgname)
-		return;
-
-	/* autostart not supported? */
-	if (TosVersion < 0x0104)
-	{
-		Log_Printf(LOG_WARN, "Only TOS versions >= 1.04 support autostarting!\n");
-		return;
-	}
-
-	if (bIsEmuTOS)
-	{
-		infname = "C:\\EMUDESK.INF";
-		size = sizeof(emudesk_inf);
-		contents = emudesk_inf;
-		max = 1;
-	} else {
-		/* need to match file TOS searches first */
-		if (TosVersion >= 0x0200)
-			infname = "NEWDESK.INF";
-		else
-			infname = "DESKTOP.INF";
-		size = sizeof(desktop_inf);
-		contents = desktop_inf;
-		max = 1;
-	}
-	/* infname needs to be exactly the same string that given
-	 * TOS version gives for GEMDOS to find.
-	 */
-	TosAutoStart.infname = infname;
-	TosAutoStart.match_max = max;
-	TosAutoStart.match_count = 0;
-
-	/* find where to insert the program name */
-	for (offset = 0; offset < size; )
-	{
-		if (contents[offset++] == '\\')
-			break;
-	}
-	assert(offset < size);
-
-	/* create the autostart file */
-	fp = tmpfile();
-	if (!(fp
-	      && fwrite(contents, offset, 1, fp) == 1
-	      && fwrite(prgname, strlen(prgname), 1, fp) == 1
-	      && fwrite(contents+offset, size-offset-1, 1, fp) == 1
-	      && fseek(fp, 0, SEEK_SET) == 0))
-	{
-		if (fp)
-			fclose(fp);
-		Log_Printf(LOG_ERROR, "Failed to create autostart file for '%s'!\n", TosAutoStart.prgname);
-		return;
-	}
-	TosAutoStart.file = fp;
-	Log_Printf(LOG_WARN, "Virtual autostart file '%s' created for '%s'.\n", infname, prgname);
-}
-
-/*-----------------------------------------------------------------------*/
-/**
- * If given name matches autostart file, return its handle, NULL otherwise
- */
-FILE *TOS_AutoStartOpen(const char *filename)
-{
-	if (TosAutoStart.file && strcmp(filename, TosAutoStart.infname) == 0)
-	{
-		/* whether to "autostart" also exception debugging? */
-		if (ConfigureParams.Log.nExceptionDebugMask & EXCEPT_AUTOSTART)
-		{
-			ExceptionDebugMask = ConfigureParams.Log.nExceptionDebugMask & ~EXCEPT_AUTOSTART;
-			fprintf(stderr, "Exception debugging enabled (0x%x).\n", ExceptionDebugMask);
-		}
-		Log_Printf(LOG_WARN, "Autostart file '%s' for '%s' matched.\n", filename, TosAutoStart.prgname);
-		return TosAutoStart.file;
-	}
-	return NULL;
-}
-
-/*-----------------------------------------------------------------------*/
-/**
- * If given handle matches autostart file, close it and return true,
- * false otherwise.
- */
-bool TOS_AutoStartClose(FILE *fp)
-{
-	if (fp && fp == TosAutoStart.file)
-	{
-		if (++TosAutoStart.match_count >= TosAutoStart.match_max)
-		{
-			/* Remove autostart INF file after TOS has
-			 * read it enough times to do autostarting.
-			 * Otherwise user may try change desktop settings
-			 * and save them, but they would be lost.
-			 */
-			fclose(TosAutoStart.file);
-			TosAutoStart.file = NULL;
-			Log_Printf(LOG_WARN, "Autostart file removed.\n");
-		}
-		return true;
-	}
-	return false;
-}
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Assert that TOS version matches the machine type and change the system
@@ -524,7 +405,7 @@ static void TOS_CheckSysConfig(void)
 	FPUTYPE oldFpuType = ConfigureParams.System.n_FPUType;
 #endif
 
-	if (((TosVersion == 0x0106 || TosVersion == 0x0162) && !Config_IsMachineSTE())
+	if (((TosVersion == 0x0106 || TosVersion == 0x0162) && ConfigureParams.System.nMachineType != MACHINE_STE)
 	    || (TosVersion == 0x0162 && ConfigureParams.System.nCpuLevel != 0))
 	{
 		Log_AlertDlg(LOG_ERROR, "TOS versions 1.06 and 1.62 are for Atari STE only.\n"
@@ -534,7 +415,7 @@ static void TOS_CheckSysConfig(void)
 		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 		Video_SetTimings ( ConfigureParams.System.nMachineType , ConfigureParams.System.VideoTimingMode );
 		IoMem_Init();
-		ConfigureParams.System.nCpuFreq = 8;
+		Configuration_ChangeCpuFreq ( 8 );
 		ConfigureParams.System.nCpuLevel = 0;
 	}
 	else if ((TosVersion & 0x0f00) == 0x0300 && !Config_IsMachineTT())
@@ -546,7 +427,7 @@ static void TOS_CheckSysConfig(void)
 		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 		Video_SetTimings ( ConfigureParams.System.nMachineType , ConfigureParams.System.VideoTimingMode );
 		IoMem_Init();
-		ConfigureParams.System.nCpuFreq = 32;
+		Configuration_ChangeCpuFreq ( 32 );
 		ConfigureParams.System.nCpuLevel = 3;
 	}
 	else if ((TosVersion & 0x0f00) == 0x0400 && !Config_IsMachineFalcon())
@@ -562,7 +443,7 @@ static void TOS_CheckSysConfig(void)
 		DSP_Enable();
 #endif
 		IoMem_Init();
-		ConfigureParams.System.nCpuFreq = 16;
+		Configuration_ChangeCpuFreq ( 16 );
 		ConfigureParams.System.nCpuLevel = 3;
 	}
 	else if (TosVersion <= 0x0104 &&
@@ -576,11 +457,11 @@ static void TOS_CheckSysConfig(void)
 		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 		Video_SetTimings ( ConfigureParams.System.nMachineType , ConfigureParams.System.VideoTimingMode );
 		IoMem_Init();
-		ConfigureParams.System.nCpuFreq = 8;
+		Configuration_ChangeCpuFreq ( 8 );
 		ConfigureParams.System.nCpuLevel = 0;
 	}
-	else if ((TosVersion < 0x0300 && Config_IsMachineFalcon())
-	         || (TosVersion < 0x0200 && Config_IsMachineTT()))
+	else if (TosVersion < 0x0300 &&
+	         (Config_IsMachineTT() || Config_IsMachineFalcon()))
 	{
 		Log_AlertDlg(LOG_ERROR, "This TOS version does not work in TT/Falcon mode.\n"
 		             " ==> Switching to STE mode now.\n");
@@ -589,7 +470,7 @@ static void TOS_CheckSysConfig(void)
 		ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
 		Video_SetTimings ( ConfigureParams.System.nMachineType , ConfigureParams.System.VideoTimingMode );
 		IoMem_Init();
-		ConfigureParams.System.nCpuFreq = 8;
+		Configuration_ChangeCpuFreq ( 8 );
 		ConfigureParams.System.nCpuLevel = 0;
 	}
 	else if ((TosVersion & 0x0f00) == 0x0400 && ConfigureParams.System.nCpuLevel < 2)
@@ -661,6 +542,8 @@ static void TOS_CheckSysConfig(void)
  * Load TOS Rom image file into ST memory space and fix image so it can be
  * emulated correctly.  Pre TOS 1.06 are loaded at 0xFC0000 and later ones
  * at 0xE00000.
+ *
+ * Return zero if all OK, non-zero value for error.
  */
 int TOS_LoadImage(void)
 {
@@ -670,11 +553,7 @@ int TOS_LoadImage(void)
 	bTosImageLoaded = false;
 
 	/* Calculate end of RAM */
-	if (ConfigureParams.Memory.nMemorySize > 0
-	    && ConfigureParams.Memory.nMemorySize <= 14)
-		STRamEnd = ConfigureParams.Memory.nMemorySize * 0x100000;
-	else
-		STRamEnd = 0x80000;   /* 512 KiB */
+	STRamEnd = ConfigureParams.Memory.STRamSize_KB * 1024;
 
 	/* Load TOS image into memory so that we can check its version */
 	TosVersion = 0;
@@ -716,6 +595,8 @@ int TOS_LoadImage(void)
 	/* Now, look at start of image to find Version number and address */
 	TosVersion = SDL_SwapBE16(*(Uint16 *)&pTosFile[2]);
 	TosAddress = SDL_SwapBE32(*(Uint32 *)&pTosFile[8]);
+	if (TosVersion == 0x206 && SDL_SwapBE16(*(Uint16 *)&pTosFile[30]) == 0x186A)
+		TosVersion = 0x208;
 
 	/* Check for reasonable TOS version: */
 	if (TosVersion == 0x000 && TosSize == 16384)
@@ -750,7 +631,7 @@ int TOS_LoadImage(void)
 		ConfigureParams.System.bAddressSpace24 = true;
 		M68000_CheckCpuSettings();
 	}
-	else if (ConfigureParams.Memory.nTTRamSize)
+	else if (ConfigureParams.Memory.TTRamSize_KB)
 	{
 		switch (ConfigureParams.System.nMachineType)
 		{
@@ -779,7 +660,7 @@ int TOS_LoadImage(void)
 
 	/* (Re-)Initialize the memory banks: */
 	memory_uninit();
-	memory_init(STRamEnd, ConfigureParams.Memory.nTTRamSize*1024*1024, TosAddress);
+	memory_init(STRamEnd, ConfigureParams.Memory.TTRamSize_KB*1024, TosAddress);
 
 	/* Clear Upper memory (ROM and IO memory) */
 	memset(&RomMem[0xe00000], 0, 0x200000);
@@ -821,7 +702,6 @@ int TOS_LoadImage(void)
 	free(pTosFile);
 
 	bTosImageLoaded = true;
-	TOS_CreateAutoInf();
-
 	return 0;
 }
+
