@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # 
-# Copyright (C) 2012-2015 by Eero Tamminen <oak at helsinkinet fi>
+# Copyright (C) 2012-2017 by Eero Tamminen <oak at helsinkinet fi>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ def add_hconsole_paths():
         if os.path.isfile(f):
             f = os.path.dirname(f)
             sys.path.append(f)
-            print "Added local hconsole path: %s" % f
+            print("Added local hconsole path: %s" % f)
             break
     sys.path += ["/usr/local/share/hatari/hconsole",
                  "/usr/share/hatari/hconsole"]
@@ -95,7 +95,7 @@ class TOS:
     
     def _add_version(self):
         "get TOS version and whether it's EmuTOS & supports GEMDOS HD"
-        f = open(self.path)
+        f = open(self.path, 'rb')
         f.seek(0x2, 0)
         version = (ord(f.read(1)) << 8) + ord(f.read(1))
         # older TOS versions don't support autostarting
@@ -140,9 +140,9 @@ class TOS:
             raise AssertionError("'%s' TOS version 0x%x isn't valid" % (name, version))
         
         if self.etos:
-            print "%s is EmuTOS v%x %dkB" % (name, version, size)
+            print("%s is EmuTOS v%x %dkB" % (name, version, size))
         else:
-            print "%s is normal TOS v%x" % (name, version)
+            print("%s is normal TOS v%x" % (name, version))
         # 0: whether / how long to wait to dismiss memory test
         # 1: how long to wait until concluding test failed
         # 2: list of machines supported by this TOS version
@@ -223,7 +223,8 @@ class Config:
             self.usage(error)
         self.handle_options(opts)
         self.images = self.check_images(paths)
-        print "Test configuration:\n\t", self.disks, self.graphics, self.machines, self.memsizes, self.ttrams
+        configs = (self.disks, self.graphics, self.machines, self.memsizes, self.ttrams, self.bools)
+        print("Test configuration:\n\t%s %s %s RAM=%s TTRAM=%s bool=%s\n" % configs)
 
     
     def check_images(self, paths):
@@ -278,13 +279,14 @@ class Config:
     def usage(self, msg=None):
         "output program usage information"
         name = os.path.basename(sys.argv[0])
-        print __doc__
+        print(__doc__)
         print("""
 Usage: %s [options] <TOS image files>
 
 Options:
 \t-h, --help\tthis help
-\t-f, --fast\tdo tests with "--fastfdc yes --fast-forward yes"
+\t-f, --fast\tspeed up boot with less accurate emulation:
+\t\t\t"--fast-forward yes --fast-boot yes --fastfdc yes --timer-d yes"
 \t-d, --disks\t%s
 \t-g, --graphics\t%s
 \t-m, --machines\t%s
@@ -412,8 +414,10 @@ class Tester:
     fifofile  = output + "midi-out"
     bootauto  = "bootauto.st.gz" # TOS old not to support GEMDOS HD either
     bootdesk  = "bootdesk.st.gz"
+    floppyprg = "A:\MINIMAL.PRG"
     hdimage   = "hd.img"
     ideimage  = "hd.img"	 # for now use the same image as for ACSI
+    hdprg     = "C:\MINIMAL.PRG"
     results   = None
     
     def __init__(self):
@@ -429,9 +433,9 @@ class Tester:
     def alarm_handler(self, signum, dummy):
         "output error if (timer) signal came before passing current test stage"
         if signum == signal.SIGALRM:
-            print "ERROR: timeout triggered -> test FAILED"
+            print("ERROR: timeout triggered -> test FAILED")
         else:
-            print "ERROR: unknown signal %d received" % signum
+            print("ERROR: unknown signal %d received" % signum)
             raise AssertionError
     
     def create_config(self):
@@ -496,27 +500,27 @@ class Tester:
         # check file truncate
         error = verify_file_empty(self.textoutput)
         if error:
-            print "ERROR: file wasn't truncated:\n\t%s" % error
+            print("ERROR: file wasn't truncated:\n\t%s" % error)
             os.rename(self.textoutput, "%s.%s" % (self.textoutput, identity))
             ok = False
         # check serial output
         error = verify_file_match(self.textinput, self.serialout)
         if error:
-            print "ERROR: serial output doesn't match input:\n\t%s" % error
+            print("ERROR: serial output doesn't match input:\n\t%s" % error)
             os.rename(self.serialout, "%s.%s" % (self.serialout, identity))
             ok = False
         # check printer output
         error = verify_file_match(self.textinput, self.printout)
         if error:
             if tos.etos or tos.version > 0x206 or (tos.version == 0x100 and memory > 1):
-                print "ERROR: printer output doesn't match input (EmuTOS, TOS v1.00 or >v2.06)\n\t%s" % error
+                print("ERROR: printer output doesn't match input (EmuTOS, TOS v1.00 or >v2.06)\n\t%s" % error)
                 os.rename(self.printout, "%s.%s" % (self.printout, identity))
                 ok = False
             else:
                 if os.path.exists(self.printout):
                     error = verify_file_empty(self.printout)
                     if error:
-                        print "WARNING: unexpected printer output (TOS v1.02 - TOS v2.06):\n\t%s" % error
+                        print("WARNING: unexpected printer output (TOS v1.02 - TOS v2.06):\n\t%s" % error)
                         os.rename(self.printout, "%s.%s" % (self.printout, identity))
         self.cleanup_test_files()
         return ok
@@ -527,17 +531,17 @@ class Tester:
         print("Waiting %ss for fifo '%s' input..." % (timeout, self.fifofile))
         sets = select.select([fifo], [], [], timeout)
         if sets[0]:
-            print "...test program is READY, read what's in its fifo:",
+            print("...test program is READY, read what's in its fifo...")
             try:
                 # read can block, make sure it's eventually interrupted
                 signal.alarm(timeout)
                 line = fifo.readline().strip()
                 signal.alarm(0)
-                print line
+                print(line)
                 return (True, (line == "success"))
             except IOError:
                 pass
-        print "ERROR: TIMEOUT without fifo input, BOOT FAILED"
+        print("ERROR: TIMEOUT without fifo input, BOOT FAILED")
         return (False, False)
     
     
@@ -552,7 +556,7 @@ class Tester:
             signal.alarm(0)
             return fifo
         except IOError:
-            print "ERROR: fifo open IOError!"
+            print("ERROR: fifo open IOError!")
             return None
 
     def test(self, identity, testargs, tos, memory):
@@ -561,7 +565,7 @@ class Tester:
         instance = hconsole.Main(self.defaults + testargs, False)
         fifo = self.open_fifo(tos.fullwait)
         if not fifo:
-            print "ERROR: failed to get fifo to Hatari!"
+            print("ERROR: failed to get fifo to Hatari!")
             self.get_screenshot(instance, identity)
             instance.run("kill")
             return (False, False, False, False)
@@ -578,7 +582,7 @@ class Tester:
         if tests_ok:
             output_ok = self.verify_output(identity, tos, memory)
         else:
-            print "TODO: collect info on failure, regs etc"
+            print("TODO: collect info on failure, regs etc")
             output_ok = False
         
         # get screenshot after a small wait (to guarantee all
@@ -617,21 +621,30 @@ class Tester:
             testargs += ["--monitor", monitor]
         
         if config.fast:
-            testargs += ["--fastfdc", "yes", "--fast-forward", "yes"]
+            testargs += ["--fast-forward", "yes", "--fast-boot", "yes",
+                         "--fastfdc", "yes", "--timer-d", "yes"]
         
         if disk == "gemdos":
             # use Hatari autostart, must be last thing added to testargs!
             testargs += [self.testprg]
+        # HD supporting TOSes support also INF file autostart, so
+        # with them test program can be run with the supplied INF
+        # file.
+        #
+        # However, in case of VDI resolutions the VDI resolution
+        # setting requires overriding the INF file...
+        #
+        # -> always specify directly which program to autostart
+        #    with --auto
         elif disk == "floppy":
             if tos.supports_gemdos_hd():
-                # GEMDOS HD supporting TOSes support also INF file autostart
-                testargs += ["--disk-a", self.bootdesk]
+                testargs += ["--disk-a", self.bootdesk, "--auto", self.floppyprg]
             else:
                 testargs += ["--disk-a", self.bootauto]
         elif disk == "acsi":
-            testargs += ["--acsi", self.hdimage]
+            testargs += ["--acsi", self.hdimage, "--auto", self.hdprg]
         elif disk == "ide":
-            testargs += ["--ide-master", self.ideimage]
+            testargs += ["--ide-master", self.ideimage, "--auto", self.hdprg]
         else:
             raise AssertionError("unknown disk type '%s'" % disk)
 
@@ -643,9 +656,7 @@ class Tester:
         self.results = {}
         for tos in config.images:
             self.results[tos.name] = []
-            print
-            print "***** TESTING: %s *****" % tos.name
-            print
+            print("\n***** TESTING: %s *****\n" % tos.name)
             count = 0
             for machine in config.machines:
                 if machine not in tos.machines:
@@ -678,7 +689,7 @@ class Tester:
         "summarize test results"
         cases = [0, 0, 0, 0]
         passed = [0, 0, 0, 0]
-        tosnames = self.results.keys()
+        tosnames = list(self.results.keys())
         tosnames.sort()
         
         report = open(self.report, "w")
@@ -714,17 +725,17 @@ class Tester:
         report.write("\n")
         
         # print report out too
-        print "--- %s ---" % self.report
+        print("--- %s ---" % self.report)
         report = open(self.report, "r")
         for line in report.readlines():
-            print line.strip()
+            print(line.strip())
 
 
 # -----------------------------------------------
 def main():
     "tester main function"
     info = "Hatari TOS bootup tester"
-    print "\n%s\n%s\n" % (info, "-"*len(info))
+    print("\n%s\n%s\n" % (info, "-"*len(info)))
     config = Config(sys.argv)
     tester = Tester()
     tester.run(config)
