@@ -15,6 +15,10 @@ args=$*
 # open fifos
 for i in $(seq $count); do
 	mkfifo midi$i
+	if [ $? -ne 0 ]; then
+		echo "ERROR: creating FIFO 'midi$i' for MIDI communication failed!"
+		exit 1
+	fi
 done
 
 # show full path
